
/*                              API.C                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "api.h"
#include "constant.h"
#include "database.h"
#include "initial_pdu.h"
#include "msg.h"
#include "present_pdu.h"
#include "record.h"
#include "recinfo.h"
#include "scan_pdu.h"
#include "scaninfo.h"
#include "search_pdu.h"
#include "session.h"
#include "strings.h"

#ifndef DEFINE_LIMITS
#include <limits.h>
#else
#define INT_MAX		2147483547	/* Maximum "int" decimal value */
#endif

#define MAX_BUFFER          256         /* Maximum character buffer size */

static int set_preferred_syntax(char *syntax_name)
{
    int syntax;
    
    if (syntax_name == NULL || syntax_name[0] == EOS)
	syntax = NORMAL;                       /* Let it default */
    else 
        {
             syntax = record_syntax_from_name(syntax_name);

             if (syntax == UNKNOWN)
	         syntax = msg_make(BAD_PREFERRED_SYNTAX, syntax_name);
        }
    return syntax;
}
 
/*--------------------------------------------------------------*
 |                    Z39_SAVE_TO_LOG                           |
 | Function:                                                    |
 |    Save the last network IO packet.                          |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    filename   : Name of file to save the log.                |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    This is useful for logging the last packet sent across.   |
 *--------------------------------------------------------------*/

int z39_save_to_log(SESSION *session,
                    char *filename)
{
    FILE *zsave;
    int  code = 0;

    zsave = fopen(filename,"wb");

    if (zsave)
        {
            int length;

            if (session->io && session->io->length > 0)
                {
                    length = fwrite(session->io->buffer,
                                    1,
                                    session->io->length,
                                    zsave);

                    msg_make(FORMAT_MSG,
                             "saved %d bytes of %d read",
                             length, session->io->length);

                }
            else
                code = msg_make(DEFAULT_MSG,"No I/O buffer found to write.");
        }
    else
        code = msg_make(FORMAT_MSG,"Unable to open %s file.",filename);

    fclose(zsave);

    return code;
}

/*--------------------------------------------------------------*
 |                    Z39_DEBUG                                 |
 | Function:                                                    |
 |    Turn the debugging flag on and off.                       |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    switch     : The on (!= 0) and off (0) switch.            |
 | Returns:                                                     |
 |    0 for success.                                            |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int z39_debug(SESSION *session,
              int toggle)
{
    extern int debug;

    debug = (toggle == 0) ? 0 : 1;

    return 0;
}

/*--------------------------------------------------------------*
 |                    Z39_SHOW                                  |
 | Function:                                                    |
 |    Show information about the session.                       |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Type of information to show.                 |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |    Show either index information or the databases which can  |
 |    be selected.                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int z39_show(SESSION *session,
             char *parameters)
{
    int  code;

    /* Parse the input of the command */

    if (strncmp(parameters, "INDEXES", strlen(parameters)) == 0)
        {
            if (session == NULL)
                {
                   msg_make(DEFAULT_MSG,"No connection to a database.");
                   return NOTOK;
                }

            code = database_show_index(session->db);
        }
    else if (strncmp(parameters, "SELECT", strlen(parameters)) == 0)
        code = session_servers();
    else
        code = msg_make(DEFAULT_MSG, "Invalid SHOW parameter: %s", parameters);

    return code;
}

/*--------------------------------------------------------------*
 |                    Z39_INDEX                                 |
 | Function:                                                    |
 |    Create or modify an index used for searching.             |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    name       : Name of index to add/modify.                 |
 |    attributes : Attributes of index (as numeric pair)        |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |    Just add an index to a database session.                  |
 | Note:                                                        |
 |    The attributes are a pair of numbers, the attribute type  |
 |    (1-6) and the attribute value corresponding to the type.  |
 |    See Appendix C of the Z39.50 standard.                    |
 *--------------------------------------------------------------*/

int z39_index(SESSION *session,
              char *name,
              char *attributes)
{
    int  code;

    /* Parse the input of the command */

    if (session == NULL)
        code = msg_make(DEFAULT_MSG, "No session has been established.");
    else
        code = database_new_index(session->db, name, attributes);

    return code;
}

/*--------------------------------------------------------------*
 |                    Z39_SELECT                                |
 | Function:                                                    |
 |    Select a database and server for searching.               |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    name       : Name of the database to use.                 |
 |    host       : Name (Internet number) of host server.       |
 |    port       : Number of port to connect with.              |
 |    globalset  : The name of sets during session.             |
 |    authorize  : The id authentication string.                |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |    This sends the initial PDU to the Z39.50 server.  If the  |
 |    pointers of host, port and globalset are not NULL they    |
 |    will be set to the values specified by the connection.    |
 | Note:                                                        |
 |    As a short-cut the file SERVERZ.H contains a list of      |
 |    servers and databases which are assigned a name.  By just |
 |    using the name you can connect to the server and the      |
 |    associated database.                                      |
 |                                                              |
 *--------------------------------------------------------------*/

int z39_initial(SESSION **p_session,
                char *name,
                char *host,
                char *port,
                char *globalset,
                char *syntax_preferred,
                char *authorize)   
{
    SESSION *session;
    int  code;

    if (name[0] == EOS)
        return msg_make(DEFAULT_MSG,"Select parameter required.\n");

    session = session_new(name, host, port, &authorize);
    if (session == NULL)
        code = NOTOK;
    else if ((session->preferred_syntax = set_preferred_syntax(syntax_preferred)) == NOTOK)
	{
	    session_dispose(&session);   
	    code = NOTOK;
	}
    else
        {
            STRING information;
            STRING authenticate;
            int  status;

            /* Initialize information */

            information.length = 0;        /* No Information */

            /* Check if authorize id is needed */

            if (authorize && authorize[0] != EOS)
                {
                    authenticate.location = authorize;
                    authenticate.length   = strlen(authorize);
                }
            else
                authenticate.length = 0;

            code = send_initial_pdu(session, 
                                    &authenticate, 
                                    &information, 
                                    &status);
#ifndef NONET
            if (status == 0)
                code = msg_make(SERVER_REJECT, session->host);
#endif

            if (code == NOTOK)
                session_dispose(&session);
            else
                *p_session = session;
        }
    return code;
}

/*--------------------------------------------------------------*
 |                    Z39_SEARCH                                |
 | Function:                                                    |
 |    Send a query to the Z39.50 server.                        |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    query      : Character string of search command.          |
 |    database   : Name of database to search.                  |
 |    setname    : Name of set to store the search.             |
 |    info       : Record information returned from search.     |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |    This routine passes on the search request to the Z39.50   |
 |    server.  If a database name is specified it will replace  |
 |    the current database to search.  The setname specifies    |
 |    result set name to store the result.                      |
 | Note:                                                        |
 |    The query string is the index name followed by the term   |
 |    to search for in the index.  Boolean operatores OR, AND,  |
 |    AND-NOT are allowed.                                      |
 *--------------------------------------------------------------*/

int z39_search(SESSION *session,
               char *query,
               char *database,
               char *setname,
               char *syntax_preferred,
               RECORD_INFO *info)
{
    static int set_number = 0;
    int  syntax;
    int  code;

    if (session == NULL)
        return msg_make(DEFAULT_MSG, "No session has been established.");

    if (info == NULL)
        return msg_make(DEFAULT_MSG, "The INFO structure is required.");

    /* Check if a different database name should be used */

    if (database != NULL && database[0] != EOS)
        strcpy(database_name(session->db), database);

    if (setname != NULL && setname[0] != EOS)
        strcpy(session->current_set, setname);
    else
        {   /* Create the appropriate set name */

            if (session->set_name[0] == EOS)
                sprintf(session->current_set, "default");
            else
                sprintf(session->current_set, "%s%d",
                        session->set_name, ++set_number);
        }

    if (syntax_preferred != NULL && syntax_preferred[0] != EOS)
        syntax = set_preferred_syntax(syntax_preferred);
    else
        syntax = session->preferred_syntax;    

    if (syntax == NOTOK)
	code = NOTOK;
    else
	{   
	    /* Set the limit sizes for returning messages */

	    session->set_replace = -1;
	    session->set_small_limit = SMALL_SET_LIMIT;
	    session->set_large_limit = LARGE_SET_LIMIT;
	    session->set_medium_limit = MEDIUM_SET_NUMBER;

	    code = send_search_pdu(session, query, syntax, info);
	}
    return code;
}

/*--------------------------------------------------------------*
 |                    Z39_PRESENT                               |
 | Function:                                                    |
 |    Retrieve & present records from the Z39.50 server.        |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    setname    : Name of set to store the search.             |
 |    elemname   : Element Set Name                             |  
 |    position   : Position in set to start retrieving records. |
 |    num_recs   : Number of records to return.                 |
 |    info       : Record information returned from present.    |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |                                                              |
 |                                                              |
 | Note:                                                        |
 |    The number of hits in the set will be incorrect when you  |
 |    switch sets by specifying the setname.  It uses the hits  |
 |    from the last search request.                             |
 *--------------------------------------------------------------*/

int z39_present(SESSION *session,
                char *setname,
		char *elemname, 
		char *syntax_preferred,
                unsigned int position,
                unsigned int num_recs,
                RECORD_INFO *info)
{
    unsigned int display;
    unsigned int syntax;
    int  code;

    if (session == NULL)
        return msg_make(DEFAULT_MSG, "No session has been established.");

    if (info == NULL)
        return msg_make(DEFAULT_MSG, "The INFO structure is required.");

    if (setname != NULL && setname[0] != EOS)
        strcpy(session->current_set, setname);

    if (elemname == NULL || elemname[0] == EOS)
        strcpy(elemname, session->elemset_name);
    
    if (position)
        session->next_record = position;
    else
        position = session->next_record;

    if (syntax_preferred != NULL && syntax_preferred[0] != EOS)
	syntax = set_preferred_syntax(syntax_preferred);
    else
        syntax = session->preferred_syntax;

    if (syntax == NOTOK)
	code = NOTOK;
    else
	{
	    display = (num_recs) ? num_recs : DISPLAY_NUMBER;

	    code = send_present_pdu(session, 
				    elemname, 
				    syntax, 
				    position, 
				    display, 
				    info);
	}
    return code;
}

/*--------------------------------------------------------------*
 |                    CLOSE_COMMAND                             |
 | Function:                                                    |
 |    Close the connection to the Z39.50 server.                |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    The parameter right now is ignored.                       |
 *--------------------------------------------------------------*/

int z39_close(SESSION **p_session,
              char *parameters)
{
    int code;

    /* Ignore the parameters */

    code = session_dispose(p_session);
    return code;
}
/* Z39_SCAN */
int z39_scan(SESSION *session,
           char *indexname,
           char *termname,
           int num_terms,
           int position,
           int stepsize,
           SCAN_INFO *scan)
{
    int  code;
    if (session == NULL)
        return msg_make(DEFAULT_MSG, "No session has been established.");
    if (scan == NULL)
        return msg_make(DEFAULT_MSG, "The SCANINFO structure is required.");
    if (num_terms <= 0)
        return msg_make(DEFAULT_MSG, "At least 1 term must be requested.");
    if (position > num_terms)
        return msg_make(DEFAULT_MSG, "Requested position is invalid.");
    if (stepsize < 0)
        return msg_make(DEFAULT_MSG, "Stepsize cannot be negative.");
    code = send_scan_pdu(session,
                       indexname,
                       termname,
                       position,
                       stepsize,
                       num_terms,
                       scan);

    return code;
}
