/*                         ATTRINFO.C                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

/*
 *  This module allows you handle attributes information.   The attributes
 *  are expressed as a module or a value.
 */

#include <stdlib.h>
#include <strings.h>

#include "attrcons.h"
#include "attrinfo.h"
#include "bib1attr.h"
#include "constant.h"
#include "msg.h"
#include "utility.h"

/*
 * TABLES of ATTRIBUTES
 */

int default_index_attributes(ATTRIBUTE_INFO *info, 
			     int attributes[])
{
    static int numeric_index[][2] =
        {
	    {7, 20},
	    {48, 53},
	    {1027, 1030}
        };
    const int numeric_index_size = sizeof(numeric_index)/(sizeof(int) * 2);
    int i;
    
    attributes[USE] = info->value;

    /* 
     * The simple method would be to use a case statement and
     * rely on the compiler do to a good job when testing.
     * But some compilers are not very good at optimizing 
     * case statements so I will use this approach for now.
     *
     * If it gets more complicated it might be best to move
     * to a case statement.
     */

    for (i = 0; i < numeric_index_size; i++)
	{
	    if (    info->value >= numeric_index[i][0]
		&&  info->value <= numeric_index[i][1])
		break;
	}

    if (i < numeric_index_size)
	{
	    /* A numeric index */

	    attributes[POSITION]  = 2;     /* First in Subfield */
	    attributes[STRUCTURE] = 2;     /* Word */
	}
    else
	{
	    /* Treat as an ordinary index */

	    attributes[POSITION]  = 3;     /* Any Position in Field */ 
	    attributes[STRUCTURE] = 6;     /* Word List */
	}
    return OK;
}

static ATTRIBUTE_INFO *search_info(ATTRIBUTE_TYPE *type,
				   char *text,
				   int  length)
{
    ATTRIBUTE_INFO *info = type->table;
    unsigned i;

    for (i = 0; i < type->table_size; i++, info++)
	if (memcmp(info->name, text, length) == 0)
	    return info;
    
    return NULL;
}

static ATTRIBUTE_TYPE *search_type(char *text,
				   int  length)
{
    ATTRIBUTE_TYPE *type = Bib1_Attribute_Types;
    unsigned i;

    for (i = 0; i < DIM(Bib1_Attribute_Types); i++, type++)
	if (memcmp(type->name, text, length) == 0)
	    return type;

    return NULL;
}

static int attribute_info(int attributes[],
		          char *attribute_start,
		          char *attribute_seperator,
		          char *attribute_end)
{
    ATTRIBUTE_TYPE *type;
    ATTRIBUTE_INFO *info;
    int result = 0;

    type = search_type(attribute_start,
		       attribute_seperator - attribute_start);

    if (type == NULL)
	result =  msg_make(BAD_ATTRIB_INFO,"unable to find a matching attribute type");
    else
	{
	    info = search_info(type,
			       attribute_seperator + 1, 
			       attribute_end - attribute_seperator - 1);

	    if (info == NULL)
		{
                    char *end;
		    int numeric_value;
		    
		    numeric_value = (int) strtol(attribute_seperator + 1, 
						 &end, 
						 10);
		    
		    if (attribute_end == end)
			attributes[type->index] = numeric_value;
		    else
			result = msg_make(BAD_ATTRIB_INFO,"unable to find matching attribute value");
		}
	    else
		attributes[type->index] = info->value;
	}
    return result;
}

int attribute_info_parse(int attributes[],
                         char *text,
                         char *assignment,
                         char *seperator)
{
    char *attribute_type = NULL;
    char *attribute_value = NULL;
    char *attribute_end;
    int  result = 0;

    while (*text != EOS && result == 0)
        {
            text += strspn(text, seperator);

            attribute_type = text;

            text = (char *) strchr(text, *assignment);

            if (attribute_type == text || *attribute_type == EOS)
                return msg_make(BAD_ATTRIB_INFO,
                                "missing the attribute type");

            attribute_value = text++;

            if (attribute_value == NULL || *text == EOS || *text == *seperator)
                return msg_make(BAD_ATTRIB_INFO,
                                "attribute value assignment incorrect");

            attribute_end = (char *) strchr(text, *seperator);

            if (attribute_end == NULL)
                text += strlen(text);
            else
                text = attribute_end;

            result = attribute_info(attributes,
                                    attribute_type,
                                    attribute_value,
                                    text);
        }
    return result;
}

int attribute_index_parse(int attributes[],
			 char *text,
			 char *assignment,
			 char *seperator)
{
    ATTRIBUTE_INFO *info;
    char *index_name;
    int size;
    int i;
    int result = NOTOK;
   
    for (i = USE; i < MAX_ATTRIBUTES; i++)
	attributes[i] = 0;

    /*  If there is no attribute seperator in the name string, or
     *  if it occurs past the end of the name,
     *  or if the first char is the attribute seperator, treat
     *  the name as an atomic index name.
     */

    text += strspn(text, seperator);
    
    size = tokenize(&text, &index_name, seperator);
   
    if (size != 0) 
        info = search_info(&Bib1_Attribute_Types[0], index_name, size);
        
    if (info != NULL)
	result = default_index_attributes(info, attributes);
	    
    if (*text == *seperator)
        {
            result = attribute_info_parse(attributes,
                                          text,
                                          assignment,
                                          seperator);
        }
    return result;
}
