/*                           ATTRLIST.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include "constant.h"
#include "attrcons.h"
#include "attrlist.h"
#include "database.h"
#include "pdutags.h"
#include "putget.h"

/*--------------------------------------------------------------*
 |                    ATTRIBUTELIST                             |
 | Function:                                                    |
 |    Create a linked list of attributes.                       |
 | Parameters:                                                  |
 |    pattelem    : The                                         |
 |    attributes  : Array of attributes to change.              |
 |    operator    : The value for the relation attribute.       |
 | Returns:                                                     |
 |    Pointer to attribute list or null pointer.                |
 | Description:                                                 |
 |                                                              |
 |    Allocate and fill in the following ASN.1 description      |
 |                                                              |
 |    AttributeList ::= [44]                                    |
 |        IMPLICIT SEQUENCE OF AttributeElement                 |
 |    AttributeElement  ::=                                     |
 |        SEQUENCE {                                            |
 |             AttributeType,                                   |
 |             AttributeValue                                   |
 |        }                                                     |
 |    AttributeType  ::=  [120] IMPLICIT INTEGER                |
 |    AttributeValue ::=  [121] IMPLICIT INTEGER                |
 |                                                              |
 | Note:                                                        |
 |    Creates a linked list from the array of attributes where  |
 |    the value of the attribute is nonzero.  The exception is  |
 |    the relation attribute which is set by the query.         |
 *--------------------------------------------------------------*/

static int attribute_element(DATA_DIR *PAttrElem,
			     int type,
			     int value)
{
    DATA_DIR *sequence;

    if (value != 0)
        {
	    sequence = put_node(PAttrElem, SEQUENCE, ASN1_UNIVERSAL);
	    if (!sequence) return NOTOK;

	    put_int(sequence, AttributeType, type);
	    put_int(sequence, AttributeValue, value);
	}    
    return OK;
}

/*--------------------------------------------------------------*
 |                    ATTRIBUTELIST                             |
 | Function:                                                    |
 |    Create a linked list of attributes.                       |
 | Parameters:                                                  |
 |    attributes  : Array of attributes to change.              |
 |    operator    : The value for the relation attribute.       |
 | Returns:                                                     |
 |    Pointer to attribute list or null pointer.                |
 | Description:                                                 |
 |                                                              |
 |    Allocate and fill in the following ASN.1 description      |
 |                                                              |
 |    AttributeList ::= [44]                                    |
 |        IMPLICIT SEQUENCE OF AttributeElement                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int attributelist(DATA_DIR *PAttrTerm,
                  int attributes[])
{
    DATA_DIR *PList;
    int type;

    PList = put_node(PAttrTerm, AttributeList, ASN1_CONTEXT);
    if (!PList) return NOTOK;

    for (type = 1; type < MAX_ATTRIBUTES; type++)
	{
	    if (attribute_element(PList, type, attributes[type]))
		return NOTOK;
	}
	    
    return OK;
}

/*--------------------------------------------------------------*
 |                    PARSE_ATTRIBUTES                          |
 | Function:                                                    |
 |    Create an attribute plus term type operand.               |
 | Parameters:                                                  |
 |    index     : Index reference being parsed                  |
 |    operator  : Relation operator from query.                 |
 |    term      : Search term.                                  |
 |    length    : Length of search term.                        |
 |    words     : Number of words in term.                      |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 |    Allocate and fill in the following ASN.1 description      |
 |                                                              |
 |    AttributesPlusTerm::=  [102]                              |
 |        IMPLICIT SEQUENCE {                                   |
 |            AttributeList,                                    |
 |            Term                                              |
 |        }                                                     |
 |                                                              |
 |    Return as a proper operand.                               |
 |                                                              |
 | Note:                                                        |
 |    Right now words is not used.  But in the future it could  |
 |    be used to create a search with implicit ANDs so          |
 |           find tw computer research                          |
 |    would become internally.                                  |
 |           find tw computer and tw research                   |
 |    This would be useful for indexes that do not support      |
 |    word lists.                                               |
 *--------------------------------------------------------------*/

int attributelist_and_term(DATA_DIR *POperand,
                           int  attributes[MAX_ATTRIBUTES],
                           STRING *term,
                           int  words)
{
    DATA_DIR *PAttrTerm;
    int code;

    PAttrTerm = put_node(POperand, AttributesPlusTerm, ASN1_CONTEXT);

    code = attributelist(PAttrTerm, attributes);

    if (code == OK) put_string(PAttrTerm, Term, term);

    return code;
}
