/*                           BIB1ATTR.H                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#ifndef BIB1ATTR_H
#define BIB1ATTR_H

#include <stddef.h>

#ifndef ATTRINFO_H
#include "attrinfo.h"
#endif

#ifndef CONSTANT_H
#include "constant.h"
#endif

/*
 * ATR.1 Attribute Set bib-1
 *
 *         In the Z39.50 ASN.1 definitions, when the value of AttributeSetId},
 * accompanying AttributeList, equals the object identifier for this attribute-
 * set, then: AttributeType (within AttributeElement within AttributeList) takes
 * values from the attribute types listed below, and AttributeValue takes values
 * from the corresponding attribute tables.
 *         A bib-1 attribute list may include zero or more attributes for each
 * attribute type. Some attribute combinations (in particular, when no attribute
 * or more than one attribute are included for a given type) may result in
 * unpredictable behavior by a given server.
 *
 * There are 6 attribute types which can be used in constructing
 *
 *    Use               - The use attribute can be considered the "index name".
 *    Relation          - The logical relation of the term to the use attribute.
 *    Position          - Position in a field/subfield (mainly for MARC records).
 *    Structure         - Further characteristics of the search term.
 *    Truncation        - Use truncation when searching the index.
 *    Completeness      - The field is complete (mainly for MARC records).
 */


/*
 *  A Use attribute specifies the type of entity being sought (e.g., corporate
 *  name, personal name, title, subject).  The Use attribute is designated in the
 *  list of attributes with an Attribute-type value of "1".  The Use attributes in
 *  Z39.50 Attribute Set bib-1 with Use values of "1"-"63" were taken from the
 *  Search and Retrieve Protocol Specification (ISO 10163) Attribute Set bib-1.
 *  This list of Use attributes has been expanded in Z39.50 (Version 2) to include
 *  some additional attributes.  Use attributes that are specific to Z39.50 have
 *  Use values equal to or greater than "1000".
 */

static ATTRIBUTE_INFO Use_Attributes[] =
{
    {"ABSTRACT",                         62},
    {"ANY",                              1016},
    {"ANYWHERE",                         1035},
    {"AUTHOR",                           1003},
    {"AUTHORITY_FORMAT_IDENTIFIER",      1013},
    {"AUTHOR_NAME_CONFERENCE",           1006},
    {"AUTHOR_NAME_CORPORATE",            1005},
    {"AUTHOR_NAME_PERSONAL",             1004},
    {"AUTHOR_TITLE",                     1000},
    {"BDI_INDEX_SUBJECT",                23},
    {"BGF_NUMBER",                       11},
    {"BIB_LEVEL",                        1021},
    {"BLISS_CLASSIFICATION",             15},
    {"BNB_CARD_NO",                      10},
    {"BODY_OF_TEXT",                     1010},
    {"CODEN",                            60},
    {"CODE_GEOGRAPHIC_AREA",             55},
    {"CODE_INSTITUTION",                 56},
    {"CODE_LANGUAGE",                    54},
    {"CONCEPT_REFERENCE",                1015},
    {"CONCEPT_TEXT",                     1014},
    {"CONFERENCE_NAME",                  3},
    {"CONTENT_TYPE",                     1034},
    {"CORPORATE_NAME",                   2},
    {"DATE",                             30},
    {"DATE_OF_ACQUISITION",              32},
    {"DATE_OF_PUBLICATION",              31},
    {"DATE_TIME_ADDED_TO_DATABASE",      1011},
    {"DATE_TIME_LAST_MODIFIED",          1012},
    {"DEWEY_CLASSIFICATION",             13},
    {"DOC_ID",                           1032},
    {"EDITOR",                           1020},
    {"GEOGRAPHIC_CLASS",                 1022},
    {"HOST_ITEM",                        1033},
    {"IDENTIFIER_STANDARD",              1007},
    {"INDEXED_BY",                       1023},
    {"INSPEC_SUBJECT",                   24},
    {"ISBN",                             7},
    {"ISSN",                             8},
    {"LC_CALL_NUMBER",                   16},
    {"LC_CARD_NUMBER",                   9},
    {"LC_SUBJECT_HEADING",               27},
    {"LOCAL_CLASSIFICATION",             20},
    {"LOCAL_NUMBER",                     12},
    {"LOCAL_SUBJECT_INDEX",              29},
    {"MAP_SCALE",                        1024},
    {"MATERIAL_TYPE",                    1031},
    {"MESH_SUBJECT",                     25},
    {"MICROFORM_GENERATION",             61},
    {"MOS_CALL_NUMBER",                  19},
    {"MUSIC_KEY",                        1025},
    {"NAL_CALL_NUMBER",                  18},
    {"NAME",                             1002},
    {"NAME_AND_TITLE",                   57},
    {"NAME_GEOGRAPHIC",                  58},
    {"NLM_CALL_NUMBER",                  17},
    {"NOTE",                             63},
    {"NUMBER_DB",                        52},
    {"NUMBER_GOVT_PUBLICATION",          50},
    {"NUMBER_LEGAL_DEPOSIT",             49},
    {"NUMBER_LOCAL_CALL",                53},
    {"NUMBER_NATIONAL_BIBLIOGRAPHY",     48},
    {"NUMBER_PUBLISHER_FOR_MUSIC",       51},
    {"PA_SUBJECT",                       26},
    {"PERSONAL_NAME",                    1},
    {"PLACE_PUBLICATION",                59},
    {"PUBLISHER",                        1018},
    {"RECORD_SOURCE",                    1019},
    {"RECORD_TYPE",                      1001},
    {"RELATED_PERIODICAL",               1026},
    {"REPORT_NUMBER",                    1027},
    {"RVM_SUBJECT_HEADING",              28},
    {"SERVER_CHOICE",                    1017},
    {"STOCK_NUMBER",                     1028},
    {"SUBJECT_HEADING",                  21},
    {"SUBJECT_LC_CHILDRENS",             1008},
    {"SUBJECT_NAME_PERSONAL",            1009},
    {"SUBJECT_PRECIS",                   45},
    {"SUBJECT_RAMEAU",                   22},
    {"SUBJECT_RSWK",                     46},
    {"SUBJECT_SUBDIVISION",              47},
    {"THEMATIC_NUMBER",                  1030},
    {"TITLE",                            4},
    {"TITLE_ABBREVIATED",                43},
    {"TITLE_ADDED_TITLE_PAGE",           37},
    {"TITLE_CAPTION",                    38},
    {"TITLE_COLLECTIVE",                 34},
    {"TITLE_COVER",                      36},
    {"TITLE_EXPANDED",                   44},
    {"TITLE_FORMER",                     42},
    {"TITLE_KEY",                        33},
    {"TITLE_OTHER_VARIANT",              41},
    {"TITLE_PARALLEL",                   35},
    {"TITLE_RUNNING",                    39},
    {"TITLE_SERIES",                     5},
    {"TITLE_SPINE",                      40},
    {"TITLE_UNIFORM",                    6},
    {"UDC_CLASSIFICATION",               14}
};

/*
 * (1)    "term" is the right operand of the  relation. For example, if 'Use' is
 *        'date', 'relation' is 'less than', and 'term' is '1800', then the
 *        relationship is "date less than 1800".
 * (2)    "stem" refers to a linguistic match; "phonetic" refers to a soundex or
 *        some similar type of match based on aural similarity. For example
 *        stemming might match 'women' and 'woman', but not 'night' and 'knight';
 *        'woman' and 'women' would not match phonetically but 'knight' and
 *        'night' would.
 */

static ATTRIBUTE_INFO Relation_Attributes[] =
{
    {"ALWAYS_MATCH",             103},
    {"EQUAL",                    3},
    {"GREATER_OR_EQUAL",         4},
    {"GREATER_THAN",             5},
    {"LESS_THAN",                1},
    {"LESS_THAN_OR_EQUAL",       2},
    {"NOT_EQUAL",                6},
    {"PHONETIC",                 100},
    {"RELEVANCE",                102},
    {"STEM",                     101}
};


/*
 *  The Position attribute specifies the location of the search term within the
 *  field/subfield in which it appears.  The Position attributes are the
 *  following:
 *
 *     First in field             1     Search term must be the first data in
 *                                      the field.
 *
 *     First in any subfield      2     Search term may appear in any subfield
 *                                      but must be the first data in the
 *                                      subfield in which it appears.
 *
 *     Any position in field      3     Search term may appear any place in
 *                                      the field.
 */

static ATTRIBUTE_INFO Position_Attributes[] =
{
    {"ANY_POSITION_IN_FIELD",    3},
    {"FFIELD",                   1},     /* A synonym for FIRST_IN_FIELD */
    {"FSUBFILED",                2},     /* A synonym for FIRST_IN_SUBFIELD */
    {"FIRST_IN_FIELD",           1},
    {"FIRST_IN_SUBFIELD",        2}
};


/*
 * The Structure Attribute attribute tells the target how the origin
 * wishes the Term to be scanned in the normalization process. Note
 * that the scanning may also be affected by the Truncation attribute,
 * if present.
 *
 *      Phrase             1    A phrase consists of one or more groups of
 *                              characters ("lexemes") separated (if >1)
 *                              by a "separator character ("SC").  If
 *                              the term is typed (tag != 45), the SC if
 *                              any is defined by the character set
 *                              referenced by the tag; otherwise the SC
 *                              is a blank (ascii 0x20).  The value to
 *                              be searched is exactly as it appears in
 *                              the search term with respect to order
 *                              and adjacency.
 *
 *      Word               2    A word consists of a group of characters
 *                              that does not include the SC - i.e., a
 *                              single lexeme.
 *
 *      Key                3    A key specifies a sequence of characters
 *                              extracted from those characters contained in
 *                              an indexed word but not necessarily
 *                              representing complete words.  Intersite, key
 *                              segments should be separated by a blank (ASCII
 *                              hex "20").  Each key segment should be the
 *                              length of the local key or the length of the
 *                              word, to a maximum of 6 characters.  (For
 *                              example, an intersite name/title derived key
 *                              search term for "Copland, Aaron, 1900-
 *                              Rodeo" could be "coplan rodeo".)   A segment
 *                              may be adjusted by the receiving agency to the
 *                              length required for the receiver's indexes.
 *                              For example, the following derived key
 *                              searches are in use at LC and at
 *                              OCLC (in Online System):
 *
 *                              Site  Index       Letters taken Source Data
 *                              ----- ----------  ------------- --------------
 *                              OCLC  TITLE       3,2,2,1       title keywords
 *                                    NAME/TITLE  4,4           name, title
 *                                    NAME        4,3,1         personal name
 *                                    CNAME       4,3,1         corporate name
 *                              LC    PTK         3,1,1,1       title keywords
 *                                    PATK        3,3           name, title
 *                                    PPNK        5,1 or 6      personal name
 *
 *      Year               4    A year search term is numeric and contains
 *                              four digits.
 *
 *      Date               5    The day, month, and year when a transaction or
 *      (normalized)            event takes place.  The date search term
 *                              structure is YYYYMMDD.
 *
 *                              Months shall be recorded using the following numberic values:
 *                                  01 = January                    07 = July
 *                                  02 = February                   08 = August
 *                                  03 = March                      09 = September
 *                                  04 = April                      10 = October
 *                                  05 = May                        11 = November
 *                                  06 = June                       12 = December
 *                                 
 *                              Examples:
 *                                  (1986)          = 1986 annual      [Note: () are required in the SICI]
 *                                  (19560815)      = August 15, 1956
 *                                  (198901)        = January, 1989
 *                                  ()              = no chronology
 *                                 
 *                              Seasons shall be recorded using the following numeric values:
 *                                  21 = Spring                     23 = Fall
 *                                  22 = Summer                     24 = Winter
 *                                 
 *                              Example: (197521) = Spring 1975
 *                                 
 *                              Designations referring to a portion of a year shall be recorded using the
 *                              following numberic values:
 *                                  31 = 1st quarter                33 = 3rd quarter
 *                                  32 = 2nd quarter                34 = 4th quarter
 *                                 
 *                              Example: (197733) = 3rd quarter, 1977
 *
 *      Word list          6    A word list consists of one or more lexemes
 *                              separated (if >1) by the SC.  No order
 *                              of the words is implied.  The attributes
 *                              (other than structure) that are
 *                              associated with the search term apply to
 *                              each word in the word list.  The
 *                              interpretation of wordlist - whether all
 *                              lexemes should lie in the same entry
 *                              point or the same record or whether the
 *                              value implies the intersection or union
 *                              of record sets retrieved on single
 *                              lexemes - is not specified.
 *
 *      Name             101    A name search term that is structured in a
 *      (normalized)            particular order ( last_name, first_name,
 *                              middle-name(s); e.g., "Pope-Hennessy,
 *                              John Wyndham".)  The resulting term is
 *                              subject to special matching rules
 *                              (generally, the first comma follows the
 *                              surname) on the target system that
 *                              differ from those applied to names
 *                              structured as phrases or unstructured
 *                              names.
 *
 *      Name             102    A name search term that is unstructured (e.g.,
 *      (un-normalized)         first_name last_name), however, the resulting
 *                              term is subject to matching rules on the
 *                              target system that may differ from those
 *                              applied to phrases or structured names
 *                              (generally, the final lexeme is
 *                              construed as the last name, & commas are
 *                              not significant).
 *
 *      Structure, urx, free-form-text, document-text, local number:
 *
 *     wordList_same_Entry    201  Same as wordlist, except all
 *                                 lexemes must occur in the same phrase
 *                                 or entry-point for that record to match.
 *
 *     wordList_same_Record   202  Same as wordlist, except that all
 *                                 lexemes must occur in the same record
 *                                 for that record to match.
 *
 *     wordList_Union         203  Same as wordlist, except that
 *                                 any record which contains any of
 *                                 the lexemes will match.
 */

static ATTRIBUTE_INFO Structure_Attributes[] =
{
    {"DATE_NORMALIZED",      5},
    {"DATE_UNNORMALIZED",    100},
    {"DOCUMENT_TEXT",        106},
    {"FREE_FORM_TEXT",       105},
    {"KEY",                  3},
    {"LOCAL_NUMBER",         107},
    {"NAME_NORMALIZED",      101},
    {"NAME_UNNORMALIZED",    102},
    {"NUMERIC_STRING",       109},
    {"PHRASE",               1},
    {"STRING",               108},
    {"STRUCTURE",            103},
    {"URX",                  104},
    {"WORD",                 2},
    {"WORD_LIST",            6},
    {"YEAR",                 4}
};


/*
 *  The Truncation attribute specifies whether one or more characters may be
 *  omitted in matching the search term in the target system at the position
 *  specified by the Truncation attribute.  For example, a word in a search term
 *  may be 1) right truncated, in which case the word is treated both as a
 *  complete word and as the beginning of a longer word; 2) left truncated in
 *  which case the word is treated both as a complete word and the ending of a
 *  longer word; 3) left and right truncated, in which case the word is treated as
 *  a complete word and the beginning and ending of a longer word; 4) and embedded
 *  truncation, in which case the word is treated as a complete word and as a
 *  longer word with additional characters at the point where the truncation
 *  symbol, "#", appears in the search term.
 *
 *      Right truncation                    1     The last word of the search
 *                                                 term is right truncated.
 *      Left truncation                     2     The first word of the search
 *                                                 term is left truncated.
 *      Left and right truncation           3     The first word of the search
 *                                                 term is left truncated and
 *                                                 the last word of the search
 *                                                 term is right truncated.
 *      Do not truncate                   100     No truncation is to be
 *                                                 applied.
 *      Process # in the search term      101     The search term contains the
 *                                                 symbol "#" (ASCII hex "23")
 *                                                 to show where truncation
 *                                                 will take place (e.g.,
 *                                                 "National H# Institute", or
 *                                                 "d#on").
 *
 * Notes:
 *  (1)    A Glob regular expression is a string of zero or more literal or special
 *         characters. Scanning the term from left to right, all characters are
 *         interpreted as literal except when one of the following is encountered:
 *         *      Matches any string of zero or more characters.
 *         ?      Matches any single character.
 *         [      When '[' is encountered, followed by a sequence of zero or more
 *                characters, followed by ']',  the entire string (left and right
 *                square bracket and the enclosed sequence of characters) matches
 *                any single character from the sequence; unless the sequence begins
 *                with '^', in which case it matches any single character not from
 *                the sequence.  If two characters in the sequence are separated by
 *                '-', this is shorthand for the full list of ASCII characters
 *                between them, inclusive (for example '[0-9]' matches any decimal
 *                digit).  ']' is interpreted as a literal in the sequence if it is
 *                the first character (following a possible '^').  '-' is
 *                interpreted as a literal if it is the first or last character.
 *         {      When '{' is encountered, followed by a sequence of zero or more
 *                regular expressions separated by commas, followed by '}',  the
 *                entire string (left and right curly bracket and the enclosed
 *                sequence of regular expressions) matches anything that matches one
 *                of the expressions from the sequence.   For example, "a{bc,de}f"
 *                matches "abcf" or "adef" (as opposed to "a[bc,de]f" which matched
 *                "abf", "acf", "a,f", "adf", or "aef").
 *  (2)    A Regexp regular expression is zero or more branches, separated by '|'.
 *         It matches anything that matches one of the branches. A branch is one or
 *         more pieces, concatenated.  It matches a match for the first followed by
 *         a match for the second,  etc. A piece is an atom possibly followed by
 *         one of the following:
 *                *      an atom followed by '*' matches a sequence of 0 or more
 *                       matches of the atom
 *                +      an atom followed by '+' matches a sequence of 1 or more
 *                       matches of the atom
 *                ?      an atom followed by '?' matches a match of the atom, or the
 *                       null string.
 *         An atom is one of the following:
 *
 * Atom                                                    Matches
 * regular expression enclosed in parenthesis              the regular expression
 * sequence enclosed in square brackets                    as described for Glob
 * '.'                                                     any single character
 * '^'                                                     null at beginning
 * '$'                                                     null at end
 * '\' followed by a single character                      the single character
 * single character                                        the character
 */

static ATTRIBUTE_INFO Truncation_Attributes[] =
{
    {"DO_NOT_TRUNCATE",                  100},
    {"GLOB",                             102},
    {"LEFT_AND_RIGHT",                   3},
    {"LEFT_TRUNCATION",                  2},
    {"PROCESS_#_IN_THE_SEARCH_TERM",     101},
    {"REGEXP",                           103},
    {"RIGHT_TRUNCATION",                 1}
};


/*
 *  The Completeness attribute specifies that the contents of the search term
 *  represent a complete or incomplete subfield or a complete field.  Completeness
 *  indicates whether additional words should appear in a field or subfield with
 *  the search term.  (Note difference from Truncation (Section 2.5 above) which
 *  handles additional characters added to single words.)  The forms of
 *  completeness are the following:
 *
 *      Incomplete subfield        1     Words other than those in the search
 *                                         term may appear in the subfield
 *                                         and/or field in which the term
 *                                         appears.
 *
 *      Complete subfield          2     No words other than those in the
 *                                         search term should appear in the
 *                                         entire subfield in which the term
 *                                         appears, but additional words may
 *                                         appear in other subfields in the
 *                                         field.
 *
 *      Complete field             3     No words other than those in the
 *                                         search term should appear in the
 *                                         entire field in which the term
 *                                         appears.
 */

static ATTRIBUTE_INFO Completeness_Attributes[] =
{
    {"CFIELD",                   3},     /* Synonym for COMPLETE_FIELD */
    {"CSUBFIELD",                2},     /* Synonym for COMPLETE_SUBFIELD */
    {"COMPLETE_FIELD",           3},
    {"COMPLETE_SUBFIELD",        2},
    {"INCOMPLETE_SUBFIELD",      1}
};


static ATTRIBUTE_TYPE Bib1_Attribute_Types[] =
{
    {"USE",           1,  1,  Use_Attributes, DIM(Use_Attributes)},
    {"RELATION",      1,  2,  Relation_Attributes, DIM(Relation_Attributes)},
    {"POSITION",      1,  3,  Position_Attributes, DIM(Position_Attributes)},
    {"STRUCTURE",     1,  4,  Structure_Attributes, DIM(Structure_Attributes)},
    {"TRUNCATE",      1,  5,  Truncation_Attributes, DIM(Truncation_Attributes)},
    {"COMPLETENESS",  1,  6,  Completeness_Attributes, DIM(Completeness_Attributes)}
};

#endif
