/*                           DATABASE.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 *
 *  This module hides the internal structure of how information
 *  about databases are stored and provides macros, subroutines
 *  to access the information.  This is so future modifications
 *  can use the EXPLAIN facility to gather information about
 *  databases, but the access procedures can be used.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef ANSI
#include <memory.h>
#endif

#include "attrcons.h"
#include "attrlist.h"
#include "constant.h"
#include "database.h"
#include "msg.h"
#include "portinfo.h"
#include "utility.h"

#define INDEX_ASSIGN      "="
#define INDEX_SEPERATOR   " "

/*--------------------------------------------------------------*
 |                    DATABASE_INDEX                            |
 | Function:                                                    |
 |    Find the database index by name.                          |
 | Parameters:                                                  |
 |    db      : Database reference.                             |
 |    name    : Name of index to find.                          |
 |    length  : Length of name.                                 |
 | Returns:                                                     |
 |    Index reference or null pointer.                          |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

INDEX_REFERENCE database_index(DATABASE_REFERENCE db,
                               char *name,
                               int  length)
{
    int  total;
    char index_name[MAX_NAME_SIZE+1];
    INDEX_REFERENCE index;

    /* 12/12/94 slh - guard against array overflow */
 
    length = MIN(length, MAX_NAME_SIZE);

    uppercase(index_name, name, length);
    index_name[length] = EOS;

    for (index = db->indexes, total = db->total_indexes;  total--; index++)
        {
            if (strcmp(index_name, index->name) == 0) return index;
        }

    return NULL_INDEX;
}

/*--------------------------------------------------------------*
 |                    DATABASE_NEW                              |
 | Function:                                                    |
 |    Create a new database reference.                          |
 | Parameters:                                                  |
 |    name  : Name of database.                                 |
 |    host  : Internet name or address of Z39.50 server.        |
 |    port  : Port number of Z39.50 server.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATABASE_REFERENCE database_new(char *name)
{
    DATABASE_REFERENCE db;

    db = (DATABASE_REFERENCE) malloc(sizeof(struct database_description));

    if (db)
        {
            strcpy(db->name, name);
            db->total_indexes = 0;
	    db->indexes = (INDEX *) NULL;
        }

    return db;
}

/*--------------------------------------------------------------*
 |                    DATABASE_SET_INDEX                        |
 | Function:                                                    |
 |    Set the database indexes.                                 |
 | Parameters:                                                  |
 |    db             : Database reference.                      |
 |    attributes_id  : Attribute object identifier.             |
 |    type           : Type of index searches.                  |
 |    indexes        : Array of indexes entries to add.         |
 |    total          : Number of index entries in array.        |
 | Returns:                                                     |
 |    Number of indexes set for database.                       |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    This is used to set BIB1 indexes, in the future EXPLAIN   |
 |    will perform this facility.                               |
 *--------------------------------------------------------------*/

int database_set_index(DATABASE_REFERENCE db,
                       char  *attributes_id,
                       int   type,
                       INDEX_REFERENCE indexes,
                       int   total)
{
    db->attribute_id = attributes_id;
    db->type = type;

    if (total > 0)
	{
	    db->indexes = (INDEX_REFERENCE) calloc(total, sizeof(INDEX));

	    if (db->indexes == (INDEX_REFERENCE) NULL)
		{
		    db->alloc_indexes = 0;
		}
	    else 
		{
		    COPY(db->indexes, indexes, total * sizeof(INDEX));
		    db->alloc_indexes = total;
		}
	}
    else
	{
	    db->indexes = (INDEX_REFERENCE) NULL;
	    db->alloc_indexes = 0;
	}
    
    db->total_indexes = db->alloc_indexes;
    return (db->total_indexes);
}

/*--------------------------------------------------------------*
 |                    DATABASE_NEW_INDEX                        |
 | Function:                                                    |
 |    Add a new index to a database description.                |
 | Parameters:                                                  |
 |    db              : Database reference.                     |
 |    name            : Name of index.                          |
 |    attribute_list  : Index attributes in type,value pairs.   |
 | Returns:                                                     |
 |    Ok or NotOk.                                              |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int database_new_index(DATABASE_REFERENCE db,
                       char  *name,
                       char  *attribute_list)
{
    INDEX_REFERENCE index;
    INDEX new;

    if (db == NULL_DATABASE)
        {
            msg_make(MISSING_DATABASE);
            return NOTOK;
        }

    if (strlen(name) == 0)
        {
            msg_make(MISSING_INDEX_NAME);
            return NOTOK;
        }

    if (strlen(attribute_list) == 0)
        {
            msg_make(MISSING_ATTRIBUTES);
            return NOTOK;
        }

    memset((void *) &new, 0, sizeof(INDEX));
    uppercase(new.name, name, strlen(name));
    uppercase(attribute_list, attribute_list, strlen(attribute_list));
    
    if (attribute_info_parse(new.attributes,
                             attribute_list, 
                             INDEX_ASSIGN,
                             INDEX_SEPERATOR))
        return NOTOK;

    if (new.attributes[1] == 0)
        {
            msg_make(MISSING_USE_ATTRIBUTE);
            return NOTOK;
        }

    index = database_index(db, name, strlen(name));
    if (index == NULL_INDEX)
        {
            if (db->total_indexes >= db->alloc_indexes)
                {
		    void *new_table;

		    if (db->indexes == (INDEX_REFERENCE) NULL)
			new_table = calloc(EXTRA_INDEXES, 
					   sizeof(INDEX));
		    else
			new_table = realloc(db->indexes, 
					    (db->alloc_indexes + 
					     EXTRA_INDEXES) * 
					    sizeof(INDEX));

		    if (new_table == NULL)
                        return msg_make(TOO_MANY_INDEXES);
		   
		    db->indexes = (INDEX_REFERENCE) new_table;
		
		    db->alloc_indexes += EXTRA_INDEXES;
                }
            index = &db->indexes[db->total_indexes++];
        }

    COPY(index, &new, sizeof(INDEX));

    return OK;
}

/*--------------------------------------------------------------*
 |                    DATABASE_SHOW_INDEX                       |
 | Function:                                                    |
 |    Show indexes in the database.                             |
 | Parameters:                                                  |
 |    db  : Database reference.                                 |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Puts the indexs defined for the database in the message   |
 |    queue.                                                    |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int database_show_index(DATABASE_REFERENCE db)
{
    INDEX_REFERENCE index;
    int  total;

    if (db == NULL_DATABASE)
        {
            msg_make(MISSING_DATABASE);
            return NOTOK;
        }

    for (index = db->indexes, total = db->total_indexes;  total--; index++)
        {
            msg_make(FORMAT_MSG,
                     "Index %s: %d %d %d %d %d %d",
                     index->name,
                     index->attributes[1],
                     index->attributes[2],
                     index->attributes[3],
                     index->attributes[4],
                     index->attributes[5],
                     index->attributes[6]);
        }

    return OK;
}

/*--------------------------------------------------------------*
 |                    DATABASE_DISPOSE                          |
 | Function:                                                    |
 |    Create a new database reference.                          |
 | Parameters:                                                  |
 |    name  : Name of database.                                 |
 |    host  : Internet name or address of Z39.50 server.        |
 |    port  : Port number of Z39.50 server.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int database_dispose(DATABASE_REFERENCE *db)
{
    if (*db != NULL)
        {
            free(*db);
            *db = NULL;
        }

    return OK;
}









