/*                             DIAG.C                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdio.h>

#include "diag.h"
#include "msg.h"
#include "record.h"
#include "putget.h"
#include "pdutags.h"
#include "series.h"
#include "session.h"

#define UNKNOWN_MESSAGE "Unknown record syntax to display"

/*--------------------------------------------------------------*
 |                 DIAGNOSTIC_ADD_TO_SERIES                     |
 | Function:                                                    |
 |    Add the diagnostic to the series.                         |
 | Parameters:                                                  |
 |    diag    : Pointer to the data nodes.                      |
 |    series  : Pointer the record series.                      |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Adds the diagnostic message to the series.  Right now     |
 |    it only handles BIB1 type error messages.                 |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void diagnostic_add_to_series(DATA_DIR *diag,
				     SERIES *diag_series)
{
    char *oid;
    STRING data;
    RECORD *error;
   
    error = (RECORD *) series_next(diag_series);
    if (error == (RECORD *) NULL) return;
                  
    get_oid(diag, OBJECTID, &oid, OPTIONAL);
    
    if (oid)      
        {
	    error->syntax = record_syntax_from_oid(oid);
	                  
            if (error->syntax != DIAG_B1)
                error->syntax = UNKNOWN;
	}
    else          
        error->syntax = DIAG_B1;

    /* Default diagnostic */
                  
    get_int(diag, INTEGER, &error->condition, REQUIRED);
    
    get_string(diag, VISIBLESTR, &data, REQUIRED);
                  
    error->location = data.location;
    error->length   = data.length;
}

/*--------------------------------------------------------------*
 |                    EXTRACT_DIAGNOSTIC                        |
 | Function:                                                    |
 |    Extract the diagnostic from the ASN.1                     |
 | Parameters:                                                  |
 |    diag    : Pointer to the data nodes.                      |
 |    series  : Pointer the record series.                      |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Handle a sequence of diagnostic messages.                 |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void diagnostic_extract(DATA_DIR *diag,
			SERIES *diag_series)
{
    char *oid;
    STRING data;
    RECORD *error;

    if (diag->fldid == SEQUENCE && diag->Class == ASN1_UNIVERSAL)
	{
	    while (diag && diag->fldid == SEQUENCE)
		{
		    DATA_DIR *dn = diag->ptr.child;

		    diagnostic_add_to_series(dn, diag_series);
		
		    diag = diag->next;
		}
	}
    else
	diagnostic_add_to_series(diag, diag_series);
}

/*--------------------------------------------------------------*
 |                    DIAG_BIB1_OUTPUT                          |
 | Function:                                                    |
 |    Output a BIB1 diagnostic record.                          |
 | Parameters:                                                  |
 |    session  : Session information.                           |
 |    record   : The record information.                        |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    If a position in the result set was provided (> 0) then   |
 |    the diagnostic was for a particular record in the result  |
 |    set.                                                      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int diag_bib1_display(SESSION *session,
                      RECORD *record,
                      int position)
{
    extern FILE *out;

    if (position < 0)
        fprintf(out, "-DIAGNOSTIC: (%d) %.*s\n",
                record->condition,
                record->length,
                record->location);
    else
#ifdef MARKER
        fprintf(out, "ADDUPD;\nID=%d; ERR=\"%d: %.*s\";\n",
                position,
                record->condition,
                record->length,
                record->location);
#else
        fprintf(out, "Record %d\n-DIAGNOSTIC: (%d) Error reported from server: %.*s\n",
                position,
                record->condition,
                record->length,
                record->location);  
#endif

    return 0;
}

/*--------------------------------------------------------------*
 |                  DIAG_UNKNOWN_DISPLAY                        |
 | Function:                                                    |
 |    Output a BIB1 diagnostic record.                          |
 | Parameters:                                                  |
 |    session  : Session information.                           |
 |    record   : The record information.                        |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    If a position in the result set was provided (> 0) then   |
 |    the diagnostic was for a particular record in the result  |
 |    set.                                                      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int diag_unknown_display(SESSION *session,
                         RECORD *record,
                         int position)
{
    record->location = UNKNOWN_MESSAGE;
    record->length   = sizeof(UNKNOWN_MESSAGE);
    record->condition = 0;

    return diag_bib1_display(session, record, position);
}
