/*                             INFO1.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdio.h>

#include "berutil.h"
#include "constant.h"
#include "msg.h"
#include "putget.h"
#include "pdutags.h"
#include "record.h"
#include "session.h"
#include "strings.h"

extern FILE *out;

enum info1_tags
{
    FieldCount = 0,
    UserMessage,
    Rank,
    PositionalFields,
    TaggedFields,
    Records,
    Tag,
    Value,
    Hits,
    Flags,
    Qualifiers,
    VariantSize,
    VariantMessage,
    Offset,
    Length,
    QualifierType,
    QualifierValue,
    ObjectId
};

/*--------------------------------------------------------------*
 |                    DISPLAY_STRING                            |
 | Function:                                                    |
 |    Display the value as a string.                            |
 | Parameters:                                                  |
 |    node     : Pointer to data units.                         |
 |    tag      : The tag number of the data element.            |
 |    name     : The name of the element.                       |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void display_string(DATA_DIR *node,
                           unsigned int tag,
                           char *name)
{
    STRING display;

    get_string(node, tag, &display, OPTIONAL);
    if (display.length > 0)
        fprintf(out, "%s = %*.*s\n", name, 
                (int) display.length,
                (int) display.length, 
                display.location);
}

/*--------------------------------------------------------------*
 |                 DISPLAY_TAGGED_FIELDS                        |
 | Function:                                                    |
 |    Display the INFO1 tagged fields.                          |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void display_tagged_fields(DATA_DIR *node)
{
    STRING value;
    int  tag;
    int  hits;

    while (node && node->fldid == SEQUENCE)
        {
            get_int(node->ptr.child, Tag, &tag, REQUIRED);
            get_string(node->ptr.child, Value, &value, OPTIONAL);
            get_int(node->ptr.child, Hits, &hits, OPTIONAL);

            if (hits > 0)
                fprintf(out, "%6d (%3d hits) = %.*s\n",
                        tag, hits, (int) value.length, value.location);
            else
                fprintf(out, "%6d = %.*s\n",
                        tag, (int) value.length, value.location);

            node = node->next;
        }
}

/*--------------------------------------------------------------*
 |                 DISPLAY_INFO1                                |
 | Function:                                                    |
 |    Display the INFO1 records.                                |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_info1(DATA_DIR *node)
{
    DATA_DIR *field;

    display_string(node, ElementSetName, "ElementSetName");
    display_string(node, UserMessage, "UserMessage");

    field = get_node(node, TaggedFields, OPTIONAL);
    if (field) display_tagged_fields(field);

    return OK;
}

/*--------------------------------------------------------------*
 |                 INFO1_DISPLAY                                |
 | Function:                                                    |
 |    Display the INFO1 records.                                |
 | Parameters:                                                  |
 |    record   : The pointer to the INFO1 record.               |
 |    length   : Length of the record.                          |
 |    position : The ordinal position in the result.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    The INFO1 records are themselves in ASN.1 format.  So     |
 |    allocate another BER decoding area and use it to decode   |
 |    the records.                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int info1_display(SESSION *session,
                  RECORD *record,
                  int  position)
{
    DATA_DIR *info = session->rdu;
    int status;

    if (bld_dir((CHAR *) record->location, info) == 0)
        status = msg_make(CANT_CONVERT_BER, "OPAC");
    else
        {
            if (info->fldid != SEQUENCE)
                status = msg_make(BAD_FORMAT,"OPAC record");
            else
                {
                   fprintf(out,"\nRecord %d\n", position);
                   status = display_info1(info->ptr.child);
                }
        }
    return status;
}
