/*                            INITIAL.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 *
 *  Encodes and decodes the Initialization response and request
 *  for the Z39.50 client.
 */

#include "constant.h"
#include "msg.h"
#include "pdutags.h"
#include "putget.h"
#include "netio.h"
#include "portinfo.h"
#include "session.h"
#include "transmit.h"
#include "z3950.h"

/* The tags for the initialize response/request */

/* Shared by both Initialize Request and Response */

enum initialize_tags
{
    ProtocolVersion = 3,
    Options,
    PreferredMessageSize,
    MaximumRecordSize,
    IdAuthentication,                     /* For Initialize REQUEST only */
    UserInformationField = 11,
    Result,                               /* For Initialize RESPONSE only */
    ImplementationId = 110,
    ImplementationName,
    ImplementationVersion
};

/*--------------------------------------------------------------*
 |                  ENCODE_INITIALIZE_REQUEST                   |
 | Function:                                                    |
 |    Endode the ASN.1 Initialization Request.                  |
 | Parameters:                                                  |
 |    session       : Pointer to session information.           |
 |    information   : String of information to send server.     |
 |    authenticate  : String of authentication code for server. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/
static void Encode_Initialize_Request(SESSION  *session,
                                      STRING   *authenticate,
                                      STRING   *information)
{
    DATA_DIR *PInit;

    PInit = put_begin(session->pdu, InitializeRequest);
    if (PInit == NULL_DIR) return;

    /* If a reference id for this session then add the reference */

    if (session->reference)
        put_cstring(PInit, ReferenceId, session->reference);

    put_bits(PInit, ProtocolVersion, PROTOCOL_VERSION);

    put_bits(PInit, Options, OPTIONS);

    /* Set defaults for message and record size if not set */

    DEFAULT(session->preferred_msg_size, PREFERRED_MESSAGE);
    put_int(PInit, PreferredMessageSize, session->preferred_msg_size);

    DEFAULT(session->maximum_msg_size, MAXIMUM_MESSAGE);
    put_int(PInit, MaximumRecordSize, session->maximum_msg_size);

    /* Add Id authentication if specified */

    if (authenticate && authenticate->length > 0)
        {
            DATA_DIR *other;
            other = put_node(PInit, IdAuthentication, ASN1_CONTEXT);
            if (other != NULL_DIR)
                {
                    if (!daddchar(other,
                                  VISIBLESTR,
                                  ASN1_UNIVERSAL,
                                  (CHAR *) authenticate->location,
                                  authenticate->length))
                        msg_make(PUT_VALUE, IdAuthentication);
                }
        }

    put_cstring(PInit, ImplementationId, CLIENT_ID);

    put_cstring(PInit, ImplementationName, CLIENT_NAME);

    put_cstring(PInit, ImplementationVersion, CLIENT_VERSION);

    if (information && information->length > 0)
        {
             DATA_DIR *other;

             other = put_node(PInit, UserInformationField, ASN1_CONTEXT);
             other = put_node(other, EXTERNAL, ASN1_UNIVERSAL);
             if (other != NULL_DIR)
                 put_string(other, 1, information);
        }
}

/*--------------------------------------------------------------*
 |                  DECODE_INITIALIZE_RESPONSE                  |
 | Function:                                                    |
 |    Decode the ASN.1 Initialization Response.                 |
 | Parameters:                                                  |
 |    session       : Pointer to session information.           |
 |    information   : String of information from server.        |
 |    status        : Integer to store status from server.      |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void Decode_Initialize_Response(SESSION *session,
                                       STRING *information,
                                       int *status)
{
    DATA_DIR *PInit;
    DATA_DIR *other;
    STRING name;
    char *options;

    PInit = get_begin(session->pdu, InitializeResponse);
    if (PInit == NULL_DIR) return;

    /* Copy options since storage is temporary and the bit string *
     * returned maybe shorter than what is being tested           */

    get_bits(PInit, Options, &options, REQUIRED);
    strncpy(session->option, options, MAX_OPTIONS);
    session->option[MAX_OPTIONS+1] = EOS;

    get_int(PInit, PreferredMessageSize,
            &session->preferred_msg_size, REQUIRED);

    get_int(PInit, MaximumRecordSize, &session->maximum_msg_size, REQUIRED);

    /* Set the buffer for I/O to the maximum message size */

    net_buffer(session->io, session->maximum_msg_size);

    /* Get the name of the server */

    get_string(PInit,  ImplementationName, &name, OPTIONAL);

    if (string_length(name) > 0)
        {
            session->target_name = (char *) malloc(string_length(name) + 1);

            if (session->target_name != NULL)
                {
                   COPY(session->target_name, 
                        string_location(name), 
                        string_length(name));

                   session->target_name[string_length(name)] = '\0';
                }
        }

    get_int(PInit, Result, status, REQUIRED);

    if (information)
        {
            other = get_node(PInit, UserInformationField, OPTIONAL);
            other = get_node(other, EXTERNAL, OPTIONAL);
            if (other)
                get_string(other, 1, information, OPTIONAL);
        }
}

/*--------------------------------------------------------------*
 |                  SEND_INITIAL_PDU                            |
 | Function:                                                    |
 |    Send the Initialization packet.                           |
 | Parameters:                                                  |
 |    session       : Pointer to session information.           |
 |    authenticate  : Authentication code for the server.       |
 |    information   : String of information to/from server.     |
 |    status        : Integer to store status from server.      |
 | Returns:                                                     |
 |    If error occured or not.                                  |
 | Description:                                                 |
 |    Sends the Initialization request and receives the         |
 |    response.                                                 |
 | Note:                                                        |
 |    If the error messages are waiting then assume that an     |
 |    error has occured.                                        |
 *--------------------------------------------------------------*/

int send_initial_pdu(SESSION *session,
                     STRING  *authenticate,
                     STRING  *information,
                     int     *status)
{

    Encode_Initialize_Request(session, authenticate, information);
    if (MSG_TEST == OK)
        {
            if (transmit_pdu(session,
                             session->pdu,
                             InitializeRequest,
                             WAIT_OPEN) == OK)
                {
                    Decode_Initialize_Response(session, information, status);
                }
        }

     return MSG_TEST;
}
