/*                              MSG.C                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "constant.h"
#include "msg.h"

static char *canned_msgs[] =
   {
        "",
        "%s",
        "The program ran out of memory while allocating '%s'.",
        "The Z39.50 server does not allow %s requests.",
        "The Z39.50 server cannot handle your request right now.",
        "Missing a matching quote in the expression: %s.",
        "No index was mentioned to search.",
        "Missing term to search for in index.",
        "No index matches %s.",
        "Unbalanced parenthesis in search expression.",
        "Expecting the logical operators AND/OR/AND NOT.",
        "The maximum number of indexes have already been defined.",
        "The attribute type for an index must be between 1-6.",
        "The attributes were not input in the correct form (n,n)",
        "Missing the index name.",
        "Missing the attribute string.",
        "The use attribute (1) is missing.",
        "No database has been selected yet.",
        "The result set name was missing.",
        "Unable to encode the tag %d into BER for transport.",
        "Missing required tag %d from packet.",
        "The server %s has rejected the connection by the target.",
        "Unable to open socket connection to the internet.",
        "The host computer %s is unknown.",
        "A system error has occured in %s (%d): %s",
        "Unable to build the BER version of the record.",
        "No connnection has been established to a Z39.50 server.",
        "Invalid PDU was reached.",
        "The protocol received an unexpected PDU.",
        "Unable to convert the BER code into the ASN.1 encoding.",
        "Unable to connect an socket which is already open.",
        "Unable to perform I/O over an unconnected socket.",
        "Unable to wait for I/O over the connection.",
        "The wait for input timed out after %d seconds",
        "Unable to handle this query type, see database owner.",
        "The format for %s is unknown and cannot be converted.",
        "The resource report type %d is unknown.",
        "Records cannot be retrieved beyond the result size.",
	"A bad preferred record syntax of %s was specified.",
        "An End-Of-File was reached while reading from %s.",
        "Record %d was not returned by the server.",
        "The attribute was not correctly specified, %s.",
        "The search term was not quoted correctly."
   };

#define MAX_ERRORS    (sizeof(canned_msgs) / sizeof(canned_msgs[0]))

EMSG Messages = (EMSG) NULL;

/*--------------------------------------------------------------*
 |                    MAKE_MSG                                  |
 | Function:                                                    |
 |    Format and save the message.                              |
 | Parameters:                                                  |
 |    errnum  : The number of the error message.                |
 |    args    : Variable arguement list of message parameters.  |
 | Returns:                                                     |
 |    NOTOK                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    The errnum indexes a canned error message so messages     |
 |    can be easily changed.  Save as LIFO linked list.         |
 *--------------------------------------------------------------*/

int msg_make(unsigned int errnum,
             ...)
{
    EMSG    msg;
    va_list args;

    assert(errnum < MAX_ERRORS);

    msg = (EMSG) malloc(sizeof(ERRMSG));
    if (msg == (EMSG) NULL)
        {
            fputs("Core exhausted in routine MAKE_MSG.", stderr);
            fflush(stderr);
            return NOTOK;
        }

    va_start(args, errnum);
    
    if (errnum == SYSTEM_ERROR)         /* System Error Message */
        {
            extern errno;
            extern char *sys_errlist[];
            char *routine;
            int sys_error = errno;

            routine = va_arg(args, char *);
/*
 * strerror is not returning the right error message
 *
            sprintf(msg->message, canned_msgs[errnum],
                    routine, sys_error, strerror(sys_error));
*/
            sprintf(msg->message, canned_msgs[errnum],
                    routine, sys_error, sys_errlist[sys_error]);
        }
    else if (errnum)                    /* Error Message from table */
        {
            vsprintf(msg->message, canned_msgs[errnum], args);
        }
    else                                /* User error message */
        {
            char *format;

            format = va_arg(args, char *);

            vsprintf(msg->message, format, args);
        }

    msg->errcode = errnum;
    msg->next = Messages;
    Messages = msg;

    va_end(args);

    return NOTOK;
}

/*--------------------------------------------------------------*
 |                    WRITE_MSG                                 |
 | Function:                                                    |
 |    Output any saved messages.                                |
 | Parameters:                                                  |
 |    out   : Address of file output device.                    |
 |    msg   : Message node                                      |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    Recursively writes out message in FIFO order.             |
 *--------------------------------------------------------------*/

static void write_msg(EMSG msg)
{
    extern FILE *out;

    if (msg != NULL)
        {
            write_msg(msg->next);

            if (msg->errcode == 0)
		fprintf(out, "%s\n", msg->message);
	    else
		fprintf(out,"-DIAGNOSTIC: (%d) %s\n", 
			msg->errcode + 1000, msg->message);
            free(msg);
        }
}

/*--------------------------------------------------------------*
 |                    OUTPUT_MSG                                |
 | Function:                                                    |
 |    Output any saved messages.                                |
 | Parameters:                                                  |
 |    out   : Address of file output device.                    |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void msg_output()
{
    write_msg(Messages);
    Messages = (EMSG) NULL;
}

/*--------------------------------------------------------------*
 |                    TEST_MSG                                  |
 | Function:                                                    |
 |    Test if any messages exist.                               |
 | Parameters:                                                  |
 |    None                                                      |
 | Returns:                                                     |
 |    True (1) if messages exist, 0 if none.                    |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int msg_test()
{
    return (Messages != NULL);
}


