/*                             NETIO.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <assert.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#ifdef _AIX
#include <sys/select.h>
#endif

#ifndef ANSI
#include <memory.h>
#endif

#include "constant.h"
#include "msg.h"
#include "netio.h"
#include "portinfo.h"
#include "z3950.h"

#ifdef IO_SELECT

int Client_FD = -1;                             /* Globals */
extern int do_io_select(int operation);         /* External declarations */

#endif 

#define NULL_FDSET             (fd_set *) NULL


static void log_data(void *data,
                     int  length)
{
    extern FILE *log;

    fwrite(data, 1, length, log);
    fflush(log);
}

/*--------------------------------------------------------------*
 |                    NET_INITIALIZE                            |
 | Function:                                                    |
 |    Initialize the network area.                              |
 | Parameters:                                                  |
 |    None.                                                     |
 | Returns:                                                     |
 |    NULL if failure, otherwise pointer to NETIO area.         |
 | Description:                                                 |
 |    Allocate storage for NETIO area.                          |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

NETIO *net_initialize()
{
    NETIO *io;

    io = (NETIO *) calloc(1, sizeof(NETIO));
    if (io == NULL)
        msg_make(NO_CORE, "netio");
    else
        {
            io->socket = NOTOK;
            if (net_buffer(io, MAXIMUM_MESSAGE) == NOTOK)
                {
                    free(io);
                    io = NULL;
                }
        }

    return io;
}

/*--------------------------------------------------------------*
 |                    NET_BUFFER                                |
 | Function:                                                    |
 |    Reset the size of the buffer.                             |
 | Parameters:                                                  |
 |    io           : Pointer to network information.            |
 |    buffer_size  : The size of the buffer to allocate.        |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |    Allocate a new read buffer if necessary.                  |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int net_buffer(NETIO *io,
               int buffer_size)
{
    if (io == NETIO_NULL)
        return msg_make(NETIO_MISSING);

    /* Only allocate if a larger buffer is needed */

    if (buffer_size > io->max_length || io->buffer == NULL)
        {
           /* Free the old buffer */

           if (io->buffer != NULL) free(io->buffer);

           /* Create the new buffer */

           io->max_length = buffer_size;

           io->buffer = (unsigned char *) malloc(io->max_length);
           if (!io->buffer)
               return msg_make(NO_CORE, "io buffer");
        }

    return OK;
}

/*--------------------------------------------------------------*
 |                    NET_RESET_BUFFER                          |
 | Function:                                                    |
 |    Reset the buffer.                                         |
 | Parameters:                                                  |
 |    io           : Pointer to network information.            |
 | Returns:                                                     |
 |    The net I/O buffer.                                       |
 | Description:                                                 |
 |    Resets the buffer to null. A new buffer will be allocated |
 |    when a read takes place.                                  | 
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

unsigned char *net_reset_buffer(NETIO *io)
{
    unsigned char *old_buffer = io->buffer;
    
    io->buffer = NULL;
    
    return old_buffer;
}

/*--------------------------------------------------------------*
 |                    NET_CONNECT                               |
 | Function:                                                    |
 |    Connect to the network.                                   |
 | Parameters:                                                  |
 |    io   : Pointer to network information.                    |
 |    host : Host name/number to establish connection.          |
 |    port : Port name/number to use when connecting.           |
 | Returns:                                                     |
 |    OK or NOTOK                                               |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    Should timeout after a while on the connection.           |
 *--------------------------------------------------------------*/

int net_connect(NETIO *io,
                char  *host,
                char  *port)
{
    struct sockaddr_in  zserver;
    struct hostent      *zhost, *gethostbyname();
    struct servent      *getservbyname(), *zport;
    int                 gethostname();
    uid_t               getuid();

#ifndef NONET

    if (io->socket != NOTOK)
        return msg_make(SOCKET_ALREADY_OPEN);

    /* create a new socket */

    if ((io->socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        return msg_make(CANT_OPEN_SOCKET);

    memset((char *) &zserver, 0, sizeof(zserver));

    /* The host may either be a name (ie. LINDY.STANDFORD.EDU)  *
     * or else a number (ie. 36.54.0.11).  Either method will   *
     * work.                                                    */

    strcpy(io->host, host);
    if ((zhost = gethostbyname(host)) != NULL)
        {
            zserver.sin_family = zhost->h_addrtype;
            COPY(&zserver.sin_addr,
                   zhost->h_addr,
                   zhost->h_length);
        }
    else
        {
            zserver.sin_family = AF_INET;
            zserver.sin_addr.s_addr = inet_addr(host);
            if (zserver.sin_addr.s_addr <= 0)
                return msg_make(UNKNOWN_HOST, host);
        }

    /* The port name be either the name of the port (z3950) or  *
     * else a port number, otherwise it uses the default port   *
     * number for Z39.50 servers.                               */

    strcpy(io->port, port);
    if (isdigit(*port))
        zserver.sin_port = htons(atoi(port));
    else
        {
            if ((zport = getservbyname(port, "tcp")) == NULL)
                zserver.sin_port = (int) htons(atoi(Z3950_PORT));
            else
                zserver.sin_port = zport->s_port;
        }

    /* connect the socket to the pat server socket */

    if (connect(io->socket, &zserver, sizeof(zserver)) == -1)
        return msg_make(SYSTEM_ERROR,"connect");

    /* no problems */
#endif

    return OK;
}

/*--------------------------------------------------------------*
 |                    NET_READ                                  |
 | Function:                                                    |
 |    Read data from the network.                               |
 | Parameters:                                                  |
 |    io : Pointer to network information.                      |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |    Data will be read into the end of the data buffer.  To    |
 |    force the data to be read at the start of the buffer the  |
 |    length of data read must be reset to zero (0).            |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int net_read(NETIO *io)
{
    extern FILE *log;
    char *buffer;
    int  length;
    int  left;
    
#ifndef NONET

    if (io == NETIO_NULL)
        return msg_make(NETIO_MISSING);

    if (io->buffer == NULL) 
        net_buffer(io, io->max_length);	

    /*
     * Check if there is any room left in the buffer. This should
     * never occur, but some of the servers have been ignoring the
     * maximum size
     */

    left = io->max_length - io->length;
    
    assert(left >= 0);
    
    if (left == 0)
	{
	    char *new_buffer;
	    
	    left = io->max_length / 2;
	    
	    new_buffer = (char *) realloc(io->buffer, io->max_length + left); 
	    if (new_buffer == NULL)
		return msg_make(READ_EOF, io->host);
	    
            io->buffer = (unsigned char *) new_buffer;

            io->max_length += left;
	}        

    /* Read the data */

    buffer = (char *) io->buffer + io->length;

    length = read(io->socket, buffer, left);

    if (length == -1)
        return msg_make(SYSTEM_ERROR,"read");
    else if (length == 0)
        return msg_make(READ_EOF,io->host);

    if (log) log_data(buffer, length);

    io->length += length;
#else
    io->length = 0;
#endif

    return OK;
}

/*--------------------------------------------------------------*
 |                    NET_WRITE                                 |
 | Function:                                                    |
 |    Write data to the network.                                |
 | Parameters:                                                  |
 |    io       : Pointer to network information.                |
 |    buffer   : Buffer of data to write out.                   |
 |    length   : Amount of data to write out.                   |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |    Writes out the data to the network.                       |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int net_write(NETIO *io,
              unsigned char *buffer,
              int  length)
{
    extern FILE *log;
    int  written;

#ifndef NONET

    if (io == NETIO_NULL)
        return msg_make(NETIO_MISSING);

    while (length > 0)
       {
           written = write(io->socket, buffer, length);
           if (written == NOTOK)
               return msg_make(SYSTEM_ERROR,"write");
           else
               {
		   if (log) log_data(buffer, written); 
                   buffer += written;
                   length -= written;
               }
       }
#endif

    return OK;
}

/*--------------------------------------------------------------*
 |                    NET_WAIT                                  |
 | Function:                                                    |
 |    Wait for input on the network or from client.             |
 | Parameters:                                                  |
 |    io             : Pointer to network information.          |
 |    seconds_wait   : Number of seconds to wait.               |
 |    useconds_wait  : Number of microseconds to wait.          |
 | Returns:                                                     |
 |    Positive int :  search response ready for read.           |
 |    0            :  timeout                                   |
 |    NOTOK        :  error occurred                            |
 |    CANCEL_SEARCH:  search-cancel packet received             |
 | Description:                                                 |
 |    Wait for a set amount of time for information to          |
 |    appear on the network.  If a search-cancel packet         |
 |    is received, return CANCEL_SEARCH.                        |
 | Note: now allows interruption of search process.             |
 |                                                              |
 *--------------------------------------------------------------*/

int net_wait(NETIO *io,
             int   seconds_wait,
             long  useconds_wait)
{
    struct timeval holdtime;
    struct timeval *timeout = &holdtime;
    fd_set read_fds;
    int    max_fds = 0;
    int    ready;
    long   total;

#ifndef NONET

    if (io == NETIO_NULL)
        return msg_make(NETIO_MISSING);

    total = useconds_wait + seconds_wait;


    if (total < 0)                   /* Wait indefinitely */
        timeout = NULL;
    else if (total == 0)            /* Poll the socket */
        {
            timeout->tv_sec = 0;
            timeout->tv_usec = 0;
        }
    else                             /* Wait n microseconds */
        {
            timeout->tv_sec = seconds_wait;
            timeout->tv_usec = useconds_wait;
        }


    max_fds = MAX(max_fds, io->socket);

#ifdef IO_SELECT
    /* Wait for z39.50 read or a client packet */
    do {
#endif
      FD_ZERO(&read_fds);
      FD_SET(io->socket, &read_fds);

#ifdef IO_SELECT
      if (Client_FD != NOTOK) /* check client fd also? */
      {
          max_fds = MAX(max_fds, Client_FD);
          FD_SET(Client_FD, &read_fds);
      }
#endif

      max_fds++;
      ready = select(max_fds, &read_fds, NULL_FDSET, NULL_FDSET, timeout);
      if (ready == NOTOK)
          return msg_make(SYSTEM_ERROR,"select");
      /* Timeout? */
      if (ready == 0)
          return (total == 0) ? 0 : READ_TIMEOUT;
      /* If from database, return as usual*/
      if (FD_ISSET(io->socket, &read_fds))
          return ready;
      /* Input from client, check if it is a cancel packet */
#ifdef IO_SELECT
    } while(do_io_select(ENQUEUE) != CANCEL_SEARCH);
#endif 
    return CANCEL_SEARCH;
#endif
}

/*--------------------------------------------------------------*
 |                    NET_CLOSE                                 |
 | Function:                                                    |
 |    Close the network connection.                             |
 | Parameters:                                                  |
 |    io : Pointer to network information.                      |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int net_close(NETIO *io)
{
    int code;

#ifndef NONET

    if (io != NETIO_NULL)
        {
            if (io->socket != NOTOK)
                {
                    code = close(io->socket);
                    io->socket = NOTOK;

                    if (code == NOTOK)
                        return msg_make(SYSTEM_ERROR,"close");
                }
        }

#endif

    return OK;
}

/*--------------------------------------------------------------*
 |                    NET_DISPOSE                               |
 | Function:                                                    |
 |    Dispose of the network area.                              |
 | Parameters:                                                  |
 |    io : Pointer to network information.                      |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void net_dispose(NETIO **p_io)
{
    NETIO *io = *p_io;

    if (io != NETIO_NULL)
        {
            if (io->socket != NOTOK) net_close(io);
            if (io->buffer != NULL) free(io->buffer);
            free(io);
            *p_io = NETIO_NULL;
        }
}
