/*                             OPAC.C                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdio.h>
#include "berutil.h"
#include "constant.h"
#include "msg.h"
#include "pdutags.h"
#include "putget.h"
#include "record.h"
#include "session.h"

/* Tags for OPAC summary records */

enum opac_record_tags
{
    Bibliographic = 1,
    HoldingsData
};

enum holdings_data_tags
{
    MarcHoldingsRecord = 1,
    HoldingsAndCircData
};

enum holdings_and_circ_tags
{
    TypeOfRecord = 1,
    EncodingLevel,
    Format,
    ReceiptAcqStatus,
    GeneralRetention,
    Completeness,
    DateOfReport,
    Nuccode,
    LocalLocation,
    ShelvingLocation,
    CallNumber,
    ShelvingData,
    CopyNumber,
    PublicNote,
    ReproductionNote,
    TermsUseRepro,
    EnumAndChronHold,
    VolumesHeld,
    CirculationData
};

enum volume_tags
{
    Enumeration = 1,
    Chronology,
    EnumAndChronVol
};

enum circulation_record_tags
{
    AvailableNow = 1,
    AvailabilityDate,
    AvailableThru,
    Restrictions,
    ItemId,
    Renewable,
    OnHold,
    EnumAndChronCirc,
    MidSpine,
    TemporaryLocation
};

/*--------------------------------------------------------------*
 |                    DISPLAY_STRING                            |
 | Function:                                                    |
 |    Display the value as a string.                            |
 | Parameters:                                                  |
 |    node     : Pointer to data units.                         |
 |    tag      : The tag number of the data element.            |
 |    name     : The name of the element.                       |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void display_string(DATA_DIR *node,
                           unsigned int tag,
                           char *name)
{
    extern FILE *out;
    STRING display;

    get_string(node, tag, &display, OPTIONAL);

    if (display.length > 0)
#ifdef MARKER
        fprintf(out, "%s = \"%.*s\";\n", name, (int) display.length, display.location);
#else
        fprintf(out, "%-18s%.*s\n", name, (int) display.length, display.location);
#endif
}

/*--------------------------------------------------------------*
 |                    DISPLAY_BOOLEAN                           |
 | Function:                                                    |
 |    Display the value as a boolean.                           |
 | Parameters:                                                  |
 |    node     : Pointer to data units.                         |
 |    tag      : The tag number of the data element.            |
 |    name     : The name of the element.                       |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void display_boolean(DATA_DIR *node,
                            unsigned int tag,
                            char *name)
{
    extern FILE *out;
    unsigned char flag;

    get_flag(node, tag, &flag, OPTIONAL);

#ifdef MARKER
    fprintf(out, "%s = %s;\n", name, (flag == 0) ? "FALSE" : "TRUE");
#else
    fprintf(out, " %-20s : %s\n", name, (flag == 0) ? "FALSE" : "TRUE");
#endif
}


/*--------------------------------------------------------------*
 |                 DISPLAY_CIRCREC                              |
 | Function:                                                    |
 |    Display the Circulation Record.                           |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_circrec(DATA_DIR *node)
{
    display_boolean(node, AvailableNow, "AvailableNow");
    display_string(node, AvailabilityDate, "AvailabilityDate");
    display_string(node, AvailableThru, "AvailableThru");
    display_string(node, Restrictions, "Restrictions");
    display_string(node, ItemId, "ItemId");
    display_boolean(node, Renewable, "Renewable");
    display_boolean(node, OnHold, "OnHold");
    display_string(node, EnumAndChronCirc, "EnumAndChron");
    display_string(node, MidSpine, "MidSpine");
    display_string(node, TemporaryLocation, "TemporaryLocation");

    return OK;
}

/*--------------------------------------------------------------*
 |                 DISPLAY_CIRCDATA                             |
 | Function:                                                    |
 |    Display the Circulation Data information.                 |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_circdata(DATA_DIR *node)
{
    while (node && node->fldid == SEQUENCE)
        {
            DATA_DIR *field = node->ptr.child;

            display_circrec(field);

            node = node->next;
        }
    return OK;
}

/*--------------------------------------------------------------*
 |                 DISPLAY_VOLUME                               |
 | Function:                                                    |
 |    Display the Volume Information                            |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_volume(DATA_DIR *node)
{
    display_string(node, Enumeration, "Enumberation");
    display_string(node, Chronology, "Chronology");
    display_string(node, EnumAndChronVol, "EnumAndChron");

    return OK;
}

/*--------------------------------------------------------------*
 |                 DISPLAY_VOLHELD                              |
 | Function:                                                    |
 |    Display the Volumes Held information.                     |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_volheld(DATA_DIR *node)
{
    while (node && node->fldid == SEQUENCE)
        {
            DATA_DIR *field = node->ptr.child;

            display_volume(field);

            node = node->next;
        }
    return OK;
}

/*--------------------------------------------------------------*
 |                 DISPLAY_HOLDCIRC                             |
 | Function:                                                    |
 |    Display the Holdings and Circulation data.                |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_holdcirc(DATA_DIR *node)
{
    DATA_DIR *field;

    display_string(node, TypeOfRecord, "TypeOfRecord");
    display_string(node, EncodingLevel, "EncodingLevel");
    display_string(node, Format, "Format");
    display_string(node, ReceiptAcqStatus, "ReceiptAcqStatus");
    display_string(node, GeneralRetention, "GeneralRetention");
    display_string(node, Completeness, "Completeness");
    display_string(node, DateOfReport, "DateOfReport");
    display_string(node, Nuccode, "Nuccode");
    display_string(node, LocalLocation, "LocalLocation");
    display_string(node, ShelvingLocation, "ShelvingLocation");
    display_string(node, CallNumber, "CallNumber");
    display_string(node, ShelvingData, "ShelvingData");
    display_string(node, CopyNumber, "CopyNumber");
    display_string(node, PublicNote, "PublicNote");
    display_string(node, ReproductionNote, "ReproductionNote");
    display_string(node, TermsUseRepro, "TermsUseRepro");
    display_string(node, EnumAndChronHold, "EnumAndChron");

    field = get_node(node, VolumesHeld, OPTIONAL);
    if (field) display_volheld(field);

    field = get_node(node, CirculationData, OPTIONAL);
    if (field) display_circdata(field);

    return OK;
}


/*--------------------------------------------------------------*
 |                 DISPLAY_HOLDCIRC                             |
 | Function:                                                    |
 |    Display the Holdings and Circulation data.                |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_holdseq(DATA_DIR *seq)
{
    while (seq && seq->fldid == SEQUENCE)
        {
            DATA_DIR *field = seq->ptr.child;

            display_holdcirc(field);

            seq = seq->next;
        }
    return OK;
}

/*--------------------------------------------------------------*
 |                 DISPLAY_HOLDINGS                             |
 | Function:                                                    |
 |    Display the holdings information.                         |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int display_holdings(DATA_DIR *node)
{
    while (node && node->fldid == SEQUENCE)
        {
            DATA_DIR *field = node->ptr.child;

            while (field)
                {
                    if (field->fldid == HoldingsAndCircData)
                        display_holdseq(field->ptr.child);

                    field = field->next;
                }
            node = node->next;
        }

    return OK;
}

/*--------------------------------------------------------------*
 |                 DISPLAY_INFO1                                |
 | Function:                                                    |
 |    Display the INFO1 records.                                |
 | Parameters:                                                  |
 |    node  : Pointer to data units.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void display_opac_record(DATA_DIR *node)
{
    DATA_DIR *field;

    field = get_node(node, HoldingsData, OPTIONAL);
    if (field) display_holdings(field);
}

/*--------------------------------------------------------------*
 |                 DISPLAY_EXTERNAL_RECORD                      |
 | Function:                                                    |
 |    Extract a record from the ASN.1 and add to the series.    |
 | Parameters:                                                  |
 |    external  : Pointer to the data nodes.                    |
 |    series    : Pointer to the record series.                 |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Adds the record to the series.  Currently recognizes both |
 |    INFO-1 and US-Marc records.                               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void display_bibliographic_record(SESSION *session,
                                         DATA_DIR *external,
                                         int position)
{
    DATA_DIR *node = external->ptr.child;
    RECORD record;
    STRING data;
    char *oid;

    get_oid(node, OBJECTID, &oid, OPTIONAL);
    if (oid)
        record.syntax = record_syntax_from_oid(oid);
    else
        record.syntax = USMARC;  /* Default display */

    get_string(node, ExternalEncodedOctet, &data, OPTIONAL);

    record.location  = data.location;
    record.length    = data.length;
    record.condition = 0;

    record_display(session, &record, position);
}

/*--------------------------------------------------------------*
 |                 INFO1_DISPLAY                                |
 | Function:                                                    |
 |    Display the INFO1 records.                                |
 | Parameters:                                                  |
 |    record   : The pointer to the INFO1 record.               |
 |    length   : Length of the record.                          |
 |    position : The ordinal position in the result.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    The INFO1 records are themselves in ASN.1 format.  So     |
 |    allocate another BER decoding area and use it to decode   |
 |    the records.                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int opac_display(SESSION *session,
                 RECORD  *record,
                 int position)
{
    DATA_DIR *seq = session->rdu;

    if (bld_dir((CHAR *) record->location, seq) == 0)
        return msg_make(CANT_CONVERT_BER, "OPAC");

    if (seq->fldid != SEQUENCE)
        return msg_make(BAD_FORMAT,"OPAC record");
    else
        {
            for (; seq && seq->fldid == SEQUENCE; seq = seq->next)
                {
                    DATA_DIR *node;

                    for (node = seq->ptr.child; node; node = node->next)
                        {
                            if (node->fldid == Bibliographic)
                                display_bibliographic_record(session,
                                                             node->ptr.child, 
                                                             position);
                            if (node->fldid == HoldingsData)
                                display_opac_record(node);
                        }
                }
        }
    return 0;
}
