/*                            OUTPUT.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdio.h>
#include "msg.h"

extern FILE *out;
 
/*--------------------------------------------------------------*
 |                    OUTPUT_BIB1_DIAG                          |
 | Function:                                                    |
 |    Output a BIB1 diagnostic record.                          |
 | Parameters:                                                  |
 |    message   : Pointer to diagnostic message.                |
 |    length    : Length of diagnostic message.                 |
 |    condition : Condition code of diagnostic.                 |
 |    position  : Ordinal position of diagnostic in result set. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    If a position in the result set was provided (> 0) then   |
 |    the diagnostic was for a particular record in the result  |
 |    set.                                                      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void output_bib1_diag(char *message,
                      int  length,
                      int  condition,
                      int  position)
{
    if (position < 0)
        fprintf(out, "-DIAGNOSTIC: (%d) %.*s\n",
                condition, length, message);
    else
        fprintf(out, "ADDUPD;\nID=%d; ERR=\"%d: %.*s\";\n",
                position, condition, length, message);
}

/*--------------------------------------------------------------*
 |                    OUTPUT_MARC                               |
 | Function:                                                    |
 |    Output a US MARC record.                                  |
 | Parameters:                                                  |
 |    message   : Pointer to the US Marc record.                |
 |    length    : Length of the record.                         |
 |    condition : Condition code of record (NOT USED).          |
 |    position  : Ordinal position of record in the result set. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Output a record in US Marc format.                        |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void output_marc(char *message,
                 int  length,
                 int  condition,
                 int  position)
{
    if (message == NULL || length <= 0)
        msg_make(INVALID_RECORD, position);
    else
        dmarc(position, message, length);
}

/*--------------------------------------------------------------*
 |                    OUTPUT_ERROR                              |
 | Function:                                                    |
 |    Output a error message.                                   |
 | Parameters:                                                  |
 |    errormsg  : Pointer to the error message.                 |
 |    condition : Condition code of error.                      |
 |    position  : Ordinal position of record in the result set. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Output a error messsage.                                  |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void output_error(char *errormsg,
                  int condition,
                  int position)
{
    output_bib1_diag(errormsg, strlen(errormsg), condition, position);
}

/*--------------------------------------------------------------*
 |                    OUTPUT_INFO1                              |
 | Function:                                                    |
 |    Output a INFO-1 record.                                   |
 | Parameters:                                                  |
 |    message   : Pointer to the INFO-1 record.                 |
 |    length    : Length of the record.                         |
 |    condition : Condition code of record (NOT USED).          |
 |    position  : Ordinal position of record in the result set. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Output a record in INFO-1 format.                         |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void output_info1(char *message,
                  int  length,
                  int  condition,
                  int  position)
{
    info1_display(message, length, position);
}


/*--------------------------------------------------------------*
 |                    OUTPUT_OPAC                               |
 | Function:                                                    |
 |    Output a OPAC record.                                     |
 | Parameters:                                                  |
 |    message   : Pointer to the OPAC record.                   |
 |    length    : Length of the record.                         |
 |    condition : Condition code of record (NOT USED).          |
 |    position  : Ordinal position of record in the result set. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Output a record in OPAC format.                           |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void output_opac(char *message,
                 int  length,
                 int  condition,
                 int  position)
{
    opac_display(message, length, position);
}

/*--------------------------------------------------------------*
 |                    OUTPUT_OPAC                               |
 | Function:                                                    |
 |    Output a OPAC record.                                     |
 | Parameters:                                                  |
 |    message   : Pointer to the OPAC record.                   |
 |    length    : Length of the record.                         |
 |    condition : Condition code of record (NOT USED).          |
 |    position  : Ordinal position of record in the result set. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Output a record in OPAC format.                           |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void output_sutr(char *message,
                 int  length,
                 int  condition,
                 int  position)
{
    sutr_display(message, length, position);
}

