/*                            PRESENT.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 *
 *  Encodes and decodes the Present response and request
 *  for the Z39.50 client.
 */

#include "berutil.h"
#include "constant.h"
#include "msg.h"
#include "putget.h"
#include "pdutags.h"
#include "place.h"
#include "recinfo.h"
#include "session.h"
#include "transmit.h"

/*--------------------------------------------------------------*
 |                  ENCODE_PRESENT_REQUEST                      |
 | Function:                                                    |
 |    Encode the ASN.1 Present Request.                         |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    elemname   : C string to the element set name to use.     |
 |    requested  : Number of records requested to be presented. |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void Encode_Present_Request(SESSION      *session,
                            char         *elemname,
			    unsigned int syntax,
                            unsigned int requested)
{
    DATA_DIR *PPresent;

    PPresent = put_begin(session->pdu, PresentRequest);
    if (PPresent == NULL_DIR) return;

    if (session->reference)
        put_cstring(PPresent, ReferenceId, session->reference);

    put_cstring(PPresent, ResultSetId, session->current_set);

    put_int(PPresent, ResultSetStartPoint, session->next_record);

    put_int(PPresent, NumberOfRecordsRequested, requested);

    place_element_set_names(PPresent, elemname, session->db);

    place_preferred_record_syntax(PPresent, syntax);
}

/*--------------------------------------------------------------*
 |                  DECODE_PRESENT_RESPONSE                     |
 | Function:                                                    |
 |    Decode the ASN.1 Present Response.                        |
 | Parameters:                                                  |
 |    session          : Pointer to session information.        |
 |    info             : Address to store record information.   |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void Decode_Present_Response(SESSION *session,
                             RECORD_INFO *info)
{
    DATA_DIR *PPresent;
    int records;
    int status;

    /* Set the position the records were picked up from */

    recinfo_set_position(info,session->next_record);

    /* Retrieve information from the response */

    PPresent = get_begin(session->pdu, PresentResponse);
    if (PPresent == NULL_DIR) return;

    get_int(PPresent, NumberOfRecordsReturned, &records, REQUIRED);

    get_int(PPresent, NextResultSetPosition, &session->next_record, REQUIRED);

    get_int(PPresent, PresentStatus, &status, REQUIRED);

    /* Set the position information */

    recinfo_set_buffer(info,net_reset_buffer(session->io));
    recinfo_set_result(info,session->result_count);
    recinfo_set_returned(info,records);
    recinfo_set_status(info,status);
    recinfo_set_dbname(info,database_name(session->db));
    recinfo_extract(info, PPresent);
}

/*--------------------------------------------------------------*
 |                  SEND_PRESENT_PDU                            |
 | Function:                                                    |
 |    Send and receive the Present packets.                     |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    elemname   : C string to the element set name to use.     |
 |    position   : Number of position in result set to present. |
 |    requested  : Number of records to present.                |
 |    records    : Address where to return record information.  |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int send_present_pdu(SESSION *session,
                     char *elemname,
		     unsigned int syntax,
                     unsigned int position,
                     unsigned int requested,
                     RECORD_INFO *records)
{
    if (position) session->next_record = position;

    requested = MIN(requested,
                    session->result_count - session->next_record + 1);

    if (requested <= 0)
        return msg_make(NO_MORE_RECORDS);

    Encode_Present_Request(session, elemname, syntax, requested);
    if (MSG_TEST == OK)
        {
            if (transmit_pdu(session,
                             session->pdu,
                             PresentRequest,
                             WAIT_PRESENT) == OK)
                {
                    Decode_Present_Response(session, records);
                }
        }

    return MSG_TEST;
}
