/*                            PUTGET.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include "berutil.h"
#include "constant.h"
#include "msg.h"
#include "portinfo.h"
#include "putget.h"
#include "pdutags.h"

/*--------------------------------------------------------------*
 |                    FIND_TAG                                  |
 | Function:                                                    |
 |    Find the data node with the matching tag value.           |
 | Parameters:                                                  |
 |    start  : Pointer to linked list of data nodes.            |
 |    tag    : Tag of data node to match.                       |
 | Returns:                                                     |
 |    Pointer to a data node, otherwise NULL.                   |
 | Description:                                                 |
 |    Sequential search through the linked list.                |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATA_DIR *find_tag(DATA_DIR *start,
                   unsigned int tag)
{
    DATA_DIR *node = start;

    while (node)
        {
            if (node->fldid == tag) return node;
            node = node->next;
        }
    return NULL_DIR;
}

/*--------------------------------------------------------------*
 |                    PUT_BEGIN                                 |
 | Function:                                                    |
 |    Start of the ASN.1 packet.                                |
 | Parameters:                                                  |
 |    irp     : Pointer to top directory node.                  |
 |    pdutype : Tag of packet type.                             |
 | Returns:                                                     |
 |    Pointer to a data node, otherwise NULL.                   |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATA_DIR *put_begin(DATA_DIR *top,
                    unsigned int pdutype)
{
    dinit(top, pdutype, ASN1_CONTEXT);

    return top;
}

/*--------------------------------------------------------------*
 |                    PUT_INT                                   |
 | Function:                                                    |
 |    Add a node to the tree with an integer value.             |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    number  : Integer value of node.                          |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_int(DATA_DIR *dir,
             unsigned int tag,
             int number)
{
    if (!daddnum(dir, tag, ASN1_CONTEXT, (CHAR *) &number, sizeof(int)))
        msg_make(PUT_VALUE, tag);
}

/*--------------------------------------------------------------*
 |                    PUT_LONG                                  |
 | Function:                                                    |
 |    Add a node to the tree with a long integer value.         |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    number  : Long integer value.                             |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_long(DATA_DIR *dir,
              unsigned int tag,
              long number)
{
    if (!daddnum(dir, tag, ASN1_CONTEXT, (CHAR *) &number, sizeof(long)))
        msg_make(PUT_VALUE, tag);
}

/*--------------------------------------------------------------*
 |                    PUT_BITS                                  |
 | Function:                                                    |
 |    Add a node to the tree with a bit string value.           |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    number  : Pointer to bit string value as a cstring.       |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Pass the command as cstring with '1' for an ON bit and    |
 |    '0' for the off bit (ie. "1001011").                      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_bits(DATA_DIR *dir,
              unsigned int tag,
              char *bits)
{
    if (!daddbits(dir, tag, ASN1_CONTEXT, (char far *) bits))
        msg_make(PUT_VALUE, tag);
}

/*--------------------------------------------------------------*
 |                    PUT_FLAG                                  |
 | Function:                                                    |
 |    Add a node to the tree with a flag (boolean) value.       |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    flag    : Flag (boolean) value.                           |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_flag(DATA_DIR *dir,
              unsigned int tag,
              CHAR flag)
{
    char *byte;

    /* Use the octal form to bypass compiler warning messages */

    if (flag)
        byte = "\377";          /* True flag (xFF) */
    else
        byte = "\000";          /* False flag (x00) */

    if (!daddchar(dir, tag, ASN1_CONTEXT, (CHAR *) byte, 1))
        msg_make(PUT_VALUE, tag);
}

/*--------------------------------------------------------------*
 |                    PUT_STRING                                |
 | Function:                                                    |
 |    Add a node to the tree with a string value.               |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    str     : String value to add.                            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_string(DATA_DIR *dir,
                unsigned int tag,
                STRING *str)
{
    if (!daddchar(dir, tag, ASN1_CONTEXT, (CHAR *) str->location, str->length))
        msg_make(PUT_VALUE, tag);
}

/*--------------------------------------------------------------*
 |                    PUT_CSTRING                               |
 | Function:                                                    |
 |    Add a node to the tree with a c-string value.             |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    str     : Pointer to a c-string.                          |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_cstring(DATA_DIR *dir,
                 unsigned int tag,
                 char *str)
{
    if (!daddchar(dir, tag, ASN1_CONTEXT, (CHAR *) str, strlen(str)))
        msg_make(PUT_VALUE, tag);
}

/*--------------------------------------------------------------*
 |                    PUT_OID                                   |
 | Function:                                                    |
 |    Add a node to the tree with a Object IDentifier value.    |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    str     : Pointer to object identifier.                   |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    The object identifier is in the form of a series of       |
 |    numbers seperated by periods (ie. "1.0.2.10002.503").     |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void put_oid(DATA_DIR *dir,
	     unsigned int tag,
             char *string)
{
    CHAR class = (tag == OBJECTID) ? ASN1_UNIVERSAL : ASN1_CONTEXT;
 
    if (!daddoid(dir, tag, class, string))
	msg_make(PUT_VALUE, OBJECTID);
}

/*--------------------------------------------------------------*
 |                    PUT_NODE                                  |
 | Function:                                                    |
 |    Add a non-leaf node to the tree.                          |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    class   : Class of new non-leaf node.                     |
 | Returns:                                                     |
 |    Pointer to new node, otherwise NULL.                      |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATA_DIR *put_node(DATA_DIR *dir,
                   unsigned int tag,
                   CHAR class)
{
    DATA_DIR *node;

    if (dir == NULL_DIR)
        node = NULL_DIR;
    else
        {
            if ((node = daddtag(dir, tag, class)) == NULL_DIR)
                 msg_make(PUT_VALUE, tag);
        }
    return node;
}

/*--------------------------------------------------------------*
 |                    PUT_NEWNODE                               |
 | Function:                                                    |
 |    Insert a non-leaf into the tree.                          |
 | Parameters:                                                  |
 |    dir     : Pointer to the directory node.                  |
 |    tag     : Tag number of node to add.                      |
 |    class   : Class of new non-leaf node.                     |
 | Returns:                                                     |
 |    Pointer to new node, otherwise NULL.                      |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATA_DIR *put_newnode(DATA_DIR *dir,
                      unsigned int tag,
                      CHAR class)
{
    DATA_DIR *node;

    if (dir == NULL_DIR)
        node = NULL_DIR;
    else
        {
            if ((node = dinstag(dir, tag, class)) == NULL_DIR)
                 msg_make(PUT_VALUE, tag);
        }
    return node;
}

/*--------------------------------------------------------------*
 |                    GET_BEGIN                                 |
 | Function:                                                    |
 |    Get the start of an ASN.1 packet.                         |
 | Parameters:                                                  |
 |    top     : Pointer to top directory node.                  |
 |    pdutype : Tag of packet type.                             |
 | Returns:                                                     |
 |    Pointer to a data nodes, otherwise NULL.                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATA_DIR *get_begin(DATA_DIR *top,
                    unsigned int pdutype)
{
    DATA_DIR *node = NULL_DIR;

    if (top)
        {
            if (top->fldid != pdutype)
                msg_make(BAD_PDU_RESPONSE, top->fldid);
            else
                if ((node = top->ptr.child) == NULL)
                    msg_make(INVALID_PDU);
        }
    else
        msg_make(INVALID_PDU);

    return node;
}

/*--------------------------------------------------------------*
 |                    GET_INT                                   |
 | Function:                                                    |
 |    Get the integer value of the node in the tree.            |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    number  : Address of where to store the integer value.    |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_int(DATA_DIR *dir,
             unsigned int tag,
             int *number,
             int section)
{
    DATA_DIR *field;

    field = find_tag(dir, tag);

    if (field)
        *number = (int) dgetnum(field);
    else
        {
            if (section == REQUIRED) msg_make(MISSING_REQUIRED, tag);
            *number = 0;
        }
}

/*--------------------------------------------------------------*
 |                    GET_LONG                                  |
 | Function:                                                    |
 |    Get the long integer value of the node in the tree.       |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    number  : Address of where to store the long int value.   |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_long(DATA_DIR *dir,
              unsigned int tag,
              long *number,
              int  section)
{
    DATA_DIR *field;

    field = find_tag(dir, tag);

    if (field)
        *number = dgetnum(field);
    else
        {
            *number = 0;
        }
}

/*--------------------------------------------------------------*
 |                    GET_BITS                                  |
 | Function:                                                    |
 |    Get the bit string value of the node in the tree.         |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    bits    : Address of where to store the bit string.       |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_bits(DATA_DIR *dir,
              unsigned int tag,
              char **bits,
              int section)
{
    DATA_DIR *field;

    field = find_tag(dir, tag);

    if (field)
        *bits = dgetbits(field);
    else
        {
            if (section == REQUIRED) msg_make(MISSING_REQUIRED, tag);
            *bits = NULL;
        }
}

/*--------------------------------------------------------------*
 |                    GET_OID                                   |
 | Function:                                                    |
 |    Get the object identifier of the node in the tree.        |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    bits    : Address of where to store the object id.        |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_oid(DATA_DIR *dir,
             unsigned int tag,
             char **oid,
             int section)
{
    DATA_DIR *field;

    field = find_tag(dir, tag);

    if (field)
        *oid = dgetoid(field);
    else
        {
            if (section == REQUIRED) msg_make(MISSING_REQUIRED, tag);
            *oid = NULL;
        }
}

/*--------------------------------------------------------------*
 |                    GET_FLAG                                  |
 | Function:                                                    |
 |    Get the flag (boolean) of the node in the tree.           |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    bits    : Address of where to store the flag (boolean).   |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_flag(DATA_DIR *dir,
              unsigned int tag,
              CHAR *flag,
              int section)
{
    DATA_DIR *node;
    char *data;

    node = find_tag(dir, tag);

    if (node)
        data = (char *) node->ptr.data;
    else
        {
            if (section == REQUIRED) msg_make(MISSING_REQUIRED, tag);
            data = (char *) NULL;
        }

    if (data)
        {
            *flag = (unsigned char) *data;
        }
    else
        {
            *flag = (unsigned char) 0;
        }
}

/*--------------------------------------------------------------*
 |                    GET_STRING                                |
 | Function:                                                    |
 |    Get the string of the node in the tree.                   |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    bits    : Address of STRING value.                        |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_string(DATA_DIR *dir,
                unsigned int tag,
                STRING *string,
                int section)
{
    DATA_DIR *node;
    char     *data;

    node = find_tag(dir, tag);

    if (node)
        data = (char *) node->ptr.data;
    else
        {
            if (section == REQUIRED) msg_make(MISSING_REQUIRED, tag);
            data = (char *) NULL;
        }

    if (data)
        {
            string->location = (char *) data;
            string->length   = node->count;
        }
    else
        {
            string->location = NULL;
            string->length = 0;
        }
}

/*--------------------------------------------------------------*
 |                    GET_CSTRING                               |
 | Function:                                                    |
 |    Get the c-string of the node in the tree.                 |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to use.                        |
 |    bits    : Address of where to store c-string.             |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    None.                                                     |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void get_cstring(DATA_DIR *dir,
                 unsigned int  tag,
                 char *string,
                 int  section)
{
    DATA_DIR *node;
    DATA_DIR *data;

    node = find_tag(dir, tag);

    if (node)
         data = node->ptr.child;  /* skip identifier tag */
    else
        {
            if (section == REQUIRED) msg_make(MISSING_REQUIRED, tag);
            data = NULL_DIR;
        }

    if (data != NULL)
        {
           COPY(string, data->ptr.data, data->count);
           string[data->count] = EOS;
        }
    else
        *string = EOS;
}

/*--------------------------------------------------------------*
 |                    GET_NODE                                  |
 | Function:                                                    |
 |    Get the node in the tree.                                 |
 | Parameters:                                                  |
 |    dir     : Pointer to directory node.                      |
 |    tag     : Tag of data node to get.                        |
 |    section : Section type (REQUIRED, OPTIONAL, etc).         |
 | Returns:                                                     |
 |    Pointer to the node, otherwise NULL.                      |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

DATA_DIR *get_node(DATA_DIR *dir,
                   unsigned int tag,
                   int section)
{
    DATA_DIR *node;

    if (dir == NULL_DIR)
        node = NULL_DIR;
    else
        {
            node = find_tag(dir, tag);

            if (node)
                node = node->ptr.child;
            else if (section == REQUIRED)
                msg_make(MISSING_REQUIRED, tag);
        }

    return node;
}
