/*                            QUERY.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include "constant.h"
#include "database.h"
#include "msg.h"
#include "putget.h"
#include "pdutags.h"
#include "query.h"
#include "rpn.h"

/*--------------------------------------------------------------*
 |                    PARSE_RPNQUERY                            |
 | Function:                                                    |
 |    Parse the RPN Query command.                              |
 | Parameters:                                                  |
 |    db       : database information reference.                |
 |    search   : search string to query database.               |
 | Returns:                                                     |
 |    Pointer to RPN Query otherwise NULL if error.             |
 | Description:                                                 |
 |                                                              |
 |    Allocate and fill in the following ASN.1 description      |
 |                                                              |
 |    RPNQuery  ::=                                             |
 |       SEQUENCE {                                             |
 |             attributeSetId  OBJECT IDENTIFIER,               |
 |             RPNStructure                                     |
 |       }                                                      |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int parse_rpnquery(DATABASE_REFERENCE db,
                   DATA_DIR *PQuery,
                   char *search)
{
    DATA_DIR *PRpn;

    PRpn = put_node(PQuery, RPNQuery, ASN1_CONTEXT);
    if (PRpn == NULL_DIR)
        return NOTOK;
    else
        {
	    /* Attribute Set Id */

            put_oid(PRpn, OBJECTID, database_attribute_id(db));

            if (parse_rpn(db, PRpn, &search) == NULL_DIR)
                return NOTOK;
        }
    return OK;
}

/*--------------------------------------------------------------*
 |                    PARSE_QUERY                               |
 | Function:                                                    |
 |    Parse the RPN Query command.                              |
 | Parameters:                                                  |
 |    db       : database information reference.                |
 |    search   : search string to query database.               |
 | Returns:                                                     |
 |    Pointer to RPN Query otherwise NULL if error.             |
 | Description:                                                 |
 |                                                              |
 |    Allocate and fill in the following ASN.1 description      |
 |                                                              |
 |    Query ::=                                                 |
 |       CHOICE {                                               |
 |             type-0[0] ANY,                                   |
 |             type-1[1] IMPLICIT RPNQuery,                     |
 |             type-2[2] OCTET STRING,                          |
 |             type-100[100] OCTET STRING,                      |
 |             type-101[101] OCTET STRING                       |
 |       }                                                      |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int parse_query(DATABASE_REFERENCE db,
                DATA_DIR *PFind,
                char *search)
{
    DATA_DIR *PQuery;
    int code;

    switch (database_query_type(db))
        {
            case RPN:

                PQuery = put_node(PFind, Query, ASN1_CONTEXT);
                code = parse_rpnquery(db, PQuery, search);
                break;

            default:

                code = msg_make(BAD_QUERY_TYPE);
                break;
         }
    return code;
}
