/*                            RECINFO.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#ifdef ANSI
#include <memory.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#include "berutil.h"
#include "constant.h"
#include "diag.h"
#include "msg.h"
#include "pdutags.h"
#include "putget.h"
#include "recinfo.h"
#include "record.h"
#include "series.h"
#include "session.h"

#ifdef ANSI
#include <memory.h>
#endif

#define MAX_NAMES           10    /* MAX_NAME_SIZE defined in database.h */

extern FILE *out;

/*--------------------------------------------------------------*
 |                    LOOKUP_NAME                               |
 | Function:                                                    |
 |    Lookup the name of the database.                          |
 | Parameters:                                                  |
 |    info     : Pointer to record information.                 |
 |    name     : Pointer to database name.                      |
 | Returns:                                                     |
 |    Pointer to the database name.                             |
 | Description:                                                 |
 |    This routine keeps a table of the database names          |
 |    referenced by the returned records.  Once the table is    |
 |    filled it just wraps and starts over again.               |
 |                                                              |
 |    The reason a table is used is that only the first record  |
 |    from a particular database needs to specify the name.     |
 |    Subsequent records with no name specified are assumed to  |
 |    have been returned from the same database.  Therefore if  |
 |    no name is passed (length <= 0) then the name of latest   |
 |    entry in the table is returned as the database name.      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static char *lookup_name(RECORD_INFO *info,
                         STRING *name)
{
    int  count;
    int  length;

    /* Table of database names */

    static char db_name[MAX_NAMES][MAX_NAME_SIZE+1];
    static int  db_index = -1;

    /* Initialize if called for first time */

    if (db_index < 0)
        {
            memset(db_name[0], 0, MAX_NAMES * (MAX_NAME_SIZE+1));
            if (info->dbname)
                strcpy(db_name[++db_index], info->dbname);
        }

    if (name && name->length > 0)
        {
            /* Check if name already exists in the table */

            for (count = 0; count < MAX_NAMES; count++)
                {
                     if (db_name[count][0] == EOS) break;

                     if (string_ccompare(name, db_name[count]) == 0)
                         return db_name[count];
                }

            /* Add the name to the table, use in a circular */
            /* fashion, to reuse the oldest entry.          */

            if (++db_index >= MAX_NAMES) db_index = 0;
            length = MIN(name->length, MAX_NAME_SIZE);
            COPY(db_name[db_index], name->location, length);
            db_name[db_index][length] = EOS;

            return db_name[db_index];
        }

    return db_name[db_index];
}

/*--------------------------------------------------------------*
 |                    EXTRACT_RECORD                            |
 | Function:                                                    |
 |    Extract a record from the ASN.1 and add to the series.    |
 | Parameters:                                                  |
 |    external  : Pointer to the data nodes.                    |
 |    series    : Pointer to the record series.                 |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Adds the record to the series.  Currently recognizes both |
 |    INFO-1 and US-Marc records.                               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void extract_record(DATA_DIR *external,
                           SERIES  *record_series)
{
    DATA_DIR *node = external->ptr.child;
    char *oid;
    STRING data;
    RECORD *record;

    record = (RECORD *) series_next(record_series);
    if (record == (RECORD *) NULL) return;

    get_oid(node, OBJECTID, &oid, OPTIONAL);
    if (oid)
        record->syntax = record_syntax_from_oid(oid);
    else
        record->syntax = USMARC;  /* Default display */

    get_string(node, ExternalEncodedOctet, &data, OPTIONAL);

    if (data.length == 0 && record->syntax != UNKNOWN)
        {
            DATA_DIR *any = get_node(node, ExternalEncodedType, OPTIONAL);

            if (any != NULL_DIR)
		{   
		    record->location = 
			(char *) bld_rec(any, (long *) &record->length);

		    record->condition = -1;
		}
	    else
		{
		    record->location = NULL;
		    record->length = 0;
		    record->condition = 0;
		}
        }
    else
        {
            record->location  = data.location;
            record->length    = data.length;
            record->condition = 0;
        }
}

/*--------------------------------------------------------------*
 |                 EXTRACT_DATABASE_RECORDS                     |
 | Function:                                                    |
 |    Extract all of the database records.                      |
 | Parameters:                                                  |
 |    info     : Pointer to the record information.             |
 |    record   : Pointer to the data nodes.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Extract the database records and diagnostics and add to   |
 |    the series.                                               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void extract_database_records(RECORD_INFO *info,
                                     DATA_DIR *record)
{
    SERIES *series = info->records;
    char *name;

    name = lookup_name(info, NULL);

    while (record && record->fldid == SEQUENCE)
        {
            DATA_DIR *npr = record->ptr.child;

            while (npr)
                {
                    if (npr->fldid == NPR_Choice)
                        {
                            DATA_DIR *drec = npr->ptr.child;

                            if (drec->fldid == NPR_DatabaseRecord)
                                extract_record(drec->ptr.child, series);
                            else if (drec->fldid == NPR_DiagnosticRecord)
                                diagnostic_extract(drec->ptr.child, series);
                        }
                    else if (npr->fldid == NPR_DatabaseName)
                        {
                            STRING dbname;

                            get_string(npr, NPR_DatabaseName,
                                       &dbname, REQUIRED);
                            name = lookup_name(info, &dbname);
                        }

                    npr = npr->next;
                }
            record = record->next;
        }
}

/*--------------------------------------------------------------*
 |                    RECORD_EXTRACT                            |
 | Function:                                                    |
 |    The records returned are extracted.                       |
 | Parameters:                                                  |
 |    info     : Pointer to the record information.             |
 |    PRecords : Pointer to the data nodes.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void recinfo_extract(RECORD_INFO  *info,
                     DATA_DIR *PRecords)
{
    DATA_DIR *record;

    record = get_node(PRecords, NonSurrogateDiagnostic, OPTIONAL);
    if (record)
        diagnostic_extract(record, info->errors);
    else
        {
            record = get_node(PRecords, DatabaseOrSurDiagnostics, OPTIONAL);
            if (record)
                extract_database_records(info, record);
        }
}

/*--------------------------------------------------------------*
 |                    RECORD_OUTPUT                             |
 | Function:                                                    |
 |    The records in the series are output.                     |
 | Parameters:                                                  |
 |    info     : Pointer to the record information.             |
 |    position : The ordinal position in the result set.        |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Depending on the type of record, output the record.       |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void recinfo_output(SESSION *session,
                    RECORD_INFO *info)
{
    SERIES *series  = info->records;
    int    position = info->position;
    int    i;

    for (i = 1; i <= series_get_total(series); i++, position++)
        {
            record_display(session, 
			   (RECORD *) series_item(series, i), 
			   position);
#ifdef MARKER
            fprintf(out, ";\n");
#else 
            fprintf(out, "\n");
#endif
            if (MSG_TEST) msg_output();
        }
    series_set_total(series,0);
}

/*--------------------------------------------------------------*
 |                    RECORD_ERROR                              |
 | Function:                                                    |
 |    The errors are output.                                    |
 | Parameters:                                                  |
 |    info  : Pointer to the record information.                |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void recinfo_error(SESSION *session,
                   RECORD_INFO *info)
{
    SERIES *series = info->errors;
    int i;

    for (i = 1; i <= series_get_total(series); i++)
        {
            record_display(session, 
			   (RECORD *) series_item(series, i),
			   -1);
        }
    series_set_total(series, 0);
}

/*--------------------------------------------------------------*
 |                   RECINFO_NEW                                |
 | Function:                                                    |
 |    Allocate a new instance of  record information.           |
 | Parameters:                                                  |
 |    None                                                      |
 | Returns:                                                     |
 |    NULL if failure otherwise pointer to new instance.        |
 | Description:                                                 |
 |    Allocate memory and initialize.                           |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

RECORD_INFO *recinfo_new()
{
    RECORD_INFO *info;
    
    info = (RECORD_INFO *) calloc(1, sizeof(RECORD_INFO));
    
    if (info != NULL)
        {
	    info->records = series_new(sizeof(RECORD));
	    if (info->records == (SERIES *) NULL)
		recinfo_dispose(&info);
	    else
		{
		    info->errors = series_new(sizeof(RECORD));
		    if (info->errors == (SERIES *) NULL)
			recinfo_dispose(&info);
		}
	}
    else
	msg_make(NO_CORE,"record info");
    
    return info;
}

/*--------------------------------------------------------------*
 |                    RECINFO_DISPOSE                           |
 | Function:                                                    |
 |   Dispose of the record information.                         |
 | Parameters:                                                  |
 |    info    : Pointer to the record information.              |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Free any allocated memory.                                |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void recinfo_dispose(RECORD_INFO **info)
{
    if (info != NULL)
        {
            if ((*info) != NULL)
                {
                    series_dispose(&(*info)->records);
		    
		    series_dispose(&(*info)->errors);
	
                    if ((*info)->buffer != NULL) free((*info)->buffer);

                    free(*info);

                    *info = NULL;
                }
        }
}
