/*                           RESOURCE.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#ifdef ANSI
#include <memory.h>
#endif

#include "constant.h"
#include "msg.h"
#include "pdutags.h"
#include "putget.h"
#include "resource.h"
#include "session.h"
#include "strings.h"

enum resource_control_tags
{
    SuspendedFlag = 39,
    ResourceReport,
    PartialResultsAvailable,
    ResponseRequired,
    TriggeredRequestFlag,
    ContinueFlag,
    ResultSetWanted
};

enum resource_report_tags
{
    ResourceReportId = 1,
    Estimates,
    Message
};

/*--------------------------------------------------------------*
 |                 RESOURCE_REPORT_BIB1_OUTPUT                  |
 | Function:                                                    |
 |    Output the resource report information.                   |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    report   : Array of bib1 report information.              |
 |    message  : Pointer to the resource report message.        |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void resource_report_bib1_output(SESSION *session,
                                 long report[],
                                 STRING *message)
{
     int count;

     if (report[0] < 0)
         msg_make(DEFAULT_MSG, "Unable to obtain resource report information");
     else
         for (count = 1; count < MAX_RESOURCE_REPORT_BIB1; count++)
             {
                if (report[count] >= 0)
                    msg_make(FORMAT_MSG,"%s :  %d",
                             resource_report_bib1_msg[count],
                             report[count]);
             }
    if (message && message->length)
        msg_make(FORMAT_MSG, "%.*s", message->length, message->location);
}

/*--------------------------------------------------------------*
 |                 RESOURCE_REPORT_STATS                        |
 | Function:                                                    |
 |    Get the resource report statistics.                       |
 | Parameters:                                                  |
 |    PReport  : Pointer to the data nodes.                     |
 |    report   : Array of report information.                   |
 |    message  : Pointer to the report message string.          |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int resource_report_stats(DATA_DIR *PReport,
                          long     report[],
                          STRING   *message)
{
    DATA_DIR *estimate;
    char *oid;
    int type;

    /* Initialize parameters */

    memset((void *) report, -1, MAX_RESOURCE_REPORT_BIB1 * sizeof(long));

    if (PReport == NULL_DIR) return OK;

    /* Check if a valid Resource Report Format */

    get_oid(PReport, ResourceReportId, &oid, REQUIRED);
    if (oid)
        {
            if (strcmp("1.2.840.10003.7.1",oid) != 0)
                return msg_make(BAD_FORMAT, "Resource Report");
        }

    /* If a valid object identifier then get the result */

    estimate = get_node(PReport, Estimates, REQUIRED);
    while (estimate)
        {
            get_int(estimate->ptr.child, EstimateType, &type, REQUIRED);
            if (type <= 0 || type >= MAX_RESOURCE_REPORT_BIB1)
                msg_make(BAD_RR_BIB1_TYPE, type);
            else
                get_long(estimate->ptr.child, EstimateType,
                         &report[type], REQUIRED);

            estimate = estimate->next;
        }
    report[0] = 0;

    if (message)
        get_string(PReport, Message, message, OPTIONAL);

    return OK;
}

/*--------------------------------------------------------------*
 |                 DECODE_RESOURCECONTROL_REQUEST               |
 | Function:                                                    |
 |    Decode the resource control request.                      |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    response : Address to store the response code.            |
 |    report   : Array of report information.                   |
 |    message  : Pointer to store the report message string.    |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void Decode_ResourceControl_Request(SESSION *session,
                                           int       *response,
                                           long      report[],
                                           STRING    *message)
{
    DATA_DIR *PResource;
    DATA_DIR *PReport;

    PResource = get_begin(session->pdu, ResourceControlRequest);
    if (PResource == NULL_DIR) return;

    get_int(PResource, ResponseRequired, response, REQUIRED);

    PReport = get_node(PResource, ResourceReport, OPTIONAL);

    resource_report_stats(PReport, report, message);
}

/*--------------------------------------------------------------*
 |                 ENCODE_RESOURCECONTROL_RESPONSE              |
 | Function:                                                    |
 |    Encode the resource control response                      |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    complete : Complete the request.                          |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void Encode_ResourceControl_Response(SESSION  *session,
                                     int complete)
{
    DATA_DIR *PResource;

    PResource = put_begin(session->pdu, ResourceControlResponse);
    if (PResource == NULL_DIR) return;

    if (session->reference)
        put_cstring(PResource, ReferenceId, session->reference);

    put_flag(PResource, ContinueFlag, complete);

    /* If you do not complete the search then keep result set */

    if (complete == 0)
        put_flag(PResource, ResultSetWanted, 1);  /* Yes */
}

/*--------------------------------------------------------------*
 |                 REPLY_RESOURCE_CONTROL_PDU                   |
 | Function:                                                    |
 |    Reply to a resource control request.                      |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int reply_resource_control_pdu(SESSION *session)
{
    STRING   message;
    long     report[MAX_RESOURCE_REPORT_BIB1];
    int      response;
    int      complete = 1;  /* By default complete */

    STRING_SET(message, 0, NULL);

    Decode_ResourceControl_Request(session, &response, report, &message);

    /* If a resource routine exists then call it instead of the default. */

    if (session->resource_control)
        (*session->resource_control)(session, report, &complete, &message);
    else
        resource_report_bib1_output(session, report, &message);

    if (response)
        Encode_ResourceControl_Response(session, complete);
    else
        session->pdu->fldid = 0;     /* Do NOT send a response pdu */

     return OK;
}
