/*                              RPN.C                              */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

/*
    The RPN routines takes a search query with logical operators
    and converts it to a Postfix tree (ie. Reverse Polish Notation
    according to ISO).

    The search query syntax is as follows:
        <query-expression> ::= <operand> [<logical-operator> <operand> ...]

    Where <operand> is either a result set name or else an
    attribute spec followed by a search term or another query
    expression in parentheses.
        <operand> ::= <result-set-name>
                      <attribute-spec> [<relational-operator>] <term>
                      '(' <query-expression> ')'
    The result set name is indicated by a special character as
    the first character (the constant SET_INDICATOR) before the
    name. Ie.
        <result-set-name> ::= <SET_INDICATOR><string>
        <SET_INDICATOR> ::= <any-single-character>

    So if the special character where a '@' and the result set
    name where 'set1' then the query to retrieve set1 would be
    something like
         @set1

    The attribute-spec specifies the index attributes and
    can be used to set the Use, Truncation, Structure,
    Completeness, and Positional attribute parameters.

       <attribute-spec> ::=  <index-name>[<attributes-info>...]

    Where <index-name> is any valid name for an index which
    may not contain any special characters (space, parenthesis,
    equal-sign, greater-than-sign, less-than-sign, slash, etc).

       <index-name> ::= <non special characters>

    The attribute information can be a set of attributes which
    are seperated from one another by a slash.  The attribute
    type is seperated from the attribute value by a hyphen ('-').
    The attributes type and value may be abbreviated.  The first
    match of will be used.

       <attribute-info> ::= /<attribute-type>:<attribute-value>
       <attribute-type> ::= USE
                            RELATION         <-- May be abbreviated
                            POSITION         <-- to 1 character.
                            STRUCTURE
                            TRUNCATE
                            COMPLETENESS
       <attribute-value> ::= <valid attribute for type>
                             <number>

    Both the attribute information separator, ie. a slach ('/') and
    the type-value seperator, ie. a colon (':') may be changed.

    Here are a few examples:

        AUTHOR/STRUC:PHRASE/POS:FIRST_IN_SUBFIELD

    Please note that  

	
    The database will associate defaults with USE Attributes
    so that an attribute spec will be fully specified
    without requiring the user to do so by hand.

    NO whitespace should appear anywhere in an
    attribute specification.  This is because the parser
    considers the attribute spec to be one token,
    and cuts it apart at the slashes.
    The search term must be a simple string
    without any special characters (like "=()<>", tabs or blanks).
    if you want to include these characters just put quotes marks
    around the search term.  If you want a quote mark within a
    term surrounded by quote marks you must double the quote mark
    within the quotes.  For example either query below
         phrase 'hogan''s hero''s'
    or
         phrase "hogan's hero's"
    would search the index "phrase" for the string "hogan's hero's".
    A quote mark may be either a single quote (') or a double
    quote ("").

    The relational operator is optional and may be any of the
    operators listed below.
            =          equal to
            <          less than
            <=         less than or equal to
            >          greater than
            >=         greater than or equal to
            ~=         not equal to
            ^=         not equal to
    You must place them between the index and the term, for example
            date > 4/11/1991
            age >= 62
            gender ~= male
    If you do not include a relational operator in a query then
    the relational operator defaults to "equal to".  So the query
            title technology
    is equivalent to
            title = technology
    If you want to use any of the characters in a search term
    you must surround the term with quotes.
            expression = 'one < two'
            symbol "="
    otherwise the parsing will fail.

    The logical operators may be either AND/OR/AND-NOT and must
    be followed by another operand.

    When using a logical operator you can also skip repeating
    the attribute spec if you want.  So the search
         title computer and title research or title network
    could be entered as
         title computer and research or network

    You may also combine searches between result set names
    and index searches, for example
         @set4 and title workstation and @set5

    The term which you search may also be surrounded in parethesis.
    So the following is acceptable:

         title (object oriented) or (visual programming)

    It should be noted that the brackets tie the terms to the
    index, so the following are interpreted to be the same:

         title (visual programming and (author knuth))

         title "visual programming" and title "author knuth"

    No index names are interpreted while inside the scope of the
    brackets when tied to an index.
    
    The procedures below use a simple recursive decent parser
    to convert the query into the postfix tree.  A lot of the
    queries above make more sense if you put the word "find"
    in front to show the queries as "find" commands.
*/

#include <stdlib.h>

#include "attrcons.h"
#include "attrinfo.h"
#include "attrlist.h"
#include "berutil.h"
#include "constant.h"
#include "database.h"
#include "msg.h"
#include "pdutags.h"
#include "portinfo.h"
#include "putget.h"
#include "utility.h"
#include "z3950.h"

#define MAX_STRING_SIZE        256

#define LOGICAL_NOP            0
#define LOGICAL_AND            1
#define LOGICAL_OR             2
#define LOGICAL_AND_NOT        3

/*
 * Allow a search command to seperate search terms with a logical
 * operator.  For example "find (tw big) or bird" is equivalent to
 * "find (tw big) or (bird)".
 */

static INDEX previous_index;

DATA_DIR *parse_rpn_expression(DATABASE_REFERENCE db,
                               DATA_DIR *rpn,
                               char **query,
                               int scan_index);

/*--------------------------------------------------------------*
 |                    CHECK_LOGICAL                             |
 | Function:                                                    |
 |    Check if the string is a logical operator.                |
 | Parameters:                                                  |
 |    word   : Pointer to string to check.                      |
 |    length : Length of string to check.                       |
 | Returns:                                                     |
 |    Number indicating logical operator (0 if none).           |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int check_logical(word, length)
    char *word;
    int length;
{
    char operator[12];
    int op = LOGICAL_NOP;

    if (length <= 7)  /* length <= strlen("AND-NOT") */
        {
            uppercase(operator, word, length);

            if (strcmp(operator,"OR") == 0 || strcmp(operator,"||") == 0)
                op = LOGICAL_OR;
            else if (strcmp(operator,"AND") == 0 || strcmp(operator,"&&") == 0)
                op = LOGICAL_AND;
            else if (strcmp(operator,"AND-NOT") == 0)
                op = LOGICAL_AND_NOT;
        }
    return op;
}

/*--------------------------------------------------------------*
 |                    LOGICAL_OPERATOR                          |
 | Function:                                                    |
 |    Parse the logical operator from a string.                 |
 | Parameters:                                                  |
 |    query  : Address of query string to parse.                |
 | Returns:                                                     |
 |    Number indicating logical operator (0 if none).           |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    If a logical operator occured update the query string     |
 |    to point past the logical operator. A logical operator    |
 |    may not occur in quotes.                                  |
 *--------------------------------------------------------------*/

int logical_operator(query)
    char **query;
{
    char *token;
    int  length;
    int  logical;

    length = tokenize(query, &token, SPECIAL_CHARACTERS);

    logical = check_logical(token, length);
    if (logical == LOGICAL_NOP) *query = token;

    return logical;
}

/*--------------------------------------------------------------*
 |                    RELATIONAL_OPERATOR                       |
 | Function:                                                    |
 |    Parse the relational operator from a string               |
 | Parameters:                                                  |
 |    query  : Address of query string to parse.                |
 | Returns:                                                     |
 |    Number indicating relational operator (0 if none).        |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    If a relational operator occured update the query string  |
 |    to point past the relational operator.                    |
 *--------------------------------------------------------------*/

int relational_operator(query)
    char **query;
{
    char *operator = *query;
    int op = RELATIONAL_NOP;

    /* Skip over blanks */

    while (*operator == BLANK || *operator == TAB) operator++;

    if (*operator == EOS)
        return op;

    switch (*operator++)
        {
            case '=':
                {
                    *query = operator;
                    op = EQUAL;
                }

                break;

            case '~':
            case '^':
                if (*operator == '=')
                    {
                        *query = ++operator;
                        op = NOT_EQUAL;
                    }
                break;

            case '<':
                if (*operator == '=')
                    {
                        *query = ++operator;
                        op = LESS_THAN_OR_EQUAL;
                    }
                else
                    {
                        *query = operator;
                        op = LESS_THAN;
                    }
                break;

            case '>':
                if (*operator == '=')
                    {
                        *query = ++operator;
                        op = GREATER_OR_EQUAL;
                    }
                else
                    {
                        *query = operator;
                        op = GREATER_THAN;
                    }
                break;

            default:
                break;
        }
    return op;
}

/*--------------------------------------------------------------*
 |                    PARENTHESIS                               |
 | Function:                                                    |
 |    Parse the parenthesis from the string.                    |
 | Parameters:                                                  |
 |    query  : Address of query string to parse.                |
 |    paren  : The parenthesis character.                       |
 | Returns:                                                     |
 |    1 if parenthesis found (0 if none).                       |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    If the parenthesis occured update the query string to     |
 |    point past the parenthesis.  Skip leading blanks.         |
 *--------------------------------------------------------------*/

int parenthesis(query, paren)
    char **query;
    char paren;
{

    while (**query == BLANK || **query == TAB) (*query)++;

    if (paren == **query)
        {
            (*query)++;
            return 1;
        }
    return 0;
}

/*--------------------------------------------------------------*
 |                    PACKAGE_RPN                               |
 | Function:                                                    |
 |    Package the tree RPN structure.                           |
 | Parameters:                                                  |
 |    logical  : The logical operator.                          |
 |    left     : The left node of the tree.                     |
 |    right    : The right node of the tree.                    |
 | Returns:                                                     |
 |    Pointer to the RPN structure.                             |
 | Description:                                                 |
 |                                                              |
 |    Allocate and fill in the following ASN.1 description      |
 |                                                              |
 |    RPNStructure ::=                                          |
 |       CHOICE {                                               |
 |           [0] Operand,                                       |
 |           [1] IMPLICIT SEQUENCE {                            |
 |                   RPNStructure,                              |
 |                   RPNStructure,                              |
 |                   Operator                                   |
 |               }                                              |
 |    }                                                         |
 |                                                              |
 | Note:                                                        |
 |    If a logical nop then create a terminating node otherwise |
 |    create a branching node.                                  |
 *--------------------------------------------------------------*/

DATA_DIR *package_rpn(int logical,
                      DATA_DIR *right,
                      DATA_DIR *left)
{
    DATA_DIR *PRpnStructure;
    DATA_DIR *POperator;

    /* Check for a valid expression */

    if (logical == LOGICAL_NOP)
        return right;

    if (left == NULL_DIR)
        return NULL_DIR;

    /* Insert node of RPN expression tree */

    PRpnStructure = put_newnode(right, RPNStructure, ASN1_CONTEXT);

    if (PRpnStructure)
        {
            POperator = put_node(PRpnStructure, Operator, ASN1_CONTEXT);

            put_cstring(POperator, logical - 1, "");
        }
    return PRpnStructure;
}

/*--------------------------------------------------------------*
 |                    PARSE_INDEX                               |
 | Function:                                                    |
 |    Parse the index and set the attributes for the index.     |
 | Parameters:                                                  |
 |    db          : The reference to database with indexes.     |
 |    name        : The index name.                             |
 |    length      : The length of the index name.               |
 |    attributes  : The attributes of the index if found.       |
 | Returns:                                                     |
 |    An index reference if a match was found, otherwise NULL.  |
 | Description:                                                 |
 |    Check if the index is a valid for the database.  Also     |
 |    set any Z39.50 attributes which are specified.  There     |
 |    may be multiple attributes set.                           |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int parse_index(DATABASE_REFERENCE db,
                char *text,
                int  length,
                int  attributes[MAX_ATTRIBUTES])
{
    INDEX_REFERENCE index;
    char *index_name;
    char *end_text = text + length;
    char end_char;
    int size;
    int result;

    end_char = *end_text; 
    *end_text = EOS;

    size = tokenize(&text, &index_name, ATTRIB_DELIM);
  
    if ((index = database_index(db, index_name, size)))
        {
            /* Set the attributes */

            COPY(attributes, index_attributes(index), sizeof(int) * MAX_ATTRIBUTES);

            if (*text != EOS)
                {
                    uppercase(text, text, length - size);

                    result = attribute_info_parse(attributes,
                                                  text,
                                                  ATTRIB_ASSIGN,
                                                  ATTRIB_DELIM);
                }
	    else
		result = OK;
        }
    else
        {
            uppercase(index_name, index_name, length);

            result = attribute_index_parse(attributes,
                                           index_name,
                                           ATTRIB_ASSIGN,
                                           ATTRIB_DELIM);
        }

    *end_text = end_char;

    return (result == OK) ? TRUE : FALSE;
}

/*--------------------------------------------------------------*
 |                    PARSE_OPERAND                             |
 | Function:                                                    |
 |    Parse the operand section.                                |
 | Parameters:                                                  |
 |    db     : Database reference.                              |
 |    PFind  : Pointer to ASN.1 representation of operand.      |
 |    query  : Address of query string to parse.                |
 |    scan_index : Scan for index name (if FALSE use previous   |
 |                 index only).                                 |
 | Returns:                                                     |
 |    Pointer to RPN structure or a null pointer.               |
 | Description:                                                 |
 |    Parse the query string for an operand which may be        |
 |    either a result set name, an index followed by a search   |
 |    term, or a query expression surrounded by parenthesis.    |
 | Note:                                                        |
 |    The query string is updated to where this routine         |
 |    finished parsing.                                         |
 |                                                              |
 |    This routine recusively calls parse_rpn to parse an       |
 |    expression in parentheses.                                |
 *--------------------------------------------------------------*/

DATA_DIR *parse_operand(DATABASE_REFERENCE db,
                        DATA_DIR *PFind,
                        char **query,
			int scan_index)
{
    DATA_DIR *POperand;
    INDEX_REFERENCE index;
    int  operator;
    int  length;
    int  words;
    char *token;
    char *term;

    /* Handle parenthesis around an operand */

    if (parenthesis(query, LEFT_PAREN))
        {
            DATA_DIR *rpn;

            if ((rpn = parse_rpn_expression(db, PFind, query, scan_index)))
                {
                    /* A right parenthesis must exist */

                    if (!parenthesis(query, RIGHT_PAREN))
                        {
                            msg_make(MISSING_PAREN);
                            rpn = NULL_DIR;
                        }
                }
            return rpn;
        }

    /* Find next token */

    length = tokenize(query, &token, SPECIAL_CHARACTERS);

    if (length < 0)                 /* Error in getting token */
        return NULL_DIR;
    else if (length == 0)           /* No token was found */
        {
            msg_make(MISSING_INDEX);
            return NULL_DIR;
        }

    /* Check if a set name */

    if (*token == SET_INDICATOR && scan_index)
        {
            STRING setid;

            if (length == 1)
                {
                    msg_make(BAD_SETID);
                    return NULL_DIR;
                }

            STRING_SET(setid, --length, ++token);

            POperand = put_node(PFind, Operand, ASN1_CONTEXT);

            if (POperand)
                put_string(POperand, ResultSetId, &setid);
        }
    else
        {
	    STRING sterm;
            int attributes[MAX_ATTRIBUTES];
            int quoted = 0;
            int quoted_length;

            if (scan_index && parse_index(db, token, length, attributes))
                {
                    previous_index.name[0] = TRUE;
                    COPY(previous_index.attributes,
			 attributes,
			 sizeof(int) * MAX_ATTRIBUTES);
                }
            else
                {
                    if (previous_index.name[0] == FALSE)
                        {
                            char bad_index[MAX_STRING_SIZE];

                            uppercase(bad_index, token, length);
                            msg_make(BAD_INDEX, bad_index);
                            return NULL_DIR;
                        }
                    index = &previous_index;
                    COPY(attributes,
			 previous_index.attributes,
			 sizeof(int) * MAX_ATTRIBUTES);

                    *query = token;
                }

	    /* Check if a parenthesis */

            if (parenthesis(query, LEFT_PAREN))
		{
		    DATA_DIR *rpn;

		    if ((rpn = parse_rpn_expression(db, PFind, query, FALSE)))
			{
			    /* A right parenthesis must exist */
			    
			    if (!parenthesis(query, RIGHT_PAREN))
				{
				    msg_make(MISSING_PAREN);
				    rpn = NULL_DIR;
				}
			}
		    return rpn;
		}
	   
            POperand = put_node(PFind, Operand, ASN1_CONTEXT);
            if (POperand == NULL_DIR) return NULL_DIR;

            /* Check if a relational operator is there */

            operator = relational_operator(query);

            if (operator != RELATIONAL_NOP)
                attributes[RELATION] = operator;

            /* Now get the term to find in the search */

            for (words = 0; *token; words++)
                {
                    length = tokenize(query, &token, SPECIAL_CHARACTERS);

                    if (length > 0)
                        {
                            if (QUOTED(token))
                                {
                                    quoted++;
                                    quoted_length = length - 2;
                                }
                            else if (check_logical(token, length))
				{
				    int ilen = 0;
				    
				    /*
				     * A phrase index may contain a logical operator 
				     * (ie. AND) as a part of the phrase.  So for a
				     * phrase index check if the next token is the
				     * name of an index.  If the token is the name of
				     * an index then treat the AND as a logical operator
				     * otherwise treat as part of the phrase.
				     */
			
				    if (attributes[STRUCTURE] == PHRASE)
					{
					    char *next = token + length;
					    char *iname;
				    
					    ilen = tokenize(&next, 
							    &iname, 
							    SPECIAL_CHARACTERS);

					    if (ilen > 0)
						{
						    int iattr[MAX_ATTRIBUTES];
					    
						    if (parse_index(db, iname, ilen, iattr))
							ilen = 0;
						}
					}

				    /* If ilen equal to zero treat AND as logical operator */

				    if (ilen == 0)
					{
					    *query = token;
					    break;
					}
				}
                        }
                    else if (length == 0)
                        break;
                    else
                        return NULL_DIR;  /* Error in getting token */

                    if (words == 0) term = token;
                }

            /* Must have a term to search against, no term was found */

            if (words == 0)
                {
                    msg_make(MISSING_TERM);
                    return NULL_DIR;
                }

            if (quoted == 1)    /* Pass quoted strings asis */
                {
                   length = quoted_length;
                   term++;
                }
            else
                {
                   /* Strip trailing blanks of term */

                   for (token--; *token == BLANK || *token == TAB; token--);

                    /* Check for truncation and calculate new length */

                    if (strchr(TRUNCATION_MARK,*token) != NULL)
                        attributes[TRUNCATION] = RIGHT_TRUNCATION;
                    else if (!quoted && strchr(NO_TRUNCATION_MARK, *token) != NULL)
                        attributes[TRUNCATION] = NO_TRUNCATION;
                    else
                        token++;

                    length = (int) (token - term);
                }

            STRING_SET(sterm, length, term);

            /* Get operand for attributes */

            if (attributelist_and_term(POperand,
                                       attributes,
                                       &sterm,
                                       words))
                POperand = NULL_DIR;
        }
    return POperand;
}

/*--------------------------------------------------------------*
 |                    PARSE_RPN_EXPRESSION                      |
 | Function:                                                    |
 |    Parse the rpn query section.                              |
 | Parameters:                                                  |
 |    db     : Database reference.                              |
 |    PFind  : Pointer to ASN.1 representation of expression.   |
 |    query  : Address of query string to parse.                |
 |    scan_index : Scan for index name (if FALSE use previous   |
 |                 index only).                                 |
 | Returns:                                                     |
 |    Pointer to RPN structure or a null pointer.               |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    Parse the query string for an query expression and        |
 |    update the query string to point past the parsed query    |
 |    expression or to where the parsing failed.                |
 *--------------------------------------------------------------*/

DATA_DIR *parse_rpn_expression(DATABASE_REFERENCE db,
                               DATA_DIR *PFind,
                               char **query,
			       int scan_index)
{
    DATA_DIR *PRpn;
    int  logical;

    if ((PRpn = parse_operand(db, PFind, query, scan_index)) == NULL_DIR)
        return NULL_DIR;

    /* If a logical relationship then another term must be found */

    while ((logical = logical_operator(query)))
        {
            PRpn = package_rpn(logical,
                               PRpn,
                               parse_operand(db, PFind, query, scan_index));

            if (PRpn == NULL_DIR) break;
        }

    if (**query != EOS && **query != RIGHT_PAREN)
        {
            msg_make(NOT_LOGICAL);
            PRpn = NULL_DIR;
        }

    return PRpn;
}

/*--------------------------------------------------------------*
 |                    PARSE_RPN                                 |
 | Function:                                                    |
 |    Parse the rpn query section.                              |
 | Parameters:                                                  |
 |    db     : Database reference.                              |
 |    PFind  : Pointer to ASN.1 representation of RPN.          |
 |    query  : Address of query string to parse.                |
 | Returns:                                                     |
 |    Pointer to RPN structure or a null pointer.               |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    This basically just sets initializes the previous index.  |
 *--------------------------------------------------------------*/

DATA_DIR *parse_rpn(DATABASE_REFERENCE db,
                    DATA_DIR *PFind,
                    char **query)
{
    /* No previous index */

    previous_index.name[0] = FALSE;

    return parse_rpn_expression(db, PFind, query, TRUE);
}
