/*                            SCANINFO.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include "attrlist.h"
#include "constant.h"
#include "database.h"
#include "msg.h"
#include "pdutags.h"
#include "putget.h"
#include "scaninfo.h"
#include "session.h"
#include "transmit.h"
#include "utility.h"

#define MAX_STRING_SIZE 80

enum scan_request_pdu_tags
{
    Attributes = 1,
    DatabaseId = 3,
    TermListAndStartPoint,
    StepSizeRequested,
    NumberOfTermsRequested,
    PreferredPositionInResponse
};

enum scan_response_pdu_tags
{
    StepSizeResponse = 3,
    ScanStatus,
    NumberOfEntriesReturned,
    PositionOfTerm,
    Entries
};

/*--------------------------------------------------------------*
 |                   ENCODE_SCAN_REQUEST                        |
 | Function:                                                    |
 |    Encode the scan request.                                  |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    find     : Pointer to the find (search) query.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void Encode_Scan_Request(SESSION  *session,
                                char     *index,
                                char     *term,
                                int      position,
				int      step_size,
                                int      number_of_terms)
{
    DATA_DIR *PScan;
    DATA_DIR *dbname;
    DATA_DIR *termlist;
    DATA_DIR *tl_attribs;
    DATA_DIR *tla_sequence;
    STRING scan_term;
    int attributes[MAX_ATTRIBUTES];
    int result;
    
    PScan = put_begin(session->pdu, ScanRequest);
    if (PScan == NULL_DIR) return;

    /* Search for the name of the index */

    if (parse_index(session->db, index, strlen(index), attributes) == FALSE)
        {
	    (void) msg_make(BAD_INDEX, index);
	    return;
	}
    
    if (session->reference)
        put_cstring(PScan, ReferenceId, session->reference);

    dbname = put_node(PScan, DatabaseId, ASN1_CONTEXT);
    put_cstring(dbname, DatabaseName, database_name(session->db));

    if (strcmp(session->target_name, "OCLC") == 0)
        {
            termlist = put_node(PScan, TermListAndStartPoint, ASN1_CONTEXT);
            if (!termlist) return;

            tl_attribs = put_node(termlist, Attributes, ASN1_CONTEXT);

            tla_sequence = put_node(tl_attribs, SEQUENCE, ASN1_UNIVERSAL);

            (void) attributelist(tla_sequence, attributes);

            put_cstring(termlist, Term, term);
        }
    else
        {
            STRING_SET(scan_term, strlen(term), term);

            attributelist_and_term(PScan, attributes, &scan_term, 0);
        }

    put_int(PScan, StepSizeRequested, step_size);

    put_int(PScan, NumberOfTermsRequested, number_of_terms);

    put_int(PScan, PreferredPositionInResponse, position);
}

/*--------------------------------------------------------------*
 |                   DECODE_SCAN_RESPONSE                       |
 | Function:                                                    |
 |    Decode the search response.                               |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    records  : Address to store number of returned records.   |
 |    status   : Address to store the search status.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void Decode_Scan_Response(SESSION *session,
                          SCAN_INFO *scan)
{
    DATA_DIR *PScan;
    DATA_DIR *PList;
    int stepsize;
    int status;
    int returned;
    int position;

    PScan = get_begin(session->pdu, ScanResponse);
    if (PScan == NULL_DIR) return;

    /* Restart at the next record */

    get_int(PScan, StepSizeResponse, &stepsize, OPTIONAL);

    get_int(PScan, ScanStatus, &status, REQUIRED);

    get_int(PScan, NumberOfEntriesReturned, &returned, REQUIRED);

    get_int(PScan, PositionOfTerm, &position, OPTIONAL);

    scaninfo_set_stepsize(scan, stepsize);
    scaninfo_set_status(scan, status);
    scaninfo_set_returned(scan, returned);
    scaninfo_set_position(scan, position);
    scaninfo_set_dbname(scan, database_name(session->db));

    PList = get_node(PScan, Entries, OPTIONAL);

    scaninfo_extract(scan, PList);
}

/*--------------------------------------------------------------*
 |                   SEND_SEARCH_PDU                            |
 | Function:                                                    |
 |    Send the search request to the server.                    |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    find     : Pointer to the find (search) query.            |
 |    info     : Pointer to the record information.             |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int send_scan_pdu(SESSION   *session,
                  char      *index,
                  char      *term,
                  int       position,
		  int       step_size,
                  int       number_of_terms,
                  SCAN_INFO *scan)
{
    Encode_Scan_Request(session, index, term, position, step_size, number_of_terms);
    if (MSG_TEST == OK)
        {
            if (transmit_pdu(session,
                             session->pdu,
                             ScanRequest,
                             WAIT_SCAN) == OK)
                {
                    Decode_Scan_Response(session, scan);
                }
        }
    return MSG_TEST;
}
