/*                            SCANINFO.C                         */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdlib.h>
#include <stdio.h>

#ifdef ANSI
#include <memory.h>
#endif

#include "constant.h"
#include "diag.h"
#include "msg.h"
#include "pdutags.h"
#include "putget.h"
#include "record.h"
#include "scaninfo.h"
#include "session.h"

#define MAX_NAMES           10  /* MAX_NAME_SIZE defined in database.h */

enum list_entries_tags
{
    ScanEntries = 1,
    NonSurrogateDiagnostics
};

enum entry_tags
{
    TermInfo = 1,
    SurrogateDiagnostic
};

enum term_info_tags
{
    ScanTerm = 1,
    GlobalOccurrences,
    OccurrenceByAttributes
};

enum occurrence_tags
{
    Attributes = 1,
    Global,
    ByDatabase
};

enum database_tags
{
    Num = 1
};

/*--------------------------------------------------------------*
 |                    LOOKUP_NAME                               |
 | Function:                                                    |
 |    Lookup the name of the database.                          |
 | Parameters:                                                  |
 |    info     : Pointer to record information.                 |
 |    name     : Pointer to database name.                      |
 | Returns:                                                     |
 |    Pointer to the database name.                             |
 | Description:                                                 |
 |    This routine keeps a table of the database names          |
 |    referenced by the returned records.  Once the table is    |
 |    filled it just wraps and starts over again.               |
 |                                                              |
 |    The reason a table is used is that only the first record  |
 |    from a particular database needs to specify the name.     |
 |    Subsequent records with no name specified are assumed to  |
 |    have been returned from the same database.  Therefore if  |
 |    no name is passed (length <= 0) then the name of latest   |
 |    entry in the table is returned as the database name.      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static char *lookup_name(SCAN_INFO *info,
			 STRING *name)
{
    int  count;
    int  length;

    /* Table of database names */

    static char db_name[MAX_NAMES][MAX_NAME_SIZE+1];
    static int  db_index = -1;

    /* Initialize if called for first time */

    if (db_index < 0)
        {
            memset(db_name[0], 0, MAX_NAMES * (MAX_NAME_SIZE+1));
            strcpy(db_name[++db_index], info->dbname);
        }

    if (name && name->length > 0)
        {
            /* Check if name already exists in the table */

            for (count = 0; count < MAX_NAMES; count++)
                {
                     if (db_name[count][0] == EOS) break;

                     if (string_ccompare(name, db_name[count]) == 0)
                         return db_name[count];
                }

            /* Add the name to the table, use in a circular */
            /* fashion, to reuse the oldest entry.         */

            if (++db_index >= MAX_NAMES) db_index = 0;
            length = MIN(name->length, MAX_NAME_SIZE);
            COPY(db_name[db_index], name->location, length);
            db_name[db_index][length] = EOS;

            return db_name[db_index];
        }

    return db_name[db_index];
}

/*--------------------------------------------------------------*
 |                    EXTRACT_SCAN                              |
 | Function:                                                    |
 |    Extract a record from the ASN.1 and add to the series.    |
 | Parameters:                                                  |
 |    external  : Pointer to the data nodes.                    |
 |    series    : Pointer to the record series.                 |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Adds the record to the series.  Currently recognizes both |
 |    INFO-1 and US-Marc records.                               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void extract_scan(DATA_DIR *node,
                  SERIES *series)
{
    SCAN *scan;

    /* Check if available space in series */

    scan = series_next(series);
    if (scan == (SCAN *) NULL) return;

    /* Due to different implementations (based on old version) try alternatives */

    if (find_tag(node, ScanTerm) == NULL_DIR)
        get_string(node, Term, &scan->term, REQUIRED);
    else
        get_string(node, ScanTerm, &scan->term, REQUIRED);

/*
    get_attributes(node, SuggestedAttributes, &scan->attributes, OPTIONAL);
*/
    get_int(node, GlobalOccurrences, &scan->occurences, OPTIONAL);
/*
    get_node(node, OccurrenceByAttributes, &scan->occbyattr, OPTIONAL);
*/
}

/*--------------------------------------------------------------*
 |                 EXTRACT_DATABASE_SCANS                       |
 | Function:                                                    |
 |    Extract all of the database records.                      |
 | Parameters:                                                  |
 |    info     : Pointer to the record information.             |
 |    record   : Pointer to the data nodes.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Extract the database records and diagnostics and add to   |
 |    the series.                                               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void extract_scan_records(SCAN_INFO *info,
				     DATA_DIR *list)
{
    char *name;

    name = lookup_name(info, NULL);

    while (list)
        {
            if (list->fldid == TermInfo)
                extract_scan(list->ptr.child, info->records);
            else if (list->fldid == SurrogateDiagnostic)
                diagnostic_extract(list->ptr.child, info->errors);

            list = list->next;
        }
}

/*--------------------------------------------------------------*
 |                    SCAN_EXTRACT                              |
 | Function:                                                    |
 |    The records returned are extracted.                       |
 | Parameters:                                                  |
 |    info     : Pointer to the record information.             |
 |    PRecords : Pointer to the data nodes.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void scaninfo_extract(SCAN_INFO  *info,
		      DATA_DIR *PEntries)
{
    DATA_DIR *List;

    series_set_total(info->records, 0);
    series_set_total(info->errors, 0);

    List = get_node(PEntries, NonSurrogateDiagnostic, OPTIONAL);
    if (List)
        diagnostic_extract(List, info->errors);
    else
	{
	    List = get_node(PEntries, ScanEntries, OPTIONAL);
	    
	    if (List)
		extract_scan_records(info, List);
	}
}

static void scan_display(SESSION *session,
			 SCAN *scan,
			 int position,
			 int count)
{
    if (scan->term.length != 0 && scan->term.location != NULL)
	{
#ifndef MARKER
	    printf(" %*.*s (%d occurrences)\n",
		   (int) scan->term.length,
		   (int) scan->term.length,
		   scan->term.location,
		   scan->occurences);
#endif
#ifdef MARKER
            printf("%d %d %*.*s\n",
		   count - position,
		   scan->occurences,
		   (int) scan->term.length,
		   (int) scan->term.length,
		   scan->term.location);
#endif
	}
    else
	printf("Missing scan term\n");
}

/*--------------------------------------------------------------*
 |                    SCAN_EXTRACT                              |
 | Function:                                                    |
 |    The records returned are extracted.                       |
 | Parameters:                                                  |
 |    info     : Pointer to the record information.             |
 |    PRecords : Pointer to the data nodes.                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void scaninfo_output(SESSION *session,
		     SCAN_INFO  *info)
{
    int    i;
    
    for (i = 1; i <= series_get_total(info->records); i++)
	{
	    scan_display(session,
			 (SCAN *) series_item(info->records, i),
			 scaninfo_get_position(info),
			 i);
	}
}

/*--------------------------------------------------------------*
 |                   SCANINFO_NEW                               |
 | Function:                                                    |
 |    Allocate a new instance of scan information.              |
 | Parameters:                                                  |
 |    None                                                      |
 | Returns:                                                     |
 |    NULL if failure otherwise pointer to new instance.        |
 | Description:                                                 |
 |    Allocate memory and initialize.                           |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

SCAN_INFO *scaninfo_new()
{
    SCAN_INFO *info;
    
    info = (SCAN_INFO *) calloc(1, sizeof(SCAN_INFO));
    
    if (info != NULL)
        {
	    info->records = series_new(sizeof(SCAN));
	    if (info->records == (SERIES *) NULL)
		scaninfo_dispose(&info);
	    else
		{
		    info->errors = series_new(sizeof(RECORD));
		    if (info->errors == (SERIES *) NULL)
			scaninfo_dispose(&info);
		}
	}
    else
	msg_make(NO_CORE,"scan info");
    
    return info;
}

/*--------------------------------------------------------------*
 |                    SCANINFO_DISPOSE                          |
 | Function:                                                    |
 |   Dispose of the scan information.                           |
 | Parameters:                                                  |
 |    info    : Pointer to the scan information.                |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Free any allocated memory.                                |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void scaninfo_dispose(SCAN_INFO **info)
{
    if (info != NULL)
        {
            if ((*info) != NULL)
                {
                    series_dispose(&(*info)->records);
		    
		    series_dispose(&(*info)->errors);
		    
                    free(*info);

                    *info = NULL;
                }
        }
}


























