/*                            SEARCH.C                             */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include "constant.h"
#include "msg.h"
#include "pdutags.h"
#include "place.h"
#include "putget.h"
#include "query.h"
#include "recinfo.h"
#include "transmit.h"

/*--------------------------------------------------------------*
 |                   ENCODE_SEARCH_REQUEST                      |
 | Function:                                                    |
 |    Encode the search request.                                |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    find     : Pointer to the find (search) query.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void Encode_Search_Request(SESSION  *session,
                                  char     *find,
				  int      syntax)
{
    DATA_DIR *PFind;
    DATA_DIR *dbname;

    PFind = put_begin(session->pdu, SearchRequest);
    if (PFind == NULL_DIR) return;

    if (session->reference)
        put_cstring(PFind, ReferenceId, session->reference);

    put_int(PFind, SmallSetUpperBound, session->set_small_limit);

    put_int(PFind, LargeSetLowerBound, session->set_large_limit);

    put_int(PFind, MediumSetPresentNumber, session->set_medium_limit);

    put_flag(PFind, ReplaceIndicator, session->set_replace);

    put_cstring(PFind, ResultSetName, session->current_set);

    dbname = put_node(PFind, DatabaseNames, ASN1_CONTEXT);
    put_cstring(dbname, DatabaseName, database_name(session->db));

    place_preferred_record_syntax(PFind, syntax);

    parse_query(session->db, PFind, find);
}

/*--------------------------------------------------------------*
 |                   DECODE_SEARCH_RESPONSE                     |
 | Function:                                                    |
 |    Decode the search response.                               |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    records  : Address to store number of returned records.   |
 |    status   : Address to store the search status.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void Decode_Search_Response(SESSION *session,
                            RECORD_INFO *info)
{
    DATA_DIR *PFind;
    unsigned char search_status;
    int  records;
    int  status;

    PFind = get_begin(session->pdu, SearchResponse);
    if (PFind == NULL_DIR) return;

    /* Restart at the next record */

    get_int(PFind, ResultCount, &session->result_count, REQUIRED);

    get_int(PFind, NumberOfRecordsReturned, &records, REQUIRED);

    get_int(PFind, NextResultSetPosition, &session->next_record, REQUIRED);
    if (session->next_record == 0) session->next_record = 1;

    get_flag(PFind, SearchStatus, &search_status, REQUIRED);

    if (search_status)    /* If search successful */
        get_int(PFind, PresentStatus, &status, OPTIONAL);
    else                  /* If search failed */
        {
            get_int(PFind, ResultSetStatus, &status, REQUIRED);
            status = -status;
        }

    /* Set the record information */

    recinfo_set_position(info, 1);
    recinfo_set_result(info,session->result_count);
    recinfo_set_returned(info,records);
    recinfo_set_status(info,status);
    recinfo_set_dbname(info,database_name(session->db));
    recinfo_extract(info, PFind);
}

/*--------------------------------------------------------------*
 |                   SEND_SEARCH_PDU                            |
 | Function:                                                    |
 |    Send the search request to the server.                    |
 | Parameters:                                                  |
 |    session  : Pointer to the session information.            |
 |    find     : Pointer to the find (search) query.            |
 |    info     : Pointer to the record information.             |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int send_search_pdu(SESSION *session,
                    char    *find,
		    int     syntax,
                    RECORD_INFO *info)
{
    Encode_Search_Request(session, find, syntax);
    if (MSG_TEST == OK)
        {
            if (transmit_pdu(session,
                             session->pdu,
                             SearchRequest,
                             WAIT_SEARCH) == OK)
                {
                    Decode_Search_Response(session, info);
                }
        }
    return MSG_TEST;
}







