/*                             SERIES.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stddef.h>
#include <stdlib.h>

#include "constant.h"
#include "msg.h"
#include "portinfo.h"
#include "series.h"

#define SERIES_SIZE     15

#define SERIES_INDEX(s,i) (void *) ((s)->items + ((i) * (s)->item_size))

/*--------------------------------------------------------------*
  |                    EXPAND_SERIES                             |
  | Function:                                                    |
  |    Allocate the memory required for a series.                |
  | Parameters:                                                  |
  |    series   : Pointer to a record series.                    |
  | Returns:                                                     |
  |    OK or NOTOK.                                              |
  | Description:                                                 |
  |    Allocate or reallocate the memory used for a record       |
  |    series.                                                   |
  | Note:                                                        |
  |                                                              |
  *--------------------------------------------------------------*/

static int expand_series(SERIES *series)
{
    char *more;
    
    if (series->items)
        more = (char *) realloc(series->items,
                                (series->allocated + SERIES_SIZE)
                                * series->item_size);
    else
        more = (char *) malloc(SERIES_SIZE * series->item_size);
    
    if (more)
        {
            series->allocated += SERIES_SIZE;
            series->items    = more;
        }
    else
        return msg_make(NO_CORE, "series");
    
    return OK;
}

/*--------------------------------------------------------------*
  |                     SERIES_APPEND                            |
  | Function:                                                    |
  |    Append a new item to the series                           |
  | Parameters:                                                  |
  |    series    : Pointer to the series.                        |
  |    item      : Pointer to item to append.                    |
  | Returns:                                                     |
  |    OK if success, otherwise NOTOK.                           |
  | Description:                                                 |
  |    Just copy the item to the end of the series items.        |
  | Note:                                                        |
  |                                                              |
  *--------------------------------------------------------------*/

int series_append(SERIES *series,
                  void *item)
{
    /* Check if available space in series */
    
    if (series->total >= series->allocated)
        if (expand_series(series)) return NOTOK;
    
    COPY(SERIES_INDEX(series,series->total++),
         item,
         series->item_size);
    
    return OK;
}

/*--------------------------------------------------------------*
  |                     SERIES_NEXT                              |
  | Function:                                                    |
  |    Point to the next item in the series.                     |
  | Parameters:                                                  |
  |    series    : Pointer to the series.                        |
  | Returns:                                                     |
  |    OK if success, otherwise NOTOK.                           |
  | Description:                                                 |
  |    Same as append but return a pointer to the new item.      |
  | Note:                                                        |
  |                                                              |
  *--------------------------------------------------------------*/

void *series_next(SERIES *series)
{
    /* Check if available space in series */
    
    if (series->total >= series->allocated)
        if (expand_series(series)) return NULL;
    
    return SERIES_INDEX(series,series->total++);
}

/*--------------------------------------------------------------*
  |                     SERIES_ITEM                              |
  | Function:                                                    |
  |    Return an item at certain position.                       |
  | Parameters:                                                  |
  |    series    : Pointer to the series.                        |
  |    index     : Number of item to return.                     |
  | Returns:                                                     |
  |    NULL if index not in series, otherwise pointer to item.   |
  | Description:                                                 |
  |    Point to an item in the series.                           |
  | Note:                                                        |
  |                                                              |
  *--------------------------------------------------------------*/

void *series_item(SERIES *series,
		  int  index)
{
    void *item;
    
    /* Check if index is within series range */
    
    if (index < 1 || index > series->total)
        item = NULL;
    else
        item = SERIES_INDEX(series,index - 1);

    return item;
}
/*--------------------------------------------------------------*
 |                    SERIES_NEW                                |
 | Function:                                                    |
 |    Allocate a new series.                                    |
 | Parameters:                                                  |
 |    info    : Pointer to the record information.              |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Free any allocated memory.                                |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

SERIES *series_new(int size)
{
    SERIES *series;

    series = (SERIES *) calloc(1, sizeof(SERIES));    /* Zero memory */

    if (series != NULL)
        series->item_size = size;

    return series;
}

/*--------------------------------------------------------------*
 |                    SERIES_DISPOSE                            |
 | Function:                                                    |
 |    The record series are returned.                           |
 | Parameters:                                                  |
 |    info    : Pointer to the record information.              |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Free any allocated memory.                                |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void series_dispose(SERIES **series)
{
    if (series != NULL)
        {
            if ((*series)->items != NULL)
                free((*series)->items);

            free(*series);

            *series = NULL;
        }
}
