

/*                            SESSION.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <stdlib.h>

#ifdef BIB1_TABLE
#include "bib1.h"
#else
#define BIB1_SET_ID             "1.2.840.10003.3.1"
#endif

#include "berutil.h"
#include "constant.h"
#include "database.h"
#include "msg.h"
#include "netio.h"
#include "serverz.h"
#include "session.h"
#include "z3950.h"
#include "utility.h"

/*--------------------------------------------------------------*
 |                    CHECK_HOST_PORT                           |
 | Function:                                                    |
 |    Check the host and port and name of the database.         |
 | Parameters:                                                  |
 |    name   : Name of the database to change.                  |
 |    host   : Name of the host to change.                      |
 |    port   : Port number.                                     |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void check_host_port(char **name,
                     char **host,
                     char **port,
		     char **authorize)
{

    if (**port == EOS) *port = Z3950_PORT;

    if (**host == EOS)
        {
            SELECT_REFERENCE select = &select_table[0];
            char upper[80];
            int  count;

            uppercase(upper, *name, strlen(*name));
            for (count = select_total; count--; select++)
                {
                    if (strcmp(select->match, upper) == 0)
                        {
                            *name = select->name;
                            *host = select->host;
			    if (*select->port && **port == EOS) 
				*port = select->port;
			    if (*select->authorize && **authorize == EOS)
				*authorize = select->authorize;
                            break;
                        }
                }
            if (count < 0) *host = Z3950_HOST;
        }
    if (**port == EOS) *port = Z3950_PORT;
}

/*--------------------------------------------------------------*
 |                    ESTABLISH_DATABASE                        |
 | Function:                                                    |
 |    Establish the database reference.                         |
 | Parameters:                                                  |
 |    db     : Pointer to database reference to set.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void establish_database(DATABASE_REFERENCE *db,
                        char *name)
{
    /* Allocate the new database structure */

    *db = database_new(name);

    /* Now set the information for the database                 */
    /* In the future this will be set with the explain facility */

    if (*db != NULL_DATABASE)
        {
#ifdef BIB1_TABLE
            database_set_index(*db,
                               BIB1_SET_ID,
                               RPN,
                               (INDEX_REFERENCE) bib1_indexes,
                               bib1_total);
#else
            database_set_index(*db,
                               BIB1_SET_ID,
                               RPN,
                               (INDEX_REFERENCE) NULL,
                               0);
#endif
	}
}

/*--------------------------------------------------------------*
 |                    ESTABLISH_CONNECTION                      |
 | Function:                                                    |
 |    Establish the connection to the Z39.50 server.            |
 | Parameters:                                                  |
 |    io     : Pointer to network information.                  |
 |    host   : Pointer to the host name (number).               |
 |    port   : Pointer to the port number (name).               |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void establish_connection(NETIO_REFERENCE *io,
                          char *host,
                          char *port)
{
    if ((*io = (NETIO_REFERENCE) net_initialize()) != NULL)
        {
            if (net_connect(*io, host, port) == NOTOK)
                net_dispose(io);
        }
}

/*--------------------------------------------------------------*
 |                    SESSION_SERVERS                           |
 | Function:                                                    |
 |    List out the names of the Z39.50 servers.                 |
 | Parameters:                                                  |
 |    None                                                      |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |    List out the names in the server table.                   |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int session_servers()
{
    SELECT_REFERENCE select;
    int  count;

    for (select = &select_table[0], count = select_total; count--; select++)
        {
            msg_make(FORMAT_MSG,
                    "%15s   (%s at %s)",
                    select->match,
                    select->name,
                    select->host);
        }
    return OK;
}

/*--------------------------------------------------------------*
 |                    SESSION_NEW                               |
 | Function:                                                    |
 |    Establish a new session.                                  |
 | Parameters:                                                  |
 |    name   : Pointer to the server entry or database name.    |
 |    host   : Pointer to the host name (number).               |
 |    port   : Pointer to the port number (name).               |
 | Returns:                                                     |
 |    Pointer to session information, or NULL if failed.        |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

SESSION *session_new(char *name,
                     char *host,
                     char *port,
		     char **authorize)
{
    SESSION *session;

    session = (SESSION *) calloc(1, sizeof(SESSION));
    if (session == NULL)
        msg_make(NO_CORE, "session");
    else
        {
            check_host_port(&name, &host, &port, authorize);

            /* New database information */

            establish_database(&session->db, name);
            if (session->db == NULL)
                {
                    free(session);
                    return NULL;
                }
/*
            if (   strcmp(name,"cat") == 0 
		&& strcmp(host,"melvyl.ucop.edu") == 0)
                {
		    database_new_index(session->db,"AUTHOR",  "USE=1  POS=3 STRUC=6");
		    database_new_index(session->db,"SUBJECT", "USE=21 POS=3 STRUC=6");
		    database_new_index(session->db,"AP",      "USE=1  POS=3 STRUC=1");
		    database_new_index(session->db,"SP",      "USE=21 POS=3 STRUC=1");
		}
*/
	    
            /* New connection */

            establish_connection(&session->io, host, port);
            if (session->io == NULL)
                {
                    database_dispose(&session->db);
                    free(session);
                    return NULL;
                }

            if (   ((session->pdu = dalloc(DIR_BLOCKS)) == NULL)
                || ((session->rdu = dalloc(DIR_BLOCKS)) == NULL) )
                {
                    msg_make(NO_CORE, "pdu");
                    database_dispose(&session->db);
                    net_dispose(&session->io);
                    free(session);
                    return NULL;
                }
        }

    return session;
}

/*--------------------------------------------------------------*
 |                    SESSION_DISPOSE                           |
 | Function:                                                    |
 |    Dispose of the session information.                       |
 | Parameters:                                                  |
 |    p_session  : Address of session pointer.                  |
 | Returns:                                                     |
 |    OK or NOTOK.                                              |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int session_dispose(SESSION **p_session)
{
   SESSION *session = *p_session;

    if (session != NULL)
        {
            database_dispose(&session->db);

            net_dispose(&session->io);

            dfree(session->pdu);

            dfree(session->rdu);

            free(session);

            *p_session = NULL;
        }
    return OK;
}

