/*                            STRINGS.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <ctype.h>

#ifdef ANSI
#include <memory.h>
#endif

#include "constant.h"
#include "portinfo.h"
#include "strings.h"

/*--------------------------------------------------------------*
 |                    STRING_UPPER                              |
 | Function:                                                    |
 |    Convert the string into upper case.                       |
 | Parameters:                                                  |
 |    str   : String to convert to uppercase                    |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void str_uc(STRING *string)
{
    char *letter;
    int length;

    if (string != NULL)
        {
             letter = string->location;
             length = string->length;

             while (length-- > 0)
                 *letter = islower(*letter) ? toupper(*letter++)
                                            : *letter++;
        }
}

/*--------------------------------------------------------------*
 |                    STRING_SQEEZE                             |
 | Function:                                                    |
 |    Squeeze multiple blanks to a single blank and strip       |
 |    trailing blanks from an input string to produce a         |
 |    squeezed string.                                          |
 | Parameters:                                                  |
 |    original  : Original string to squeeze.                   |
 |    squeezed  : Squeezed string.                              |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |    Assumes that squeezed has allocated memory.               |
 *--------------------------------------------------------------*/

void str_squ(STRING *original,
             STRING *squeezed)
{
    char *copy;
    char *letter;
    int  length;

    if (original == NULL || squeezed == NULL)
        return;

    length = original->length;
    letter = original->location;
    copy   = squeezed->location;

    /* Squeeze blanks */

    while (length-- > 0)
       {
           if (*letter == BLANK)
              {
                  if (copy != squeezed->location)
                      *copy++ = *letter++;
                  while (length-- > 0 && *letter == BLANK) letter++;
                  if (!length) break;
              }
           *copy++ = *letter++;
       }

    if (*(copy - 1) == BLANK) copy--;

    /* Find Length */

    squeezed->length = copy - squeezed->location;
}

/*--------------------------------------------------------------*
 |                    STRING_COMPARE                            |
 | Function:                                                    |
 |    Compare two strings.                                      |
 | Parameters:                                                  |
 |    one  : First string to compare                            |
 |    two  : Second string to compare                           |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Returns -1, 0 or 1 if the first string is logically       |
 |    greater (>), equal (=) or less (<) than a second string.  |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int str_cmp(STRING *one,
            STRING *two)
{
    int result;
    unsigned long length;

    length = MIN(one->length, two->length);

    if (length > 0)
        result = memcmp((void *) one->location,
                        (void *) two->location,
                        length);
    else
        result = 0;

    if (result == 0)
        if (one->length > two->length)
            result = 1;
        else if (one->length < two->length)
            result = -1;

    return result;
}


/*--------------------------------------------------------------*
 |                    CSTRING_COMPARE                           |
 | Function:                                                    |
 |    Compare a string and a C-string.                          |
 | Parameters:                                                  |
 |    one  : First string to compare                            |
 |    two  : C-string to compare                                |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Returns -1, 0 or 1 if the first string is logically       |
 |    greater (>), equal (=) or less (<) than a second string.  |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int str_ccmp(STRING *one,
             char   *two)
{
    int result;
    int length;
    int c_len;

    c_len = strlen(two);

    length = MIN(one->length, c_len);

    if (length > 0)
        result = memcmp((void *) one->location, (void *) two, length);

    else
        result = 0;

    if (result == 0)
        if (one->length > c_len)
            result = 1;
        else if (one->length < c_len)
            result = -1;

    return result;
}

/*--------------------------------------------------------------*
 |                    STRING_DUPLICATE                          |
 | Function:                                                    |
 |    Duplicate a string.                                       |
 | Parameters:                                                  |
 |    source    : Original string to copy.                      |
 |    duplicate : Duplicate string destination.                 |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void str_dup(STRING *source,
             STRING *duplicate)
{
    if ((duplicate->length = source->length) > 0)
        {
            if (duplicate->location != NULL)
                COPY(duplicate->location, source->location, source->length);
        }
}

/*--------------------------------------------------------------*
 |                    STRING_CONCATINATE                        |
 | Function:                                                    |
 |    Concatinate a string.                                     |
 | Parameters:                                                  |
 |    source    : String to concatinate (append) with another.  |
 |    concat    : String to concatinate to source.              |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void str_cat(STRING *source,
             STRING *concat)
{
    if (concat->length > 0)
        {
	    if (concat->location != NULL)
		{
		    MOVE(&source->location[source->length], 
			 concat->location,
			 concat->length);

		    source->length += concat->length;
		}
        }
}

/*--------------------------------------------------------------*
 |                  STRING_C_CONCATINATE                        |
 | Function:                                                    |
 |    Concatinate a string.                                     |
 | Parameters:                                                  |
 |    source    : String to concatinate (append) with another.  |
 |    concat    : C-String to concatinate to source.            |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

void str_ccat(STRING *source,
	      char *concat)
{
    int length = (concat == NULL) ? 0 : strlen(concat);
    
    if (length > 0)
        {
	    MOVE(&source->location[source->length], 
		 concat, 
		 length);

	    source->length += length;
        }
}

/*--------------------------------------------------------------*
 |                    STRING_TRIM                               |
 | Function:                                                    |
 |    Trim a string.                                            |
 | Parameters:                                                  |
 |    source    : String to trim.                               |
 | Returns:                                                     |
 |    Number of characters trimmed at front of string.          |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int str_trim(STRING *source)
{
    int skip = 0;
    int length;
    
    while (source->length > 0 && isspace(source->location[0]))
	{
	    source->length--;
	    source->location++;
	    skip++;
	}
   
    for (length = source->length - 1; length > 0; length--)
	{
	    if (isspace(source->location[length]))
		source->length--;
	    else
		break;
	}
    
    return skip;
}
