/*                           TRANSMIT.C                            */

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

/*
 * 05/25/94  slh  _SEQUENT_ conds
 */

#include <stdio.h>
#include <stdlib.h>
#ifdef   _SEQUENT_
#include  <sys/procstats.h>
#else  /*_SEQUENT_*/
#include <sys/time.h>
#ifndef vax
#include <time.h>
#endif
#endif /*_SEQUENT_*/
#include <sys/types.h>

#include "berutil.h"
#include "constant.h"
#include "msg.h"
#include "netio.h"
#include "pdutags.h"
#include "portinfo.h"
#include "session.h"

/* External routines to handle access and resource control */

extern void reply_access_control_pdu();
extern int reply_resource_control_pdu();
 
/* Once a response is received it usually requires more than
   one read to obtain the complete packet.  These routines
   controls how often the connection should be polled before
   giving up.
 */

#define USEC_WAIT_FOR_DATA      100           /* microseconds to wait */
#define TRIES_TO_READ_DATA      100           /* 1 second */

/* Read from the server socket and poll for more data    *
 * in the socket.  The PDU can be split into multiple    *
 * packets by the network.                               */

static int read_server(SESSION *session)
{
   int code;

   do
      if ((code = net_read(session->io))) break;
   while ((code = net_wait(session->io, 0, 0)) > 0);

  return code;
}

/* Read the data from the network until the remainder    *
 * bytes are read.                                       */

static int read_fixed_amount(SESSION *session,
                             long remainder,
                             int secs_wait)
{
    int code = OK;

    remainder += session->io->length;

    while (session->io->length < remainder)
        {
            if ((code = read_server(session))) break;
        }
    return code;
}

/* Read the data from the network and poll for more data *
 * since more than one read may be necessary (I think    *
 * because each read will only a packet at a time and    *
 * some data requires multiple packets, of course this   *
 * is largely operating system dependent).               */

static int read_indefinite_amount(SESSION *session,
                                  int secs_wait)
{
    long remainder;
    int code;

    do
        {
            code = net_wait(session->io, secs_wait, 0);
    
	    if (code > 0)
                code = read_server(session);

            if (code == OK)
                {
		    code = IsCompleteBER((CHAR *) session->io->buffer,
					 (long) session->io->length,
					 &remainder);

		    if (code != 0)           /* BER is complete */
			return OK;
		    else if (remainder > 0)  /* Read remainder */
			return read_fixed_amount(session, remainder, secs_wait);
		    else
			code = OK;
		}
	    else if (code == READ_TIMEOUT && session->user_timeout != NULL)
		code = (*session->user_timeout)(session);

            /*
             * This is not an else if since session->user_timeout
             * could return a code of CANCEL_SEARCH.
             */

            if (code == CANCEL_SEARCH && session->user_cancel != NULL)
                code = (*session->user_cancel)(session);
        }
    while (code == OK);
    
    return code;
}

static int read_receive_pdu(SESSION *session,
                            int secs_wait)
{
    int code;
   
    code = read_indefinite_amount(session, secs_wait);

    switch (code) {
      case OK:
          break;

      case READ_TIMEOUT:
          code = msg_make(TIME_OUT, secs_wait);
          break;

      case CANCEL_SEARCH:
          code = msg_make(DEFAULT_MSG, "Search Cancel not supported.");
          break;

      default:
          code = msg_make(SYSTEM_ERROR, "network read");
    }
    return code;
}

static int io_server(SESSION *session,
                     DATA_DIR *pdu,
                     int secs_wait)
{
    extern int debug;
    struct timeval sent;
    struct timeval received;
    
    /* Only send if the field identifier is set, this allows *
     * this routine to only recieve data.                    */

    if (pdu->fldid)
        {
	    CHAR *record;
	    int  length;
	    int  code;

            if (debug) hex_dir(pdu,0);

            if ((record = bld_rec(pdu, (long *) &length)) == NULL)
                return msg_make(BUILD_RECORD);

            code = net_write(session->io, record, length);
            free(record);
            if (code) return NOTOK;
        }

    /* Set time in milliseconds */

#ifdef   _SEQUENT_
    (void)get_process_stats( &sent , PS_SELF , NULL , NULL );
#else  /*_SEQUENT_*/
    (void) gettimeofday(&sent, NULL);
#endif /*_SEQUENT_*/

    /* Wait for a response from the server */

    session->io->length = 0;

    if (read_receive_pdu(session, secs_wait))
	return NOTOK;

#ifdef   _SEQUENT_
    (void)get_process_stats( &received , PS_SELF , NULL , NULL );
#else  /*_SEQUENT_*/
    (void) gettimeofday(&received, NULL);
#endif /*_SEQUENT_*/

    session->wait_for_pdu = ((received.tv_sec - sent.tv_sec) * USECS_IN_SECOND
			     + received.tv_usec - sent.tv_usec);
    
    if (bld_dir((CHAR *) session->io->buffer, pdu) == 0)
        return msg_make(CANT_CONVERT_BER, "PDU");

    if (debug) hex_dir(pdu,0);

    return OK;
}

int transmit_pdu(SESSION *session,
                 DATA_DIR *pdu,
                 int pdu_request,
                 int max_wait)
{
    session->state = pdu_request;

    if (session->io->socket == 0)
        return msg_make(NO_CONNECTION);

    for (; ;)    /* Loop to send & receive request */
        {
            if (io_server(session, pdu, max_wait))
                return NOTOK;

            if (pdu->ptr.child == NULL)
                return msg_make(INVALID_PDU);
            else if (pdu->fldid == AccessControlRequest)
                reply_access_control_pdu(session);
            else if (pdu->fldid == ResourceControlRequest)
                reply_resource_control_pdu(session);
            else
                break;    /* Escape loop */
        }
    return OK;
}
