/*                            CLIENT.C                       */   

/*
 *  Copyright (c) 1992 Stanford Leland Junior University
 *
 *  Commercial use prohibited
 *
 *  Author: Harold Finkbeiner (Harold@Forsythe.Stanford.Edu)
 */

#include <ctype.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#ifdef ANSI
#include <memory.h>
#endif

#include "constant.h"
#include "api.h"
#include "msg.h"
#include "utility.h"

#define MAX_BUFFER          256           /* Maximum character buffer size */

/* Used by the assignment subroutine below */

#define NUMBER              1              /* Integer parameters */
#define P_WORD              2              /* A word parameter */
#define P_WORD_RAW          3              /* A raw word (without quote processing) */
#define P_PHRASE            4              /* A phrase parameter */
#define P_TOKEN             5              /* The token word exists */

typedef struct command_entry
    {
        char *command;
        void (*routine)();
    } COMMAND;

extern FILE *out;
extern FILE *in;

static int terse_mode;

#ifdef MARKER

#define BEGIN_MARKER        "<<BEGIN>>\n"   /* Beginning marker */
#define END_MARKER          "\n<<END>>\n"   /* End marker */

#endif

/*--------------------------------------------------------------*
 |                    DO_IO_SELECT                            |
 | Function:                                                  |
 |    Upcall to read client input during a Network read.      |
 | Parameters:                                                |
 |    operation : Control flag: OPERATE or ENQUEUE            |
 |    ENQUEUE means to enqueue all client input               |
 |    for later operation.                                    |
 |    OPERATE means to operate on any input in the            |
 |    queue, and then operate on any new input.               |
 | Returns:                                                   |
 |    CANCEL_SEARCH if in the course of doing client I/O      |
 |    it received a CANCEL_SEARCH command.                    |
 |    OK otherwise.                                           |
 | Description:                                               |
 |    This routine is called when netio's select loop notices |
 |    that Client_FD has pending input.                       |
 |    It's purpose in life is to read that pending input and  |
 |    either OPERATE on it or ENQUEUE it for later operation, |
 |    as specified by the control flag.                       |
 | Note:                                                      |
 |    If the control flag is OPERATE, do_io_select will       |
 |    operate on all input pending in the queue and THEN read |
 |    the input, and operate on it.                           |
 | Note Also:                                                 |
 |    The code in netio.c assumes that 'Client_FD' is the     |
 |    client file descriptor number.  It is up to you to set  |
 |    Client_FD to the input descriptor number so that the    |
 |    code in netio.c will select on it and make this upcall. |
 | (Asynchronous I/O is so painful for an API to support ...) |
 |-------------------------------------------------------------*/
/*
 * The code given below is an example taken from another client.
 * It is included to give you an idea of how you might implement
 * the queue.
 * It takes client input as packets.  You should define your own
 * idea of packets and your own way of recognizing a CANCEL operation.
 *
 * The only thing you NEED in order for the API to compile is the
 * stub routine so that the linker will not complain.
 *
 * Perhaps a subsequent version of client.c will make user input
 * asynchronous and actually use the do_io_select client upcall.
 */
extern int Client_FD;
int do_io_select(int operation)
{
    return 0;
}
#ifdef WILLOW_EXAMPLE
typedef struct packet_node    /* used by the queue */
  {
      packet_rec packet;
      struct packet_node *next;
  } packet_node, *queue_ptr;
/* Dispatches to handler depending on packet type received */
void packet_operate(packet_rec *p_packet)
{
    static SESSION *session;
    switch (p_packet->type) {
      case CONNECT:
          /* zclient SELECT command */
          select_command(&session, p_packet);
          break;
      case SEARCH:
          /* zclient FIND command */
          search_wcb(&session, p_packet);
          break;
      case CANCEL:
          search_cancel_wcb(&session, p_packet);
          break;
      case SUMM_REQ:
          /* zclient DISPLAY command (for now) */
          summ_req_wcb(&session, p_packet);
          break;
      case GET_FULL:
          get_full_wcb(&session, p_packet);
          break;
      case DISCONNECT:
          /* zclient CLOSE command */
          disconnect_wcb(&session);
          break;
      case ECHO:
          echo_wcb(p_packet);
          break;
      case CHEAT:
          cheat_wcb(p_packet);
          break;
      default:
          error("unknown packet type");
    }
    free(p_packet->data);
}
int do_io_select(int operation)
{
    /* externals */
    extern int debug_level;
    SESSION *session;
    packet_rec packet;
    static queue_ptr packet_queue=(queue_ptr) NULL;
    static queue_ptr tmp_ptr=(queue_ptr) NULL;
    /* On Operates, first flush out any enqueued packets */
    if (operation == OPERATE)
      while (packet_queue != (queue_ptr) NULL) {
          packet_operate(&packet_queue->packet);
          tmp_ptr = packet_queue->next;
          free(packet_queue);
          packet_queue = tmp_ptr;
      }
    /* Read in the packet */
    read_from_client(&packet);
    /* Call a handler routine for this packet type */
    if (debug_level) {
      fprintf(stderr,"DRIVER RECEIVED PACKET.\n");
      fflush(stderr);
    }
    if (operation == ENQUEUE) {
      if (debug_level) {
          fprintf(stderr,"ENQUEUED request.\n");
          fflush(stderr);
      }
      if (packet.type == CANCEL) {
          /* Ignore previous packets? */
          while (packet_queue != (queue_ptr) NULL) {
              free(packet_queue->packet.data);
              tmp_ptr = packet_queue->next;
              free(packet_queue);
              packet_queue = tmp_ptr;
          }
          return CANCEL_SEARCH;
      }
      else { /* enqueue packet, return */
          tmp_ptr = packet_queue; /* save head of queue */
          while(packet_queue != (queue_ptr) NULL)
              packet_queue = packet_queue->next;
          packet_queue = (queue_ptr) malloc(sizeof(packet_node));
          packet_queue->packet = packet;
          packet_queue->next = (queue_ptr) NULL;
          /* if queue was initially non-empty, restore packet_queue to saved head */
          if (tmp_ptr != (queue_ptr) NULL)
              packet_queue = tmp_ptr;
          return OK;
      }
    }
    else if (operation == OPERATE) {
      if (debug_level) {
          fprintf(stderr,"OPERATE request, packet type %d\n",packet.type);
          fflush(stderr);
      }
      /* operate on current packet */
      packet_operate(&packet);
      return OK;
    }
    else {
      return msg_make(DEFAULT_MSG,"Bad operation type in io_select");
    }
    return OK;
}
#endif

/*--------------------------------------------------------------*
 |                    INPUT_COMMAND                             |
 | Function:                                                    |
 |    Input a command.                                          |
 | Parameters:                                                  |
 |    buffer : Address of input buffer.                         | 
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int input_command(char *input_buffer,
                         int buffer_size)
{
    char *input;
    int  length;
    int  prompt;

    do
	{
#ifndef MARKER
	    if (prompt) fprintf(out, "Z39.50> ");
#endif

	    input = fgets(input_buffer, buffer_size, in);
	    if (input == NULL)
		length = 0;
	    else
		{
		    length = strlen(input) - 1;
		    while (length >= 0)
			{
			    if (input_buffer[length] == NL)
				input_buffer[length--] = EOS;
			    else
				break;
			}
		}
	}
    while (length <= 0); 

    return length;
}

/*--------------------------------------------------------------*
 |                    CLIENT_MSG                                |
 | Function:                                                    |
 |    Print out the message.                                    |
 | Parameters:                                                  |
 |    session  : Pointer to session information.                |
 |    format   : Format for the message.                        |
 |    va_list  : Arguements for the message.                    |
 | Returns:                                                     |
 |    Nothing.                                                  |
 | Description:                                                 |
 |    Uses vfprintf to print out the message.  Also checks the  |
 |    messages saved in IRI and prints them out.                |
 | Note:                                                        |
 |    Should probably just redirect io to stdin and stdout.     |
 *--------------------------------------------------------------*/

static void client_msg(char *format,
                       ...)
{
    extern FILE *msg_log;
    va_list more;

    va_start(more, format);
    vfprintf(out, format, more);
    if (msg_log)
        {
            vfprintf(msg_log, format, more);
            fflush(msg_log);
        }
    msg_output();
    fflush(out);

    va_end(more);
}

/*--------------------------------------------------------------*
 |                    ASSIGNMENT                                |
 | Function:                                                    |
 |    Assign a the next word in the string.                     |
 | Parameters:                                                  |
 |    input   : Address of input string.                        |
 |    name    : Pointer to assignment name.                     |
 |    type    : Type of value to assign.                        |
 |    assign  : Pointer to string to hold assignment            |
 | Returns:                                                     |
 |    length of assigned string.                                |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static int assignment(char **input,
                      char *name,
                      int  type,
                      char *assign)
{
    char compare[MAX_BUFFER];
    char *token;
    int  length;
    char *phrase = *input;

    /* Check if a name needs to be matched */

    if (*name != EOS)
        {
            length = tokenize(&phrase, &token, " =");

            UNQUOTE(token, length);

            uppercase(compare, token, length);
            if (strncmp(name, compare, length) != 0 || length == 0)
                {
                    /* No match so set to no value */
 
                    if (assign != NULL)
                        {
                            if (type == NUMBER)
                                *((int *) assign) = 0;
                            else
                                assign[0] = EOS;
                        }
                    return 0;
                }
            if (type == P_TOKEN) return 1;
        }

    if (assign != NULL)
        {
	    length = tokenize(&phrase, &token, " =");

	    if (*name != EOS && *token == EQUAL_SIGN)
                {
                    phrase++;
                    length = tokenize(&phrase, &token, " ");
                }
	    
            switch (type)
                {
                    case NUMBER:
                        if (length <= 0)
                           *((int *) assign) = 0;
                        else
                           *((int *) assign) = atoi(token);
                        break;

                    case P_WORD:

			UNQUOTE(token, length);

			/* Fall through into P_WORD_RAW */

		    case P_WORD_RAW:

                        if (length <= 0)
                            assign[0] = EOS;
                        else
                            {
                                COPY(assign, token, length);
                                assign[length] = EOS;
                            }
                        break;

                    case P_PHRASE:
                        length = strlen(token);
			phrase = token + length;   /* Point to EOS */

			/* Trim trailing blanks */

			while (token[length - 1] == BLANK && length > 1) length--;
			
                        COPY(assign, token, length);
                        assign[length] = EOS;
                        break;

                    default:
                        break;
                }
        }

    *input = phrase;
    return length;
}

/*--------------------------------------------------------------*
 |                    SAVE_COMMAND                              |
 | Function:                                                    |
 |    Save the last network IO packet.                          |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for debug command.          |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void zsave_command(SESSION **p_session,
                          char *parameters)
{
    if (z39_save_to_log(*p_session, "zsave.log"))
       client_msg("-OK: Output to file worked\n");
    else
       client_msg("-ERROR: Unable to save Z3950 packet\n");
}

/*--------------------------------------------------------------*
 |                    DEBUG_COMMAND                             |
 | Function:                                                    |
 |    Turn the debugging flag on and off.                       |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for debug command.          |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |                                                              |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void debug_command(SESSION **p_session,
                          char *parameters)
{
    if (assignment(&parameters, "OFF", P_WORD, NULL) > 0)
        {
            z39_debug(*p_session, 0);
            client_msg("-OK: Debugging is turned OFF\n");
        }
    else
        {
            z39_debug(*p_session, 1);
            client_msg("-OK: Debugging is turned ON\n");
        }
}

/*--------------------------------------------------------------*
 |                    SHOW_COMMAND                              |
 | Function:                                                    |
 |    Show information about the session.                       |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for index command.          |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |    Show various information about the session.               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void show_command(SESSION **p_session,
                         char *parameters)
{
    int  code;

    /* Parse the input of the command */

    if (assignment(&parameters, "INDEXES", P_WORD, NULL) > 0)
        code = z39_show(*p_session, "INDEXES");
    else if (assignment(&parameters, "SELECT", P_WORD, NULL) > 0)
        code = z39_show(*p_session, "SELECT");
    else
        code = msg_make(DEFAULT_MSG,
                        "Invalid SHOW command: SHOW [INDEXES | SELECT]\n");

    if (code)
        client_msg("-ERROR: Show command failed\n");
    else
        client_msg("-OK: Show command succeeded\n");
}

/*--------------------------------------------------------------*
 |                    INDEX_COMMAND                             |
 | Function:                                                    |
 |    Create or modify an index used for searching.             |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for index command.          |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |    Just add an index to a database session.                  |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void index_command(SESSION **p_session,
                          char *parameters)
{
    char name[MAX_BUFFER];
    char attributes[MAX_BUFFER];
    char index[MAX_BUFFER];
    char *token;
    int  code;

    /* Parse the input of the command */

    do
	{
	    assignment(&parameters, "", P_WORD_RAW, index);
	    if (index[0] != EOS)
		{
		    if (index[0] == QUOTE || index[0] == DOUBLE_QUOTE)
			{
			    /* Parse index command between quotes */

			    token = &index[1];
			    index[strlen(token)] = EOS;

			    assignment(&token, "", P_WORD, name);
			    assignment(&token, "", P_PHRASE, attributes);
			}
		    else
			{
			    strcpy(name, index);
			    assignment(&parameters, "", P_PHRASE, attributes);
			}
		    
		    code = z39_index(*p_session, name, attributes);
		}
	}
    while (index[0] != EOS && *parameters != EOS && code == 0);
    
    if (code)
        client_msg("-ERROR: Indexing %s\n", parameters);
    else
        client_msg("-OK: Indexing succeeded.\n");
}

/*--------------------------------------------------------------*
 |                    SELECT_COMMAND                            |
 | Function:                                                    |
 |    Select a database and server for searching.               |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 if successful, -1 if failure.                           |
 | Description:                                                 |
 |    Interface to call ir_open to connect to Z39.50 server.    |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void select_command(SESSION **p_session,
                           char *parameters)
{
    char name[MAX_BUFFER];
    char port[MAX_BUFFER];
    char host[MAX_BUFFER];
    char globalset[MAX_BUFFER];
    char id[MAX_BUFFER];
    char syntax[MAX_BUFFER];
    int  code;

    assignment(&parameters, "", P_WORD, name);
    assignment(&parameters, "HOST", P_WORD, host);
    assignment(&parameters, "PORT", P_WORD, port);
    assignment(&parameters, "GLOBALSET", P_WORD, globalset);
    assignment(&parameters, "ID", P_WORD, id);
    assignment(&parameters, "SYNTAX", P_WORD, syntax);

#ifdef MARKER 
    assignment(&parameters, "MARC", NUMBER, &marc_type);
#endif

    while (*parameters && *parameters == BLANK) parameters++;

    if (name[0] == EOS)
        client_msg("-ERROR: Missing database name in SELECT command\n");
    else if (*parameters != EOS)
        client_msg("-ERROR: Extra paramaters on SELECT: %s\n", parameters);
    else
        {
            code = z39_initial(p_session, name, host, port, globalset, syntax, id);

            if (code)
                client_msg("-ERROR: Unable to establish connection.\n");
            else
		{
#ifdef MARKER
		    (*p_session)->marc_type = marc_type;
#endif 
		    client_msg("-OK: Connection to %s is established.\n", 
                               session_host(*p_session));
		}
        }
}

/*--------------------------------------------------------------*
 |                    FIND_COMMAND                              |
 | Function:                                                    |
 |    Send a query to the Z39.50 server.                        |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |    Interface to call ir_find to search the Z39.50 server.    |
 | Note:                                                        |
 |    This required a slight change to the Berkeley client      |
 |    code so the "records" were returned instead of printed    |
 |    out directly.  This allows more control over how & when   |
 |    the records are output.                                   |
 *--------------------------------------------------------------*/

static void find_command(SESSION **p_session,
                         char *find)
{
    RECORD_INFO *info;
    char database[MAX_BUFFER];
    char setname[MAX_BUFFER];
    char syntax[MAX_BUFFER];
    int  code;

    assignment(&find, "DATABASE", P_WORD, database);
    assignment(&find, "SETNAME", P_WORD, setname);
    assignment(&find, "SYNTAX", P_WORD, syntax);

    info = recinfo_new();

    code = z39_search(*p_session, find, database, setname, syntax, info);

    if (code == NOTOK)
        {
            client_msg("-ERROR: Search request '%s'.\n", find);
        }
    else
        {
#ifdef MARKER
            client_msg("-OK: %d %d %d\n",
                       recinfo_get_status(info),
                       recinfo_get_result(info),
                       recinfo_get_returned(info));
#endif
#ifndef MARKER
            client_msg("-OK: Status %d, Hits %d, Received %d (Set: %s)\n",
                       recinfo_get_status(info),
                       recinfo_get_result(info),
                       recinfo_get_returned(info),
                       session_setname(*p_session));
#endif
        }

    recinfo_error(*p_session, info);
    recinfo_output(*p_session, info);    
    recinfo_dispose(&info);
}

/*--------------------------------------------------------------*
 |                    SCAN_COMMAND                              |
 | Function:                                                    |
 |    Send a scan request to the Z39.50 server.                 |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |    Send a scan request to browse an index.                   |
 | Note:                                                        |
 |    Example usage: Request five sequential titles starting    |
 |    with the title 2 titles before "blah".                    |
 |                                                              |
 |    scan number=5 position=3 stepsize=0 title blah            |
 |                                                              |
 |    n.b. also that the index and start term are only words    |
 |         at this point.  Some testing is done for the numeric |
 |         values(in api).                                      |
 *--------------------------------------------------------------*/

static void scan_command(SESSION **p_session,
                         char *browse)
{
    SCAN_INFO *scan;
    char index[MAX_BUFFER];
    char term[MAX_BUFFER];
    int num_terms;
    int position;
    int stepsize;
    int code;

    if (assignment(&browse, "NUMBER", NUMBER, (char *) &num_terms) == 0)
	num_terms = SCAN_NUMBER;
    if (assignment(&browse, "POSITION", NUMBER, (char *) &position) == 0)
	position = SCAN_POSITION;
    if (assignment(&browse, "STEPSIZE", NUMBER, (char *) &stepsize) == 0)
	stepsize=0;

    assignment(&browse, "", P_WORD, index);
    assignment(&browse, "", P_PHRASE, term);

    scan = scaninfo_new();

    code = z39_scan(*p_session, index, term, num_terms, position, stepsize, scan);
    if (code == NOTOK)
        {
            client_msg("-ERROR: Scan request.\n", browse);
        }
    else
	{
#ifdef MARKER
	    client_msg("-OK: %d %d\n",
		       scaninfo_get_status(scan),
		       scaninfo_get_returned(scan));
#endif
#ifndef MARKER
	    client_msg("-OK: Status %d, Returned %d\n",
		       scaninfo_get_status(scan),
		       scaninfo_get_returned(scan));
	    if (!terse_mode)
	        client_msg("     Optional, may not be correct: Position %d, StepSize %d\n",
		           scaninfo_get_position(scan),
		           scaninfo_get_stepsize(scan));
#endif
	}

    if MSG_TEST msg_output();
    scaninfo_output(*p_session, scan);
    scaninfo_dispose(&scan);
}

/*--------------------------------------------------------------*
 |                    DISPLAY_COMMAND                           |
 | Function:                                                    |
 |    Retrieve & display records from the Z39.50 server.        |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |    Interface to call ir_retrieve to get records from         |
 |    the Z39.50 server.                                        |
 | Note:                                                        |
 |    This required a slight change to the Berkeley client      |
 |    code so the "records" were returned instead of printed    |
 |    out directly.  This allows more control over how & when   |
 |    the records are output.                                   |
 *--------------------------------------------------------------*/

static void display_command(SESSION **p_session,
                            char *parameters)
{
    RECORD_INFO *info;
    char setname[MAX_BUFFER];   
    char elemname[MAX_BUFFER];           /* Element Set Name */
    char syntax[MAX_BUFFER];             /* Perferred Syntax */
    int  position;
    int  display;
    int  code;

    assignment(&parameters, "SETNAME", P_WORD, setname);
    assignment(&parameters, "ELEMNAME", P_WORD, elemname);
    assignment(&parameters, "SYNTAX", P_WORD, syntax);
    if (assignment(&parameters, "RECORDS", NUMBER, (char *) &display) == 0)
        display = DISPLAY_NUMBER;
    assignment(&parameters, "", NUMBER, (char *) &position);

    info = recinfo_new();

    code = z39_present(*p_session, 
		       setname, 
		       elemname,
		       syntax,
		       position, 
		       display, 
		       info);

    if (code == NOTOK)
        client_msg("-ERROR: Display request '%s'.\n", parameters);
    else
        {
#ifdef MARKER
            client_msg("-OK: %d %d\n",
                       recinfo_get_status(info),
                       recinfo_get_returned(info));
#endif
#ifndef MARKER
            client_msg("-OK: Status %d, Received %d, Position %d, (Set: %s)\n",
                       recinfo_get_status(info),
                       recinfo_get_returned(info),
                       recinfo_get_position(info),
                       session_setname(*p_session));
#endif
        }

    recinfo_error(*p_session, info);
    recinfo_output(*p_session, info);
    recinfo_dispose(&info);

    if (terse_mode) client_msg("-END-OF-DISPLAY-COMMAND-OUTPUT-\n"); 
}

/*--------------------------------------------------------------*
 |                    MDISPLAY_COMMAND                          |
 | Function:                                                    |
 |    Retrieve & display records from the Z39.50 server.        |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |    Interface to call ir_retrieve to get records from         |
 |    the Z39.50 server.                                        |
 | Note:                                                        |
 |    This required a slight change to the Berkeley client      |
 |    code so the "records" were returned instead of printed    |
 |    out directly.  This allows more control over how & when   |
 |    the records are output.                                   |
 *--------------------------------------------------------------*/

static void mdisplay_command(SESSION **p_session,
			     char *parameters)
{
    SESSION *session = *p_session;
    RECORD_INFO *info;
    char setname[MAX_BUFFER];   
    char elemname[MAX_BUFFER];           /* Element Set Name */
    char syntax[MAX_BUFFER];             /* Perferred Syntax */
    int  position;
    int  display;
    int  code;

    assignment(&parameters, "SETNAME", P_WORD, setname);
    assignment(&parameters, "ELEMNAME", P_WORD, elemname);
    assignment(&parameters, "SYNTAX", P_WORD, syntax);
    if (assignment(&parameters, "RECORDS", NUMBER, (char *) &display) == 0)
        display = DISPLAY_NUMBER;
    assignment(&parameters, "", NUMBER, (char *) &position);

    info = recinfo_new();

    if (!position) position = 1;

    if (position > session->result_count)
	{
	    client_msg("-ERROR: Unable to display record %d when only %d were found.\n",
		       position,
		       session->result_count);
	    display = 0;
	}
    else if ((position + display) > session->result_count)
	display = session->result_count - position + 1;
	    
    while (display > 0)
	{    
	    info = recinfo_new();
	    
	    code = z39_present(*p_session, 
			       setname, 
			       elemname,
			       syntax,
			       position, 
			       display, 
			       info);

	    if (code == NOTOK)
		{
		    client_msg("-ERROR: Display request '%s'.\n", parameters);
		    break;
		}
	    else
		{
#ifdef MARKER
		    client_msg("-OK: %d %d\n",
			       recinfo_get_status(info),
			       recinfo_get_returned(info));
#endif
#ifndef MARKER
		    if (recinfo_get_returned(info) <= 0)
			break;

		    display -= recinfo_get_returned(info);

                    position += recinfo_get_returned(info);

		    if (!terse_mode)
			client_msg("-OK: Status %d, Received %d, Position %d, (Set: %s)\n",
				   recinfo_get_status(info),
				   recinfo_get_returned(info),
				   recinfo_get_position(info),
				   session_setname(*p_session));
#endif
		}
	    recinfo_error(*p_session, info);
	    recinfo_output(*p_session, info);
	    recinfo_dispose(&info);
	}
}

/*--------------------------------------------------------------*
 |                    CLOSE_COMMAND                             |
 | Function:                                                    |
 |    Close the connection to the Z39.50 server.                |
 | Parameters:                                                  |
 |    session    : Pointer to session information.              |
 |    parameters : Character string for select command.         |
 | Returns:                                                     |
 |    0 is successful, -1 if failed.                            |
 | Description:                                                 |
 |    Interface to call ir_close to close the Z39.50 server     |
 |    connection.                                               |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

static void close_command(SESSION **p_session,
                          char *parameters)
{
    int code;

    code = z39_close(p_session, parameters);

    if (!terse_mode)
        {
            if (code)
                client_msg("-ERROR: On closing connection\n");
            else
                client_msg("-OK: Connection was closed\n");
        };
}

/*--------------------------------------------------------------*
 |                    Z3950_CLIENT                              |
 | Function:                                                    |
 |    The Z39.50 client to send commands to Z39.50 server.      |
 | Parameters:                                                  |
 |    in   : Pointer to input file description.                 |
 |    out  : Pointer to output file description.                |
 | Returns:                                                     |
 |    0 if successful (is returned all the time).               |
 | Description:                                                 |
 |    Parse the command and branch to appropriate routine.      |
 | Note:                                                        |
 |                                                              |
 *--------------------------------------------------------------*/

int z3950_client(int terse)
{
    extern FILE *msg_log;
    SESSION *session;
    COMMAND *cmd;
    char input_buffer[MAX_BUFFER * 4];
    char token[MAX_BUFFER];
    char parameters[MAX_BUFFER * 4];
    char *input;
    int  length;
    int  prompt;

    static COMMAND commands[] =
    {
        {"MDISPLAY",  mdisplay_command}, /* Multi-Display command */
        {"DISPLAY",   display_command},  /* Display command */
        {"FIND",      find_command},     /* Find command */
        {"SELECT",    select_command},   /* Select command */
        {"CLOSE",     close_command},    /* Close command */
        {"INDEX",     index_command},    /* Index command */
        {"SHOW",      show_command},     /* Show command */
        {"DEBUG",     debug_command},    /* Debug command */
        {"ZSAVE",     zsave_command},    /* ZSave command */
        {"SCAN",      scan_command},     /* Scan command */
        {"",          NULL}              /* Null command */
    };

    session = NULL;

    terse_mode = terse;

    prompt = isatty(1);          /* Output to terminal? */

    /* Loop to accept cmds */

    do
        {
#ifndef MARKER
            if (prompt) fprintf(out, "Z39.50> ");
#endif

/*
#ifdef MARKER
            {
                int input_fd = 0;
                int io_done;  
    
                io_done = fd_wait(600, 0, 1, &input_fd);
                if (io_done <= 0) 
                    {
                        close_command(&session, parameters);
                        break;
                    }
            }
#endif
*/
            
            input = fgets(input_buffer, MAX_BUFFER * 4, in);
            if (input == NULL) continue;
            length = strlen(input) - 1;
            while (length >= 0)
                {
                     if (input_buffer[length] == NL)
                         input_buffer[length--] = EOS;
                     else
                         break;
                }

            assignment(&input, "", P_WORD, token);
            assignment(&input, "", P_PHRASE, parameters);

            if ((length = strlen(token)) <= 0) continue;
            uppercase(token, token, length);

            if (msg_log)
                {
                    fprintf(msg_log, "%s\n", input_buffer);
                    fflush(msg_log);
                }

            for (cmd = &commands[0]; *cmd->command; cmd++)
                {
                    if (strncmp(token, cmd->command, length) == 0)
                       {
                            (*cmd->routine)(&session, parameters);
                            break;
                       }
                }

            if (*cmd->command == EOS)
                client_msg("-ERROR: Invalid cmd %s.\n", token);
            else if (strncmp(token, "CLOSE", length) == 0)
                break;

#ifdef MARKER
            fprintf(out, END_MARKER);   /* End of output marker */
#else
	    if (session && !terse_mode)
                         printf("-- Time to receive pdu: %d.%d seconds\n",
		                session->wait_for_pdu / USECS_IN_SECOND,
                                session->wait_for_pdu % USECS_IN_SECOND);
#endif
            fflush(out);                /* Flush any buffers */
        }
    while (input);

#ifdef MARKER
    fprintf(out, END_MARKER);
#endif
    fflush(out);

    return 0;
}
